<?php
namespace Codexonics;

/*
 * This file is part of the Codexonics package.
 *
 * (c) Codexonics
 *
 * This package is Open Source Software. For the full copyright and license
 * information, please view the LICENSE file which was distributed with this
 * source code.
 */

use Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemAuthorization;
use Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemInitialization;
use Codexonics\PrimeMoverFramework\classes\PrimeMoverProgressHandlers;
use Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemFunctions;
use Codexonics\PrimeMoverFramework\utilities\PrimeMoverSystemUtilities;
use Codexonics\PrimeMoverFramework\utilities\PrimeMoverSystemCheckUtilities;
use Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemChecks;
use Codexonics\PrimeMoverFramework\utilities\PrimeMoverUploadUtilities;
use Codexonics\PrimeMoverFramework\classes\PrimeMoverImporter;
use Codexonics\PrimeMoverFramework\classes\PrimeMoverExporter;
use Codexonics\PrimeMoverFramework\utilities\PrimeMoverExportUtilities;
use Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemProcessors;
use Codexonics\PrimeMoverFramework\classes\PrimeMoverHookedMethods;
use Codexonics\PrimeMoverFramework\classes\PrimeMover;
use Codexonics\PrimeMoverFramework\utilities\PrimeMoverShutdownUtilities;
use Codexonics\PrimeMoverFramework\classes\PrimeMoverErrorHandlers;
use Codexonics\PrimeMoverFramework\utilities\PrimeMoverImportUtilities;
use Codexonics\PrimeMoverFramework\utilities\PrimeMoverDownloadUtilities;
use Codexonics\PrimeMoverFramework\utilities\PrimeMoverSearchReplaceUtilities;
use Codexonics\PrimeMoverFramework\utilities\PrimeMoverLockUtilities;
use Codexonics\PrimeMoverFramework\utilities\PrimeMoverOpenSSLUtilities;
use Codexonics\PrimeMoverFramework\classes\PrimeMoverValidationHandlers;
use Codexonics\PrimeMoverFramework\utilities\PrimeMoverValidationUtilities;
use Codexonics\PrimeMoverFramework\streams\PrimeMoverStreamFilters;
use Codexonics\PrimeMoverFramework\utilities\PrimeMoverConfigUtilities;
use Codexonics\PrimeMoverFramework\utilities\PrimeMoverComponentAuxiliary;
use Codexonics\PrimeMoverFramework\utilities\PrimeMoverFreemiusIntegration;
use Codexonics\PrimeMoverFramework\compatibility\PrimeMoverCompatibility;

/**
 *  Instantiate new plugin object
 */
class PrimeMoverFactory
{
    
    /**
     * Initialize hook
     */
    public function initHook()
    {
        add_action('init', array( $this, 'composeObjects' ), 0);
    }
    
    /**
     * Hooked to `init`
     */
    public function composeObjects()
    {        
        $prime_mover_user = wp_get_current_user();
        $lock_utilities = new PrimeMoverLockUtilities();

        $system_authorization = new PrimeMoverSystemAuthorization($prime_mover_user);        
        $system_initialization = new PrimeMoverSystemInitialization($system_authorization);        
        
        $openssl_utilities = new PrimeMoverOpenSSLUtilities($system_authorization, $system_initialization);
        $openssl_utilities->initHooks();
        
        $system_functions = new PrimeMoverSystemFunctions($system_authorization, $system_initialization);        
        $system_utilities = new PrimeMoverSystemUtilities($system_functions);
        $system_utilities->initHooks();  
        $shutdown_utilities = new PrimeMoverShutdownUtilities($system_functions, $system_initialization);
        
        global $pm_fs;
        $freemius_integration = new PrimeMoverFreemiusIntegration($system_authorization, $system_functions, $shutdown_utilities, $pm_fs);
        $freemius_integration->initHooks();
        
        $progress_handlers = new PrimeMoverProgressHandlers($system_authorization,$system_initialization, $shutdown_utilities);
        $progress_handlers->initHooks();
        
        $system_check_utilities = new PrimeMoverSystemCheckUtilities($system_authorization, $system_initialization, $system_functions); 
        $system_check_utilities->initHooks();
        $system_checks = new PrimeMoverSystemChecks($system_authorization, $system_initialization, $system_functions, $system_check_utilities);
        
        $upload_utilities = new PrimeMoverUploadUtilities($system_authorization, $system_initialization, $system_functions, $system_checks, $progress_handlers);            
        $upload_utilities->initHooks();
        $stream_entity = new PrimeMoverStreamFilters();
        
        $importer = new PrimeMoverImporter($system_authorization, $system_functions, $system_initialization, $system_checks, $progress_handlers);
        
        $exporter = new PrimeMoverExporter($system_authorization, $system_initialization, $system_checks, $system_functions, $progress_handlers, $stream_entity);
        $export_utilities = new PrimeMoverExportUtilities($exporter);
        $export_utilities->initHooks();            
        
        $system_processors = new PrimeMoverSystemProcessors(
            $system_authorization,
            $system_checks,
            $system_initialization,
            $system_functions,
            $importer,
            $upload_utilities,
            $export_utilities,
            $progress_handlers
        );
        
        $hooked_methods = new PrimeMoverHookedMethods($system_authorization, $system_initialization, $system_checks, $system_functions, $progress_handlers);                      
                   
        $prime_mover = new PrimeMover(
            $system_authorization,
            $hooked_methods,
            $system_processors,
            $system_initialization,
            $importer,
            $exporter,
            $system_functions,
            $system_checks
        );
        $prime_mover->primeMoverLoadHooks();        
        $error_handlers = new PrimeMoverErrorHandlers($system_functions, $system_initialization, $shutdown_utilities, $system_authorization );
        $error_handlers->initHooks();
        
        $import_utilities = new PrimeMoverImportUtilities($importer, $export_utilities, $lock_utilities);
        $import_utilities->initHooks();
        
        $config_utilities = new PrimeMoverConfigUtilities($system_authorization, $import_utilities);
        $config_utilities->initHooks();
        
        $download_utilities = new PrimeMoverDownloadUtilities($system_authorization, $system_initialization, $system_functions);
        $download_utilities->initHooks();

        $search_utilities = new PrimeMoverSearchReplaceUtilities($system_authorization, $system_initialization);
        $search_utilities->initHooks();        
        
        $component_utilities = new PrimeMoverComponentAuxiliary($exporter, $importer);
        $component_utilities->initHooks();
        
        $utilities = [
            'sys_utilities' => $system_utilities,
            'error_handlers' => $error_handlers,
            'import_utilities' => $import_utilities,
            'download_utilties' => $download_utilities,
            'lock_utilities' => $lock_utilities,
            'openssl_utilities' => $openssl_utilities,
            'config_utilities' => $config_utilities,
            'component_utilities' => $component_utilities,
            'freemius_integration' => $freemius_integration
        ];        
        
        $validation_utilities = new PrimeMoverValidationUtilities($prime_mover, $utilities);
        $validation_utilities->initHooks();
        
        $input_validator = new PrimeMoverValidationHandlers($prime_mover, $utilities, $validation_utilities);
        $input_validator->initHooks();
  
        $compatibility = new PrimeMoverCompatibility($prime_mover, $utilities);
        $compatibility->initHooks();
        
        do_action( 'prime_mover_load_module_apps', $prime_mover, $utilities);        
    }
}

/**
 * Instantiate
 * @var \PrimeMoverFramework\PrimeMoverFactory $loaded_instance
 */
$loaded_instance = new PrimeMoverFactory();
$loaded_instance->initHook();
