<?php
namespace Codexonics\PrimeMoverFramework\classes;

/*
 * This file is part of the Codexonics.PrimeMoverFramework package.
 *
 * (c) Codexonics Ltd
 *
 * This package is Open Source Software. For the full copyright and license
 * information, please view the LICENSE file which was distributed with this
 * source code.
 */

if (! defined('ABSPATH')) {
    exit;
}

/**
 * Prime Mover Main Class
 *
 * The complete migration solution for moving single-site to multisite platform and vice versa. 
 * This can also be used to migrate single-site to another single-site install or multisite subsite to 
 * another multisite subsite install.
 */
class PrimeMover
{
    private $system_authorization;
    private $hooked_methods;
    private $system_processors;
    private $system_initialization;
    private $importer;
    private $exporter;
    private $system_functions;
    private $system_checks;
    
    /**
     * Constructor
     * @param PrimeMoverSystemAuthorization $system_authorization
     * @param PrimeMoverHookedMethods $hooked_methods
     * @param PrimeMoverSystemProcessors $system_processors
     * @param PrimeMoverSystemInitialization $system_initialization
     * @param PrimeMoverImporter $importer
     * @param PrimeMoverExporter $exporter
     * @param PrimeMoverSystemFunctions $system_functions
     * @param PrimeMoverSystemChecks $system_checks
     */
    public function __construct(
        PrimeMoverSystemAuthorization $system_authorization,
        PrimeMoverHookedMethods $hooked_methods,
        PrimeMoverSystemProcessors $system_processors,
        PrimeMoverSystemInitialization $system_initialization,
        PrimeMoverImporter $importer,
        PrimeMoverExporter $exporter,
        PrimeMoverSystemFunctions $system_functions,
        PrimeMoverSystemChecks $system_checks
    ) 
    {
        $this->system_authorization = $system_authorization;
        $this->hooked_methods = $hooked_methods;
        $this->system_processors = $system_processors;
        $this->system_initialization = $system_initialization;
        $this->importer = $importer;
        $this->exporter = $exporter;
        $this->system_functions = $system_functions;
        $this->system_checks = $system_checks;
    }
    
    /**
     * Gets System authorization
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemAuthorization
     * @compatible 5.6
     */
    public function getSystemAuthorization()
    {
        return $this->system_authorization;
    }

    /**
     * Gets hooked methods
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverHookedMethods
     * @compatible 5.6
     */
    public function getHookedMethods()
    {
        return $this->hooked_methods;
    }
    
    /**
     * Gets System Processors
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemProcessors
     * @compatible 5.6
     */
    public function getSystemProcessors()
    {
        return $this->system_processors;
    }
    
    /**
     * Get System Initialization
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemInitialization
     * @compatible 5.6
     */
    public function getSystemInitialization()
    {
        return $this->system_initialization;
    }
    
    /**
     * Get Importer
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverImporter
     * @compatible 5.6
     */
    public function getImporter()
    {
        return $this->importer;
    }

    /**
     * Get exporter
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverExporter
     * @compatible 5.6
     */
    public function getExporter()
    {
        return $this->exporter;
    }
    
    /**
     * Get system functions
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemFunctions
     * @compatible 5.6
     */
    public function getSystemFunctions()
    {
        return $this->system_functions;
    }
    
    /**
     * Get System checks
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemChecks
     * @compatible 5.6
     */
    public function getSystemChecks()
    {
        return $this->system_checks;
    }
    
    /**
     * Hook in methods
     * @compatible 5.6
     * @tested PrimeMoverFramework\Tests\TestPrimeMover::testInithooks() 
     */
    public function primeMoverLoadHooks()
    {
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        $notice_hook = 'admin_notices';
        if (is_multisite()) {
            $notice_hook = 'network_admin_notices';
        }
        /** Enqueue scripts and AJAX processor*/                
        foreach ($this->getSystemInitialization()->getPrimeMoverAjaxActions() as $action_name => $action_processor) {
            add_action("wp_ajax_{$action_name}", [$this,$action_processor]);
        }
        
        /**
         * GUI hooks for export, import and delete section in Network -> Sites
         * Hooked methods class: PrimeMoverHookedMethods
         */
        add_filter('manage_sites-network_columns', [$this, 'primeMoverAddNetworkColumn'], 10, 1);
        add_action('manage_sites_custom_column', [$this, 'primeMoverExportSection'], 10, 2);
        add_action('manage_sites_custom_column', [$this, 'primeMoverImportSection'], 10, 2);
        
        add_action('admin_enqueue_scripts', [$this, 'primeMoverEnqueueScripts'], 10, 1);        
        add_action($notice_hook, [$this, 'multisiteShowNetworkAdminNotice'] );
        
        /** System Initialization hooks */
        add_action('init', [$this, 'primeMoverCreateFolder'], 1);
        add_action('admin_init', [$this, 'multisiteInitializeWpFilesystemApi']);
        add_action('admin_init', [$this, 'initializeExportDirectoryProtection'], 11);
        add_action('admin_init', [$this, 'initializeTroubleShootingLog'], 12);
        add_action('admin_init', [$this, 'initializeSiteInfoLog'], 13);
        add_action('admin_init', [$this, 'initializeExportDirIdentity'], 14);
        
        /** Initialize importer hooks */
        add_action('init', [$this, 'importerHooks']);
        
        /** Initialize exporter hooks */
        add_action('init', [$this, 'exporterHooks']);
        
        /** System hooks */
        add_action('init', [$this, 'systemHooks']);

        /** System check hooks */
        add_action('init', [$this, 'systemCheckHooks']);
        add_filter('site_option_upload_filetypes', [$this, 'addZipFileTypeSupport'], 10, 3);
        add_filter('network_admin_plugin_action_links', [$this, 'addPluginActionLinks'], 99, 4);
        
        /** Added js body-class on sites network page */
        add_filter('admin_body_class', [$this, 'addJsBodyClassOnNetworkSitesPage'], 10, 1);
        
        /** Single site support */        
        add_action('prime_mover_exporter_block', [$this, 'primeMoverExportSection']);
        add_action('prime_mover_importer_block', [$this, 'primeMoverImportSection']);
        add_action('admin_menu', [$this, 'addExportImportOptionsSingleSite']);
        
        /** Basic menu related hooks */ 
        add_action('network_admin_menu', [$this,'addMenuPage']);
        add_action('admin_menu', [$this,'addMenuPage'], 0);
        add_action('admin_enqueue_scripts', [$this, 'removeDistractionsOnSettingsPage'], 5, 1);
        
        /** Translation-ready */
        add_action('init', [$this, 'loadPluginTextdomain']);
    }

    /**
     * Load plugin text domain
     */
    public function loadPluginTextdomain()
    {
        $this->getHookedMethods()->loadPluginTextdomain();
    }
    
    /**
     * Settings page cleanup
     */
    public function removeDistractionsOnSettingsPage()
    {
        $this->getHookedMethods()->removeDistractionsOnSettingsPage();
    }
    
    /**
     * Add menu page
     */
    public function addMenuPage()
    {
        $this->getHookedMethods()->addMenuPage();
    }
    
    /**********************
     * SINGLE SITE SUPPORT*
     * ********************
     */
    
    /**
     * Add export - import option to single site
     */
    public function addExportImportOptionsSingleSite()
    {
        if ( ! $this->getSystemFunctions()->maybeLoadMenuAssets() || is_multisite() ) {
            return;
        }
        
        $this->getHookedMethods()->addExportImportOptionsSingleSite();
    }
    
    /*****************
     * HOOKED METHODS*
     * ***************
     */
    
    /**
     * Add network column
     * @param array $column_headers
     * @return array|void|string
     * @compatible 5.6
     */
    public function primeMoverAddNetworkColumn($column_headers = [])
    {
        return $this->getHookedMethods()->primeMoverAddNetworkColumn($column_headers);
    }
    
    /**
     * Add js body class
     * @param string $classes
     * @return string
     * @compatible 5.6
     */
    public function addJsBodyClassOnNetworkSitesPage($classes = '')
    {
        return $this->getHookedMethods()->addJsBodyClassOnNetworkSitesPage($classes);
    }
    
    /**
     * Export section
     * @param string $column_name
     * @param number $blog_id
     * @compatible 5.6
     */
    public function primeMoverExportSection($column_name = '', $blog_id = 0)
    {
        $this->getHookedMethods()->primeMoverExportSection($column_name, $blog_id);
    }
    
    /**
     * Import section
     * @param string $column_name
     * @param number $blog_id
     * @compatible 5.6
     */
    public function primeMoverImportSection($column_name = '', $blog_id = 0)
    {
        $this->getHookedMethods()->primeMoverImportSection($column_name, $blog_id);
    }

    /**
     * Enqueue scripts
     * @param string $hook
     * @compatible 5.6
     */
    public function primeMoverEnqueueScripts($hook = '')
    {
        $this->getHookedMethods()->primeMoverEnqueueScripts($hook);
    }

    /**
     * Show network admin notice
     * @compatible 5.6
     */
    public function multisiteShowNetworkAdminNotice()
    {
        $this->getHookedMethods()->multisiteShowNetworkAdminNotice();
    }
    
    /*********************
     * SYSTEM PROCESSORS**
     * *******************
     * @compatible 5.6
     */
    public function primeMoverExportProcessor()
    {
        $this->getSystemProcessors()->primeMoverExportProcessor();
    }
    public function primeMoverImportProcessor()
    {
        $this->getSystemProcessors()->primeMoverImportProcessor();
    }
    public function primeMoverImportProgressProcessor()
    {
        $this->getSystemProcessors()->primeMoverImportProgressProcessor();
    }
    public function primeMoverTempfileDeleteProcessor()
    {
        $this->getSystemProcessors()->primeMoverTempfileDeleteProcessor();
    }
    public function primeMoverUploadsProcessor()
    {
        $this->getSystemProcessors()->primeMoverUploadsProcessor();
    }
    public function primeMoverExportProgressProcessor()
    {
        $this->getSystemProcessors()->primeMoverExportProgressProcessor();
    }
    public function primeMoverShutdownExportProcessor()
    {
        $this->getSystemProcessors()->primeMoverShutdownExportProcessor();
    }
    public function primeMoverShutdownImportProcessor()
    {
        $this->getSystemProcessors()->primeMoverShutdownImportProcessor();
    }
    public function primeMoverVerifyEncryptedPackage()
    {
        $this->getSystemProcessors()->primeMoverVerifyEncryptedPackage();
    }
    /******************************
     * SYSTEM INITIALIZATION HOOKS*
     * ****************************
     * @compatible 5.6
     */
    public function primeMoverCreateFolder()
    {
        $this->getSystemInitialization()->primeMoverCreateFolder();
    }
    public function multisiteInitializeWpFilesystemApi()
    {
        $this->getSystemInitialization()->multisiteInitializeWpFilesystemApi();
    }
    public function initializeExportDirectoryProtection()
    {
        $this->getSystemInitialization()->initializeExportDirectoryProtection();
    }
    public function initializeTroubleShootingLog()
    {
        $this->getSystemInitialization()->initializeTroubleShootingLog();
    }
    public function initializeSiteInfoLog()
    {
        $this->getSystemInitialization()->initializeSiteInfoLog();
    }
    public function initializeExportDirIdentity()
    {
        $this->getSystemInitialization()->initializeExportDirIdentity();
    }
    /************
     * IMPORTERS*
     * **********
     * @compatible 5.6
     */
    public function importerHooks()
    {
        $this->getImporter()->importerHooks();
    }
    
    /************
     * EXPORTERS*
     * **********
     * @compatible 5.6
     */
    public function exporterHooks()
    {
        $this->getExporter()->exporterHooks();
    }
    
    /***************
     * SYSTEM HOOKS*
     * *************
     * @compatible 5.6
     */
    public function systemHooks()
    {
        $this->getSystemFunctions()->systemHooks();
    }
    
    /****************
     * SYSTEM CHECKS*
     * **************
     * @compatible 5.6
     *
     */
    public function systemCheckHooks()
    {
        $this->getSystemChecks()->systemCheckHooks();
    }
    public function addZipFileTypeSupport($value = '', $option = '', $network_id = 0)
    {
        return $this->getSystemChecks()->addZipFileTypeSupport($value, $option, $network_id);
    }
    public function addPluginActionLinks($actions = [], $plugin_file = '', $plugin_data = [], $context ='')
    {
        return $this->getSystemChecks()->addPluginActionLinks($actions, $plugin_file, $plugin_data, $context);
    }
}
