<?php
namespace Codexonics\PrimeMoverFramework\classes;

/*
 * This file is part of the Codexonics.PrimeMoverFramework package.
 *
 * (c) Codexonics Ltd
 *
 * This package is Open Source Software. For the full copyright and license
 * information, please view the LICENSE file which was distributed with this
 * source code.
 */

use Codexonics\PrimeMoverFramework\interfaces\PrimeMoverExport;
use Ifsnop\Mysqldump as IMysqldump;
use Codexonics\PrimeMoverFramework\streams\PrimeMoverStreamFilters;
use Exception;

if (! defined('ABSPATH')) {
    exit;
}

/**
 * Prime Mover Export Class
 *
 * The Prime Mover Export Class aims to provide the export facility of this plugin.
 *
 */
class PrimeMoverExporter implements PrimeMoverExport
{
    private $system_authorization;
    private $system_initialization;
    private $system_checks;
    private $system_functions;
    private $progress_handlers;
    private $stream_filters;
    private $can_export_plugins_themes = ['complete_export_mode', 'development_package'];
    
    /**
     * Constructor
     * @param PrimeMoverSystemAuthorization $system_authorization
     * @param PrimeMoverSystemInitialization $system_initialization
     * @param PrimeMoverSystemChecks $system_checks
     * @param PrimeMoverSystemFunctions $system_functions
     * @param PrimeMoverProgressHandlers $progress_handlers
     * @audited
     */
    public function __construct(
        PrimeMoverSystemAuthorization $system_authorization,
        PrimeMoverSystemInitialization $system_initialization,
        PrimeMoverSystemChecks $system_checks,
        PrimeMoverSystemFunctions $system_functions,
        PrimeMoverProgressHandlers $progress_handlers,
        PrimeMoverStreamFilters $stream_entity
    ) 
    {
        $this->system_authorization = $system_authorization;
        $this->system_initialization = $system_initialization;
        $this->system_checks = $system_checks;
        $this->system_functions = $system_functions;
        $this->progress_handlers = $progress_handlers;
        $this->stream_filters = $stream_entity;
    }

    /**
     * Get export modes that includes plugins/themes
     * @return string[]
     * @audited
     */
    public function getCanExportPluginsThemes()
    {
        return $this->can_export_plugins_themes;
    }
    
    /**
     * Get stream filters
     * @return \Codexonics\PrimeMoverFramework\streams\PrimeMoverStreamFilters
     * @audited
     */
    public function getStreamFilters()
    {
        return $this->stream_filters;
    }
    
    /**
     * Get system functions
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemFunctions
     * @compatible 5.6
     * @audited
     */
    public function getSystemFunctions()
    {
        return $this->system_functions;
    }
    
    /**
     * Get System Initialization
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemInitialization
     * @compatible 5.6
     * @audited
     */
    public function getSystemInitialization()
    {
        return $this->system_initialization;
    }
 
    /**
     * Get System checks
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemChecks
     * @compatible 5.6
     * @audited
     */
    public function getSystemChecks()
    {
        return $this->system_checks;
    }
    
    /**
     * Exporter hooks
     * @compatible 5.6
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverExporter::itAddsExporterHooks()
     * @audited
     */
    public function exporterHooks()
    {
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }        
        foreach ($this->getSystemInitialization()->getPrimeMoverExportMethods() as $export_method) {            
            add_filter("prime_mover_export_{$export_method}", [$this, $export_method], 10, 2);
        }  
        
        add_filter('prime_mover_save_return_export_progress', [$this, 'saveExportProgressData'], 10, 4);
        add_filter('prime_mover_get_export_progress', [$this, 'getExportProgressData'], 10, 2);
    }
    
    /**
     * Optionally export plugins and themes of this site
     * @param array $ret
     * @param number $blogid_to_export
     * @return void|
     * @compatible 5.6
     * @audited
     */
    public function optionallyExportPluginsThemes($ret = [], $blogid_to_export = 0)
    {        
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }       
        $ret = apply_filters('prime_mover_get_export_progress', $ret, $blogid_to_export);
        if (isset($ret['error'])) {
            return $ret;
        }
        $this->getSystemInitialization()->setSlowProcess();
        $export_mode = $ret['multisite_export_options'];
        $allowed = $this->getCanExportPluginsThemes();
        
        if ( ! in_array($export_mode, $allowed, true)) {            
            return apply_filters('prime_mover_save_return_export_progress', $ret, $blogid_to_export, 'optionallyRenameToTargetIds', 'optionallyExportPluginsThemes');
        }
        if (! isset($ret['export_system_footprint'])) {
            return apply_filters('prime_mover_save_return_export_progress', $ret, $blogid_to_export, 'optionallyRenameToTargetIds', 'optionallyExportPluginsThemes');
        }
        $this->getProgressHandlers()->updateTrackerProgress(esc_html__('Exporting plugins', 'prime-mover'), 'export' );        
        do_action('prime_mover_export_plugins', $ret);

        $this->getProgressHandlers()->updateTrackerProgress(esc_html__('Exporting themes', 'prime-mover'), 'export' );
        do_action('prime_mover_export_themes', $ret);
        
        do_action('prime_mover_log_processed_events', $ret, $blogid_to_export, 'export', 'optionallyExportPluginsThemes', $this);        
        $this->getSystemInitialization()->testRequestTerminateTimeout();

        return apply_filters('prime_mover_save_return_export_progress', $ret, $blogid_to_export, 'optionallyRenameToTargetIds', 'optionallyExportPluginsThemes');
    }
    
    /**
     * Get System Authorization
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemAuthorization
     * @compatible 5.6
     * @audited
     */
    public function getSystemAuthorization()
    {
        return $this->system_authorization;
    }
 
    /**
     * Get progress handlers
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverProgressHandlers
     * @compatible 5.6
     * @audited
     */
    public function getProgressHandlers() 
    {
        return $this->progress_handlers;
    }
    
    /**
     * Create tempfolder for this site export
     * @param number $blogid_to_export
     * {@inheritDoc}
     * @see PrimeMoverExport::createTempfolderForThisSiteExport()
     * @compatible 5.6
     * @audited
     */
    public function createTempfolderForThisSiteExport($ret = [], $blogid_to_export = 0)
    {
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        $this->getSystemInitialization()->setSlowProcess();        
        global $wp_filesystem;
        $blogid_to_export = (int) $blogid_to_export;
        if ($blogid_to_export < 1) {
            $ret['error'] = esc_html__('Blog ID is not defined', 'prime-mover');
            return $ret;
        }
        $this->getSystemInitialization()->setExportBlogID( $blogid_to_export );
        $this->getProgressHandlers()->updateTrackerProgress(esc_html__('Creating temp folder', 'prime-mover'), 'export' );

        $main_site_export_folder	= $this->getSystemInitialization()->getMultisiteExportFolderPath() . $blogid_to_export . DIRECTORY_SEPARATOR;
        if (false === $wp_filesystem->exists($main_site_export_folder)) {
            wp_mkdir_p($main_site_export_folder);
        }
        
        if (false === $wp_filesystem->exists($main_site_export_folder)) {
            $ret['error'] = esc_html__('Unable to create site export folders. Check permission.', 'prime-mover');
            return $ret;
        }
        $folder_name = $this->multisiteCreateFoldername($blogid_to_export);
        $folder_path_export = $main_site_export_folder. $folder_name;
        if (wp_mkdir_p($folder_path_export)) {
            $ret['temp_folder_path'] = $folder_path_export.DIRECTORY_SEPARATOR;
            $this->getSystemInitialization()->setTemporaryExportPackagePath($ret['temp_folder_path']);
        }
        
        if (false === $wp_filesystem->exists($folder_path_export)) {
            $ret['error'] = esc_html__('Uable to create export folder for zip files. Check permission.', 'prime-mover');
            return $ret;
        }
        
        do_action('prime_mover_log_processed_events', $ret, $blogid_to_export, 'export', 'createTempfolderForThisSiteExport', $this);       
        return apply_filters('prime_mover_save_return_export_progress', $ret, $blogid_to_export, 'dumpDbForExport', 'createTempfolderForThisSiteExport');
    }
        
    /**
     * Create unique folder name
     * @param number $blogid_to_export
     * @return string
     * {@inheritDoc}
     * @see PrimeMoverExport::multisiteCreateFoldername()
     * @compatible 5.6
     * @audited
     */
    public function multisiteCreateFoldername($blogid_to_export = 0)
    {
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        
        $blog_name = $this->getSystemFunctions()->getBlogOption($blogid_to_export, 'blogname' );
        $sanitized_site_name = mb_strimwidth( sanitize_key( $blog_name ), 0, 8, '');        
        $file_name = $sanitized_site_name . wp_generate_password( 15, false, false ) . 'blogid_' . $blogid_to_export;

        return sanitize_file_name($file_name);
    }
    
    /**
     * Handle random db prefix change
     * @param array $ret
     * @return boolean
     * @audited
     */
    private function handleRandomdBPrefix($ret = [])
    {
        $randomize_db_prefix = false;
        $export_target_id = 1;
        $is_target_set = false;
        if ( ! empty($ret['prime_mover_export_targetid']) ) {
            $export_target_id = (int)$ret['prime_mover_export_targetid'];
            $is_target_set = true;
        }
        if ($export_target_id > 0 && $is_target_set) {
            $randomize_db_prefix = true;
        }
        $this->getSystemInitialization()->setRandomizeDbPrefix($randomize_db_prefix);
        $random_string = wp_generate_password( 7, false, false );
        $updated_prefix = $random_string . '_';
        $this->getSystemInitialization()->setRandomDbPrefix($updated_prefix);
        
        $ret['mayberandomizedbprefix'] = $randomize_db_prefix;
        $ret['randomizedbprefixstring'] = strtolower($updated_prefix);
        
        return $ret;
    }
    
    /**
     * Dump database for export
     * @param number $blogid_to_export
     * @param string $tmp_folderpath
     * {@inheritDoc}
     * @see PrimeMoverExport::dumpDbForExport()
     * @compatible 5.6
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverExporter::itDumpsDbforExportWhenAllSet()
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverExporter::itDoesNotDumpWhenNotAuthorized()
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverExporter::itDoesNotDumpEncryptedWhenRequested() 
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverExporter::itDumpsEncryptedWhenCoreOnlyAndKeySet()
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverExporter::itReturnsErrorWhenExporterDetectsError()
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverExporter::itDumpsDbForDevSiteWithNoPassword() 
     * @audited
     * 
     */
    public function dumpDbForExport($ret = [], $blogid_to_export = 0)
    {
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }        
        $ret = apply_filters('prime_mover_get_export_progress', $ret, $blogid_to_export);
        
        if (isset($ret['error'])) {
            return $ret;
        }
        
        $this->getSystemInitialization()->setSlowProcess();        
        $this->getProgressHandlers()->updateTrackerProgress(esc_html__('Dumping database', 'prime-mover'), 'export' );         
        $use_mysqldump_php = true;
                
        $ret = $this->handleRandomdBPrefix($ret);
        $randomize_db_prefix = $ret['mayberandomizedbprefix'];
        $encrypt = $this->maybeEncryptDatabaseDump($ret);
        $target_path = $this->getTargetDumpPath($ret, $blogid_to_export);
        $clean_tables = $this->getTablesToExport($blogid_to_export);
        if (empty($clean_tables)) {
            $ret['error'] = esc_html__('Unable to dump database, please check that your database is not empty or these tables exists.', 'prime-mover');
            return $ret;
        }        
        $ret['enable_encryption'] = $encrypt;
        if ($encrypt) {
            $this->getSystemInitialization()->setEncryptExportData(true);
            do_action('prime_mover_encrypted_db_command_generated', $ret, $blogid_to_export);
        }
        $filter_export_data = false;
        if ($encrypt || $randomize_db_prefix) {
            $filter_export_data = true;
        }
        do_action('prime_mover_before_db_dump_export', $blogid_to_export, $ret);
        if ($use_mysqldump_php) {
            do_action('prime_mover_log_processed_events', 'Starting MySQLdump using PHP', $blogid_to_export, 'export', 'dumpDbForExport', $this);
            $this->getSystemFunctions()->temporarilyIncreaseMemoryLimits();
            $dump_result = $this->executeDumpUsingPHP($filter_export_data, $target_path, $clean_tables);
        } 
        do_action('prime_mover_after_db_dump_export', $blogid_to_export, $ret);
        if ( ! empty($dump_result['error']) ) {
            $ret['error'] = $dump_result['error'];
            return $ret;
        }        
        $ret['exported_db_tables'] = apply_filters('prime_mover_filter_tables', $clean_tables, $ret, $blogid_to_export);          
        
        do_action('prime_mover_log_processed_events', $ret, $blogid_to_export, 'export', 'dumpDbForExport', $this);
        $this->getSystemInitialization()->testRequestTerminateTimeout();

        /**
         * Export not yet done, mark as ongoing and save ongoing progress
         */        
        return apply_filters('prime_mover_save_return_export_progress', $ret, $blogid_to_export, 'copyBlogsdirFilesToTempfolder', 'dumpDbForExport');
    }

    /**
     * Checks if dump should be encrypted
     * @param array $ret
     * @return boolean
     * @audited
     */
    private function maybeEncryptDatabaseDump($ret = [])
    {
        $db_encryption_key = $this->getSystemInitialization()->getDbEncryptionKey();
        $encrypt = false;

        if ($this->maybeEncryptDbIfRequested($db_encryption_key, $ret)) {
            $encrypt = true;
        }
        return $encrypt;
    }
    
    /**
     * Get target dump path
     * @param array $ret
     * @param number $blogid_to_export
     * @return string
     * @audited
     */
    private function getTargetDumpPath($ret = [], $blogid_to_export = 0)
    {
        $enc = '';        
        $tmp_folderpath = $ret['temp_folder_path'];
        $target_path = $tmp_folderpath . $blogid_to_export . '.sql';
        
        if ($this->maybeEncryptDatabaseDump($ret)) {            
            $enc = '.enc';
            $target_path = $target_path . $enc;
        }

        return $target_path;
    }
    
    /**
     * Clean Tables to export
     * @param number $blogid_to_export
     * @return array
     * @audited
     */
    private function getTablesToExport($blogid_to_export = 0)
    {
        $this->getSystemFunctions()->switchToBlog($blogid_to_export);
        
        global $wpdb;
        $target_prefix = $wpdb->prefix;        
        
        $target_prefix = $wpdb->esc_like($target_prefix) . '%';
        $this->getSystemFunctions()->restoreCurrentBlog();
        
        $db_search = "SHOW TABLES LIKE %s";        
        $tables_to_export = $wpdb->get_results($wpdb->prepare($db_search, $target_prefix), ARRAY_N);
        
        return $this->getSystemFunctions()->cleanDbTablesForExporting($tables_to_export, $blogid_to_export, $wpdb, 'export');     
    }
    
    /**
     * Generate MySQL dump shell command
     * @param array $ret
     * @param number $blogid_to_export
     * @param array $clean_tables
     * @return string
     * @audited
     */
    private function generateMySQLDumpShellCommand($ret = [], $blogid_to_export = 0, $clean_tables = [])
    {
        $no_password = false;
        if (empty(DB_PASSWORD)) {
            $no_password = true;
        }
    
        $db_username = escapeshellarg(DB_USER);
        $db_password = escapeshellarg(DB_PASSWORD);
        $db_hostname = escapeshellarg(DB_HOST);
        $db_name = escapeshellarg(DB_NAME);

        if (empty($clean_tables)) {
            $ret['error']		= esc_html__('Unable to dump database, please check that your database is not empty or these tables exists.', 'prime-mover');
            return $ret;
        }
        $tables	= implode(" ", $clean_tables);       
        $mysqldump_path = $this->getSystemChecks()->getMySqlDumpPath();
        if (! $mysqldump_path) {
            $ret['error']		= esc_html__('Unable to get correct MySQLdump command.', 'prime-mover');
            return $ret;
        }
        $mysqldump_path = escapeshellarg($mysqldump_path);
        $password_phrase = ' ';
        if (false === $no_password) {
            $password_phrase = ' -p'. $db_password;
            $password_phrase = apply_filters('prime_mover_passwordless_dump', $password_phrase);
        }
        
        $dump_command = $mysqldump_path . ' '.
            $password_phrase .
            ' -u' . $db_username .
            ' -h'. $db_hostname . ' ' .
            $db_name . ' ' .
            $tables;
            
       return $dump_command;
    }
    
    /**
     * Encrypt dB on export if requested or applicable
     * @param string $db_encryption_key
     * @param array $ret
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverExporter::itDumpsDbforExportWhenAllSet()
     * @audited
     */
    private function maybeEncryptDbIfRequested($db_encryption_key = '', $ret = [])
    {    
        if (defined('PRIME_MOVER_GEARBOX_VERSION') && isset($ret['prime_mover_encrypt_db'])
            && 'true' === $ret['prime_mover_encrypt_db'] && $db_encryption_key) {
            return true;
        }

        return false;
    }
    
    /**
     * Execute MySQLdump in PHP
     * @param boolean $filter_export_data
     * @param string $target_path
     * @param array $clean_tables
     * @return array
     * @audited
     */
    private function executeDumpUsingPHP($filter_export_data = false, $target_path = '', $clean_tables = [])
    {
        $dump_ret = [];
        if ( ! $target_path ) {
            $dump_ret['error'] = esc_html__('Undefined execute dump in PHP parameters.', 'prime-mover');
            return $dump_ret;
        }
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            $dump_ret['error'] = esc_html__('Unauthorized dump.', 'prime-mover');
            return $dump_ret;
        }
        $register_stream = false;
        if ($filter_export_data) {
            $register_stream = $this->getStreamFilters()->register();
        }
        
        if ($filter_export_data && ! $register_stream) {
            $dump_ret['error'] = esc_html__('Unable to register encrypted stream filter.', 'prime-mover');
            return $dump_ret;
        }
        try {
            $dumpSettings = $this->getMySQLDumpPHPParameters($clean_tables);
            if ( ! defined('DB_HOST') ) {
                $dump_ret['error'] = esc_html__('ERROR: DB_HOST constant is undefined in wp-config.php.', 'prime-mover');
                return $dump_ret;
            }
            $db_host = $this->getSystemFunctions()->parsedBHostForPDO();
            if ( ! $db_host ) {
                $dump_ret['error'] = esc_html__('Unable to parse DB_HOST for PDO connection.', 'prime-mover');
                return $dump_ret;
            }
            if (empty($db_host['host'])) {
                $dump_ret['error'] = esc_html__('Error, no known DB host to connect.', 'prime-mover');
                return $dump_ret;
            }
            $hostname = $db_host['host'];
            $port = '';
            if ( ! empty($db_host['port'] ) &&  $db_host['port'] > 0) {
                $port = $db_host['port'];
            }
            $socket = '';
            if ( ! empty($db_host['socket'] )) {
                $socket = $db_host['socket'];
            }
            $mysql_server = 'mysql:host=' . $hostname . ';dbname=' . DB_NAME;
            if ($port) {
                $mysql_server = 'mysql:host=' . $hostname . ';port=' . $port . ';dbname=' . DB_NAME;
            }
            if ($socket) {
                $mysql_server = 'mysql:unix_socket=' . $socket . ';dbname=' . DB_NAME;
            }
            do_action('prime_mover_log_processed_events', $mysql_server, 0, 'export', 'executeDumpUsingPHP', $this);
            $dump = new IMysqldump\Mysqldump($mysql_server, DB_USER, DB_PASSWORD, $dumpSettings);
            
            if ($filter_export_data) {
                $encryted_streamer = $this->getStreamFilters()->getStreamFilter($target_path);
                $dump->start($encryted_streamer);
            } else {
                $dump->start($target_path);
            }
        } catch (Exception $e) {
            $dump_ret['error'] = $e->getMessage();
            return $dump_ret;
        }
        
        $dump_ret['result'] = true;
        return $dump_ret;        
    }
    
    /**
     * Execute dump in shell if supported
     * @param string $command
     * @param boolean $filter_export_data
     * @param string $target_path
     * @return array
     * @audited
     */
    private function executeDumpUsingShellFunctions($command = '', $filter_export_data = false, $target_path = '')
    {
        $dump_ret = [];
        if ( ! $command || ! $target_path ) {
            $dump_ret['error'] = esc_html__('Undefined execute dump in shell parameters.', 'prime-mover');
            return $dump_ret;
        }
        
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            $dump_ret['error'] = esc_html__('Unauthorized dump.', 'prime-mover');
            return $dump_ret;
        }
        $handle = popen("($command)","r");
        if (false === $handle) {
            $dump_ret['error'] = esc_html__('Error in MySQLdump shell using popen.', 'prime-mover');
            return $dump_ret;
        }
        $this->getSystemFunctions()->primeMoverDoDelete($target_path);
        while(!feof($handle)){
            $line = fgets($handle);
            if ($filter_export_data) {
                $line = apply_filters('prime_mover_filter_export_db_data', $line);
                $line = $line . PHP_EOL;
            }
            file_put_contents($target_path, $line, FILE_APPEND);
        }
        pclose($handle);   

        $dump_ret['result'] = true;
        return $dump_ret;
    }
    
    /**
     * Get MySQL dump - PHP parameters
     * @param array $clean_tables
     * @return array[]
     * @audited
     */
    private function getMySQLDumpPHPParameters($clean_tables = [])
    {
        /**
         * Optimal MySQL dump settings
         * References:
         * https://dba.stackexchange.com/questions/87100/what-are-the-optimal-mysqldump-settings
         * https://serversforhackers.com/c/mysqldump-with-modern-mysql
         */
        return [
            'include-tables' => $clean_tables,
            'exclude-tables' => [],
            'compress' => IMysqldump\Mysqldump::NONE,
            'init_commands' => [],
            'no-data' => [],
            'reset-auto-increment' => false,
            'add-drop-database' => false,
            'add-drop-table' => true,
            'add-drop-trigger' => true,
            'add-locks' => true,
            'complete-insert' => false,
            'databases' => false,
            'default-character-set' => IMysqldump\Mysqldump::UTF8,
            'disable-keys' => true,
            'extended-insert' => true,
            'events' => false,
            'hex-blob' => false,
            'insert-ignore' => false,
            'no-autocommit' => false,
            'no-create-info' => false,
            'lock-tables' => true,
            'routines' => false,
            'single-transaction' => true,
            'skip-triggers' => false,
            'skip-tz-utc' => false,
            'skip-comments' => false,
            'skip-dump-date' => false,
            'skip-definer' => false,
            'where' => ''
        ];
    }

    /**
     * Copy media files of this sub-site to the temporary export folder
     * @param number $blogid_to_export
     * @param string $tmp_folderpath
     * {@inheritDoc}
     * @see PrimeMoverExport::copyBlogsdirFilesToTempfolder()
     * @compatible 5.6
     * @audited
     */
    public function copyBlogsdirFilesToTempfolder($ret = [], $blogid_to_export = 0)
    {
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        $ret = apply_filters('prime_mover_get_export_progress', $ret, $blogid_to_export);
        if (isset($ret['error'])) {
            //Errors now detected, bail out and inform user
            return $ret;
        }
        
        $this->getSystemInitialization()->setSlowProcess();

        if ( ! isset($ret['multisite_export_options']) ) {
            $ret['error'] = esc_html__('Error ! Export options not set.', 'prime-mover');
            return $ret;
        }

        if ('dbonly_export' === $ret['multisite_export_options']) {
            /**
             * dB only export, save ongoing progress and move on to next step
             */            
            return apply_filters('prime_mover_save_return_export_progress', $ret, $blogid_to_export, 'generateExportFootprintConfig', 'copyBlogsdirFilesToTempfolder');
        }

        if ('development_package' === $ret['multisite_export_options']) {
            /**
             * Dev package, save ongoing progress and move on to next step
             */
            return apply_filters('prime_mover_save_return_export_progress', $ret, $blogid_to_export, 'generateExportFootprintConfig', 'copyBlogsdirFilesToTempfolder');
        }
        
        //Blog dir path of the exported blog
        $this->getProgressHandlers()->updateTrackerProgress(esc_html__('Copying media files', 'prime-mover'), 'export' );
        $blogsdir_exported_blog = $this->getSystemFunctions()->primeMoverGetBlogsDirPath($blogid_to_export);
        $tmp_folderpath = $ret['temp_folder_path'];
        
        //Blogs dir target copy path
        $blogsdir_target_copy_path = $tmp_folderpath . $blogid_to_export . DIRECTORY_SEPARATOR;
        
        //If source directory exist, copy blogs.dir to target path   
        $exportdir_slug = $this->getSystemInitialization()->getMultisiteExportFolderSlug();
        $importdir_slug = $this->getSystemInitialization()->getUploadTmpPathSlug();
        $skip = [$exportdir_slug, $importdir_slug];
        if (is_multisite()) {
            $skip = [];
        }
        
        $skip = apply_filters('prime_mover_excluded_filesfolders_export', $skip, $ret, $blogid_to_export);        
        $excluded_filetypes = apply_filters('prime_mover_excluded_filetypes_export', [], $ret, $blogid_to_export);
        global $wp_filesystem;
        if (is_dir($blogsdir_exported_blog)) {            
            $make_directory_result	= $wp_filesystem->mkdir($blogsdir_target_copy_path);
            $copy_directory_result	= $this->getSystemChecks()->getSystemCheckUtilities()->copyDir(
                $blogsdir_exported_blog, $blogsdir_target_copy_path, $skip, $excluded_filetypes);
            
            //Error handling
            if (false === $make_directory_result) {
                $ret['error']	= esc_html__('Error creating a directory insides plugin export directory.', 'prime-mover');
                return $ret;
            } elseif (is_wp_error($copy_directory_result)) {
                $ret['error']	= $copy_directory_result->get_error_message();
                return $ret;
            }
        } else {
            //Uploads does not exist
            $ret['error'] = esc_html__('Uploads directory of this blog does not seem to exist.', 'prime-mover');
            return $ret;
        }     
        
        $ret['blogsdir_target_copy_path'] = $blogsdir_target_copy_path;
        do_action('prime_mover_log_processed_events', $ret, $blogid_to_export, 'export', 'copyBlogsdirFilesToTempfolder', $this);
        
        $this->getSystemInitialization()->testRequestTerminateTimeout();
        
        /**
         * Export not yet done, mark as ongoing and save ongoing progress
         */
        return apply_filters('prime_mover_save_return_export_progress', $ret, $blogid_to_export, 'generateExportFootprintConfig', 'copyBlogsdirFilesToTempfolder');
    }
    
    /**
     *
     * Generate footprint data of activated plugins and themes of this sub-site export
     * {@inheritDoc}
     * @see PrimeMoverExport::generateExportFootprintConfig()
     * @compatible 5.6
     * @audited
     */
    public function generateExportFootprintConfig($ret = [], $blogid_to_export = 0)
    {
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        $ret = apply_filters('prime_mover_get_export_progress', $ret, $blogid_to_export);
        if (isset($ret['error'])) {
            //Errors now detected, bail out and inform user
            return $ret;
        }
        
        $this->getSystemInitialization()->setSlowProcess();
        
        //Proceed..
        $tmp_folderpath = $ret['temp_folder_path'];
        
        //Generate system footprint during this export
        $export_system_footprint = $this->getSystemChecks()->primeMoverGenerateSystemFootprint($blogid_to_export, $tmp_folderpath, $ret);
        
        //Validate system footprint data
        $footprint_analysis = $this->getSystemFunctions()->primeMoverValidateFootprintData($export_system_footprint);
        $valid_system_footprint = $footprint_analysis['overall_valid'];        
        $this->getProgressHandlers()->updateTrackerProgress(esc_html__('Generating config', 'prime-mover'), 'export' );
        if (true === $valid_system_footprint) {
            //Add exported tables to footprint
            $export_system_footprint['exported_db_tables'] = $ret['exported_db_tables'];
            
            //Put valid system footprint to $ret for easier accessing of this data
            $ret['export_system_footprint'] = $export_system_footprint;
            
            //Write footprint to export directory
            $construct_message = json_encode(apply_filters('prime_mover_filter_export_footprint', $export_system_footprint, $ret, $blogid_to_export));
            
            //Log and write to export directory
            global $wp_filesystem;
            $put_results = $wp_filesystem->put_contents($tmp_folderpath . 'footprint.json', $construct_message);
            $blog_id_otherfiles = $blogid_to_export;
            
            if ( ! empty($ret['prime_mover_export_targetid']) ) {
                $blog_id_otherfiles = $ret['prime_mover_export_targetid'];
            }
            do_action( 'prime_mover_write_otherfiles', $tmp_folderpath, $blog_id_otherfiles, $export_system_footprint );
            
            if (false === $put_results) {
                $ret['error'] = esc_html__('Error putting footprint files.', 'prime-mover');
                return $ret;
            }
        } else {
            $ret['error'] = $footprint_analysis['errors'];
            return $ret;
        }
        
        do_action('prime_mover_log_processed_events', $ret, $blogid_to_export, 'export', 'generateExportFootprintConfig', $this);
        
        /**
         * Export not yet done, mark as ongoing and save ongoing progress
         */       
        return apply_filters('prime_mover_save_return_export_progress', $ret, $blogid_to_export, 'optionallyExportPluginsThemes', 'generateExportFootprintConfig');
    }
  
    /**
     * Optional rename package resources to target ID
     * @param array $ret
     * @param number $blogid_to_export
     * @return array
     * @audited
     */
    public function optionallyRenameToTargetIds($ret = [], $blogid_to_export = 0)
    {
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        $ret = apply_filters('prime_mover_get_export_progress', $ret, $blogid_to_export);
        if (isset($ret['error'])) {
            //Errors now detected, bail out and inform user
            return $ret;
        }
        
        if ( empty($ret['prime_mover_export_targetid']) ) {
            /**
             * We don't need this method if we dont customize target ID, save ongoing process
             * and move on to next step
             */              
            return apply_filters('prime_mover_save_return_export_progress', $ret, $blogid_to_export, 'zippedFolder', 'optionallyRenameToTargetIds');
        }
        
        global $wp_filesystem;
        clearstatcache();
        
        $tmp_folderpath = $ret['temp_folder_path'];
        $export_target_id = $ret['prime_mover_export_targetid'];
        
        /**
         * SQL file renaming
         */
        $sql_source_raw = $tmp_folderpath . $blogid_to_export . '.sql';
        $sql_destination_raw = $tmp_folderpath . $export_target_id . '.sql';
        
        $sql_source_encrypted = $tmp_folderpath . $blogid_to_export . '.sql.enc';
        $sql_destination_encrypted = $tmp_folderpath . $export_target_id . '.sql.enc';
        
        if ($wp_filesystem->exists($sql_source_raw)) {
            $wp_filesystem->move($sql_source_raw, $sql_destination_raw);
        }
  
        if ($wp_filesystem->exists($sql_source_encrypted)) {
            $wp_filesystem->move($sql_source_encrypted, $sql_destination_encrypted);
        }
        
        /**
         * Media folder renaming
         */
        $media_source = $tmp_folderpath . $blogid_to_export . DIRECTORY_SEPARATOR;
        $media_destination = $tmp_folderpath . $export_target_id . DIRECTORY_SEPARATOR;
        
        if ($wp_filesystem->exists($media_source)) {
            $wp_filesystem->move($media_source, $media_destination);
        }
        
        /**
         * Main temp folder renaming
         */
        $original_temp_source = basename($tmp_folderpath);
        $dir_temp_source = dirname($tmp_folderpath);
        $newfoldername = str_replace("_$blogid_to_export", "_$export_target_id", $original_temp_source);
        $target_temp_destination = $dir_temp_source . DIRECTORY_SEPARATOR . $newfoldername . DIRECTORY_SEPARATOR;
        
        if ($wp_filesystem->exists($tmp_folderpath)) {
            $wp_filesystem->move($tmp_folderpath, $target_temp_destination);
        }
        
        $ret['temp_folder_path'] = $target_temp_destination;
        $ret['blogsdir_target_copy_path'] = $target_temp_destination . $export_target_id . DIRECTORY_SEPARATOR;

        /**
         * Export not yet done, mark as ongoing and save ongoing progress
         */
        return apply_filters('prime_mover_save_return_export_progress', $ret, $blogid_to_export, 'zippedFolder', 'optionallyRenameToTargetIds');
    }
    
    /**
     * Zipped export directory
     * {@inheritDoc}
     * @see PrimeMoverExport::zippedFolder()
     * @compatible 5.6
     * @audited
     */
    public function zippedFolder($ret = [], $blogid_to_export = 0)
    {
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        $ret = apply_filters('prime_mover_get_export_progress', $ret, $blogid_to_export);
        if (isset($ret['error'])) {
            //Errors now detected, bail out and inform user
            return $ret;
        }
        
        if ( ! empty($ret['blogsdir_target_copy_path']) ) {
            global $wp_filesystem;
            $media_zip_target = $ret['temp_folder_path'] . 'media.zip';
            $blogsdir_target_copy_path = $ret['blogsdir_target_copy_path'];
            
            $enable_media_encryption = apply_filters('prime_mover_enable_media_encryption', false, $ret);
            $ret['encrypted_media'] = $enable_media_encryption;
            $this->getSystemChecks()->primeMoverZipDirectory($blogsdir_target_copy_path, $media_zip_target, $enable_media_encryption);
            
            $wp_filesystem->rmdir($blogsdir_target_copy_path, true);
            do_action('prime_mover_before_zipping_package', $enable_media_encryption, $blogid_to_export, $ret);
        }
        
        $this->getSystemInitialization()->setSlowProcess();
        
        //Let's zipped the temporary export directory
        //Define the source
        $tmp_folderpath = $ret['temp_folder_path'];
        $target_export_zip_basename = basename($tmp_folderpath);
        $target_export_file_name = $target_export_zip_basename.'.zip';
        $target_zip_path = $this->getSystemInitialization()->getMultisiteExportFolderPath() . $blogid_to_export . DIRECTORY_SEPARATOR . $target_export_file_name;
        $this->getSystemInitialization()->setExportZipPath( $target_zip_path );
        $this->getProgressHandlers()->updateTrackerProgress(esc_html__('Zipping folders', 'prime-mover'), 'export' );
        
        $zip_success =	$this->getSystemChecks()->primeMoverZipDirectory($tmp_folderpath, $target_zip_path);
        if ( ! isset($zip_success['close']) || ! isset($zip_success['non_conforming'] ) ) {
            $ret['error'] = esc_html__('Zip failed! The returned result is undefined.', 'prime-mover');
            return $ret;
        }
        $close = $zip_success['close'];
        $non_conforming = $zip_success['non_conforming'];
        if (true === $close) {
            //Successfully zipped
            $ret['target_zip_path']	=	$target_zip_path;
        } elseif ( ! empty($non_conforming) ) {
            $ret['error'] =	$ret['error']	= esc_html__('Zip failed! Some file paths exceed Windows MAX_PATH limit.', 'prime-mover');
            return $ret;
        } else {
            //Not successfully zipped
            $ret['error'] =	$ret['error']	= esc_html__('Zip failed! Please check if you have zip extension enabled.', 'prime-mover');
            return $ret;
        }        
        do_action('prime_mover_log_processed_events', $ret, $blogid_to_export, 'export', 'zippedFolder', $this);

        return apply_filters('prime_mover_save_return_export_progress', $ret, $blogid_to_export, 'deleteTemporaryFolder', 'zippedFolder');
    }
    
    /**
     * Delete temporary export folder
     * {@inheritDoc}
     * @see PrimeMoverExport::deleteTemporaryFolder()
     * @compatible 5.6
     * @audited
     */
    public function deleteTemporaryFolder($ret = [], $blogid_to_export = 0)
    {
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        $ret = apply_filters('prime_mover_get_export_progress', $ret, $blogid_to_export);
        if (isset($ret['error'])) {
            //Errors now detected, bail out and inform user
            return $ret;
        }
        $this->getSystemInitialization()->setSlowProcess();
        
        global $wp_filesystem;        
        $tmp_folderpath = $ret['temp_folder_path'];
        $this->getProgressHandlers()->updateTrackerProgress(esc_html__('Deleting temp folder', 'prime-mover'), 'export' );
        
        $delete_result = $wp_filesystem->rmdir($tmp_folderpath, true);
        if (false === $delete_result) {
            $ret['error'] = esc_html__('Unable to delete temporary directory', 'prime-mover');
            return $ret;
        }
        do_action('prime_mover_log_processed_events', $ret, $blogid_to_export, 'export', 'deleteTemporaryFolder', $this);        

        /**
         * Export not yet done, mark as ongoing and save ongoing progress
         */
        return apply_filters('prime_mover_save_return_export_progress', $ret, $blogid_to_export, 'generateDownloadUrl', 'deleteTemporaryFolder');
    }
    
    /**
     * Generate download URL
     * {@inheritDoc}
     * @see PrimeMoverExport::generateDownloadUrl()
     * @compatible 5.6
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverExporter::itGeneratesDownloadURLWhenAllSet()
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverExporter::itDoesNotGenerateDownloadURLNotAuthorized()
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverExporter::itGeneratesDownloadURLWhenExportDirectoryOn()
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverExporter::itGeneratesDownloadUrlWhenExportDirectoryOff()
     * @audited
     */
    public function generateDownloadUrl($ret = [], $blogid_to_export = 0)
    {
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        $ret = apply_filters('prime_mover_get_export_progress', $ret, $blogid_to_export);
        if (isset($ret['error'])) {
            //Errors now detected, bail out and inform user
            return $ret;
        }
        $this->getSystemInitialization()->setSlowProcess();
        $export_directory_on = false;
        $this->getProgressHandlers()->updateTrackerProgress(esc_html__('Generate download URL', 'prime-mover'), 'export' );
        $results = $ret['target_zip_path'];

        $download_nonce = wp_create_nonce('prime_mover_download_package_' . $blogid_to_export);
        $export_directory_on = $this->isExportDirectoryOn($ret);
        $hash = $this->getSystemFunctions()->sha256File($results);
       
        $args = [
            'prime_mover_download_nonce' => $download_nonce,
            'prime_mover_export_hash' => $hash,
            'prime_mover_blogid' => $blogid_to_export
        ];
        
        if ($export_directory_on) {
            unset($args['prime_mover_download_nonce']);    
        }
        
        $download_url = $this->getSystemInitialization()->getDownloadURLGivenParameters($args);
        $ret['download_url']	= $download_url;
        
        $results = str_replace($this->getSystemInitialization()->getMultisiteExportFolderPath(), '', $results);
        update_option($hash, $results);
        do_action('prime_mover_after_generating_download_url', $results, $hash, $blogid_to_export, $export_directory_on, $ret);
        do_action('prime_mover_log_processed_events', $ret, $blogid_to_export, 'export', 'generateDownloadUrl', $this);

        /**
         * Export not yet done, mark as ongoing and save ongoing progress
         */        
        return apply_filters('prime_mover_save_return_export_progress', $ret, $blogid_to_export, 'doPostExportProcessing', 'generateDownloadUrl');
    }
    
    /**
     * Check if export directory mode is on.
     * @param array $ret
     * @return boolean
     * @audited
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverExporter::itDoPostExportProcessingWhenUsingCustomExportDir()
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverExporter::itDoPostExportProcessingWhenUsingDefaultDir(
     */
    private function isExportDirectoryOn($ret = [])
    {
        if (isset($ret['multisite_export_location']) && 'export_directory' === $ret['multisite_export_location'] ) {
            return true;
        } 
        return false;
    }
    
    /**
     * Post export processing filter
     * @param array $ret
     * @audited
     * @param number $blogid_to_export
     * @return mixed|NULL|array
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverExporter::itDoPostExportProcessingWhenUsingCustomExportDir()
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverExporter::itDoPostExportProcessingWhenUsingDefaultDir()
     */
    public function doPostExportProcessing($ret = [], $blogid_to_export = 0)
    {
        $ret = apply_filters('prime_mover_get_export_progress', $ret, $blogid_to_export);
        $export_directory_on = $this->isExportDirectoryOn($ret);
        $this->getSystemInitialization()->setSlowProcess();
        
        $ret = apply_filters('prime_mover_post_export_processing', $ret, $blogid_to_export, $export_directory_on );
        do_action('prime_mover_log_processed_events', $ret, $blogid_to_export, 'export', 'doPostExportProcessing', $this);
        
        /**
         * Last step return without saving anymore
         * And clean up the progress
         */
        
        return $this->cleanUpProgressForFinalReturn($ret);
    }
    
    /**
     * Save and return export progress data
     * @audited
     * @param array $ret
     * @param number $blogid_to_export
     * @param string $next_method
     * @param string $current_method
     * @return array
     */
    public function saveExportProgressData($ret = [], $blogid_to_export = 0, $next_method = '', $current_method = '')
    {
        if ( ! $this->getSystemAuthorization()->isUserAuthorized() ) {
            return $ret;
        }
        $user_id = get_current_user_id();
        if ( ! $user_id ) {
            return $ret;
        }      
        if ( ! empty($ret['error']) ) {
            return $ret;
        }
        if ( ! $blogid_to_export || ! $next_method || ! $current_method ) {
            return $ret;
        }
        $ret['ongoing_export'] = true;
        $ret['next_method'] = $next_method;
        $ret['current_method'] = $current_method;

        $meta_key = $this->getProgressHandlers()->generateTrackerId($blogid_to_export, 'export');
        wp_cache_delete($user_id, 'user_meta' );
        update_user_meta($user_id, $meta_key, $ret);
        
        return $ret;
    }
    
    /**
     * Return export progress data to continue processing
     * @param array $ret
     * @param number $blogid_to_export
     * @audited
     * @return array
     */
    public function getExportProgressData($ret = [], $blogid_to_export = 0)
    {
        if ( ! $this->getSystemAuthorization()->isUserAuthorized() || ! $blogid_to_export) {
            return $ret;
        }
        $user_id = get_current_user_id();
        $meta_key = $this->getProgressHandlers()->generateTrackerId($blogid_to_export, 'export');
        wp_cache_delete($user_id, 'user_meta' );
        
        return get_user_meta($user_id, $meta_key, true);        
    }
    
    /**
     * Clean up export progress
     * @audited
     * @param array $ret
     * @return array
     */
    protected function cleanUpProgressForFinalReturn($ret = [])
    {        
        if (isset($ret['ongoing_export'])) {
            unset($ret['ongoing_export']);
        }
        if (isset($ret['next_method'])) {
            unset($ret['next_method']);
        }
        if (isset($ret['current_method'])) {
            unset($ret['current_method']);
        }
        return $ret;
    }
}
