<?php
namespace Codexonics\PrimeMoverFramework\classes;

/*
 * This file is part of the Codexonics.PrimeMoverFramework package.
 *
 * (c) Codexonics Ltd
 *
 * This package is Open Source Software. For the full copyright and license
 * information, please view the LICENSE file which was distributed with this
 * source code.
 */

use Codexonics\PrimeMoverFramework\utilities\PrimeMoverShutdownUtilities;

if (! defined('ABSPATH')) {
    exit;
}

/**
 * The Prime Mover Progress handling class
 *
 * The Prime Mover Progress Handler Class handles tracking the import and export progress for user tracking migration progress.
 *
 */
class PrimeMoverProgressHandlers
{
    private $system_authorization;
    private $system_initialization;
    private $shutdown_utilities;
    private $active_processes;
    
    const SINGLESITE_PLUGINS = 'prime_mover_standalone_plugins';
    const PRIMEMOVER_MAINTENANCE = 'prime_mover_maintenance';
    
    /**
     *
     * Constructor
     */
    public function __construct(
        PrimeMoverSystemAuthorization $system_authorization,
        PrimeMoverSystemInitialization $system_initialization,
        PrimeMoverShutdownUtilities $shutdown_utilities
        ) 
    {
            $this->system_authorization = $system_authorization;
            $this->system_initialization = $system_initialization;
            $this->shutdown_utilities = $shutdown_utilities;
            $this->active_processes = [];
    }
    
    /**
     * Get system functions
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemFunctions
     * @compatible 5.6
     */
    public function getSystemFunctions()
    {
        return $this->getShutDownUtilities()->getSystemFunctions();
    }
    
    /**
     * Get active process ID
     * @param string $process_id
     * @return number|mixed
     */
    public function getActiveProcessBlogId($process_id = '')
    {
        $active_processes = $this->active_processes;
        $blog_id = 0;
        if ( ! empty($active_processes[$process_id] ) ) {
            $blog_id = $active_processes[$process_id];
        }
        return $blog_id;
    }
    
    /**
     * Set active processes
     * @param string $process_id
     * @param number $blog_id
     */
    public function setActiveProcesses($process_id = '', $blog_id = 0)
    {
        if (array_key_exists($process_id, $this->active_processes)) {
            return;
        }
        $this->active_processes[$process_id] = $blog_id;
    }
    
    /**
     * Initialize hooks
     * @compatibility 5.6
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverProgressHandlers::itAddsInitHooks()
     */
    public function initHooks() 
    {
        /**
         * On shutdown progress hooks
         */
        add_action('prime_mover_shutdown', [$this,'primeMoverDeleteImportId'], 10, 2);
        add_action('prime_mover_shutdown', [$this,'primeMoverDeleteExportId'], 10, 2);        
        add_action('prime_mover_shutdown', [$this,'primeMoverDeleteExportResultId'], 10, 2);
        add_action('prime_mover_shutdown', [$this,'primeMoverDeleteImportResultId'], 10, 2);        
        add_action('prime_mover_shutdown', [$this,'primeMoverDeleteDownloadResultId'], 10, 2);        
        add_action('prime_mover_shutdown', [$this,'primeMoverDeleteDownloadSizeId'], 10, 2);
        add_action('prime_mover_shutdown', [$this,'primeMoverDeleteDownloadTmpFileId'], 10, 2);
        add_action('prime_mover_shutdown', [$this,'primeMoverDeleteUploadDropBoxSize'], 10, 2);
        add_action('prime_mover_shutdown', [$this,'primeMoverDeleteMaintenanceOption'], 10, 2);
        add_action('prime_mover_shutdown', [$this,'primeMoverDeleteAssemblyOption'], 10, 2);
        add_action('prime_mover_shutdown', [$this,'primeMoverDeleteAssembledOption'], 10, 2);
        add_action('prime_mover_shutdown', [$this,'primeMoverDeleteUploadProgressOption'], 10, 2);
        add_action('prime_mover_shutdown', [$this,'primeMoverDeleteUserProgressMeta'], 10, 2);
        
        /**
         * On boot-up progress hooks
         */
        add_action('prime_mover_bootup', [$this,'primeMoverDeleteRunTimeErrorLog'], 0, 2);        
        add_action('prime_mover_bootup', [$this,'primeMoverDeleteImportId'], 10, 2);
        add_action('prime_mover_bootup', [$this,'primeMoverDeleteExportId'], 10, 2);
        add_action('prime_mover_bootup', [$this,'primeMoverDeleteExportResultId'], 10, 2);
        add_action('prime_mover_bootup', [$this,'primeMoverDeleteImportResultId'], 10, 2);
        add_action('prime_mover_bootup', [$this,'primeMoverDeleteDownloadResultId'], 10, 2);        
        add_action('prime_mover_bootup', [$this,'primeMoverDeleteDownloadSizeId'], 10, 2);
        add_action('prime_mover_bootup', [$this,'primeMoverDeleteDownloadTmpFileId'], 10, 2);
        add_action('prime_mover_bootup', [$this,'primeMoverDeleteUploadDropBoxSize'], 10, 2);
        add_action('prime_mover_bootup', [$this,'primeMoverDeleteMaintenanceOption'], 10, 2);
        add_action('prime_mover_bootup', [$this,'primeMoverDeleteAssemblyOption'], 10, 2);
        add_action('prime_mover_bootup', [$this,'primeMoverDeleteAssembledOption'], 10, 2);
        add_action('prime_mover_bootup', [$this,'primeMoverDeleteUploadProgressOption'], 10, 2);        
        add_action('prime_mover_bootup', [$this,'primeMoverDeleteUserProgressMeta'], 10, 2);
        
        /**
         * Must be after all boot-up actions
         */
        add_action('prime_mover_bootup', [$this,'primeMoverBooted'], 999, 3);
        
        /**
         * Other hooks
         */
        add_filter('prime_mover_ajax_rendered_js_object', [$this, 'setProgressShutDownNonces'], 10, 1 );        
        add_filter('prime_mover_ajax_rendered_js_object', [$this, 'setProgressError'], 35, 1 );        
        add_action('prime_mover_doing_package_download', [$this, 'setTempFilePath'], 10, 1);
        
        add_action('prime_mover_before_doing_import', [$this,'primeMoverDeactivateAllPlugins'], 10, 2);
        add_action('prime_mover_before_doing_import', [$this,'putPublicSiteToMaintenance'], 15, 1);
        
        add_action('prime_mover_do_things_tmp_deleted', [$this,'primeMoverReactivateAllPlugins'], 10, 1);
        add_action('prime_mover_after_renamedb_prefix', [$this,'putPublicSiteToMaintenance'], 10, 1);
        add_action('prime_mover_shutdown_actions', [$this, 'removePublicMaintenanceOnRunTimeErrors']);
        
        add_action('prime_mover_before_db_dump_export', [$this,'putPublicSiteToMaintenance'], 10, 1);
        add_action('prime_mover_after_db_dump_export', [$this,'deleteMaintenanceOption'], 10, 1);
        
        $this->getSystemFunctions()->switchToBlog(get_current_blog_id());
        $enabled = get_option(self::PRIMEMOVER_MAINTENANCE);

        $this->getSystemFunctions()->restoreCurrentBlog();
        
        if ($enabled) {
            add_action('get_header', [$this, 'publicMaintenanceDuringRestore']);
            add_action('admin_init', [$this, 'publicMaintenanceDuringRestore']);
        }        
        
        add_action('init', [ $this, 'maintenanceModeControl' ], 35);
    }
    
    /**
     * Get parameters
     * @return array
     */
    private function getParameters()
    {
        $args = [
            'prime_mover_maintenance' => FILTER_SANITIZE_STRING,
            'prime_mover_blogid' => FILTER_SANITIZE_NUMBER_INT,
            'prime_mover_disable_wpnonce' => FILTER_SANITIZE_STRING
        ];
        return $this->getSystemInitialization()->getUserInput('get', $args, 'maintenance_control_parameters', '', 0, false);  
    }
    
    /**
     * Maintenance mode control
     */
    public function maintenanceModeControl()
    {
        $params = $this->getParameters();
        if (empty($params['prime_mover_maintenance']) || empty($params['prime_mover_blogid']) || empty($params['prime_mover_disable_wpnonce'])) {
            return;
        }
        $nonce = $params['prime_mover_disable_wpnonce'];
        $maintenance = $params['prime_mover_maintenance'];
        $blog_id = (int)$params['prime_mover_blogid'];
        
        if ( ! $maintenance || ! $blog_id ) {
            return;
        }
        if ( ! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        if ( ! wp_verify_nonce( $nonce, 'prime-mover-disable-maintenance' ) ) {
            return;
        } 
        if ('off' === $maintenance) {
            $this->deleteMaintenanceOption($blog_id);
        }
        if ('on' === $maintenance) {
            $this->putPublicSiteToMaintenance($blog_id);
        }
    }
    
    /**
     * Checks status of public maintenance
     * @return boolean
     */
    public function isNowPublicMaintenance($blog_id = 0)
    {
        if ( ! $blog_id && is_multisite()) {
            return false;
        }
        wp_cache_delete('alloptions', 'options');
        $enabled = $this->getSystemFunctions()->getBlogOption($blog_id, self::PRIMEMOVER_MAINTENANCE);
        if ($enabled) {
            return true;
        }
        return false;
    }
    
    /**
     * Delete maintenance option
     * @param number $process_id
     * @param number $blog_id
     */
    public function primeMoverDeleteMaintenanceOption($process_id = 0, $blog_id = 0)
    {
        $this->deleteMaintenanceOption($blog_id); 
    }
    
    /**
     * Delete assembly option on boot
     * @param number $process_id
     * @param number $blog_id
     */
    public function primeMoverDeleteAssemblyOption($process_id = 0, $blog_id = 0)
    {
        $assembly_option = '_assembly_' . $process_id;
        delete_site_option($assembly_option);
    }
    
    /**
     * Delete assembled option on boot
     * @param number $process_id
     * @param number $blog_id
     */
    public function primeMoverDeleteAssembledOption($process_id = 0, $blog_id = 0)
    {
        $assembled_option = '_assembled_' . $process_id;
        delete_site_option($assembled_option);
    }
    
    /**
     * Delete upload progress option on boot
     * @param number $process_id
     * @param number $blog_id
     */
    public function primeMoverDeleteUploadProgressOption($process_id = 0, $blog_id = 0)
    {
        $uploadtracker_option = '_upload_progress_' . $process_id;
        delete_site_option($uploadtracker_option);
    }
    
    /**
     * Remove public maintenance mode on runtime errors
     */
    public function removePublicMaintenanceOnRunTimeErrors()
    {
        $blog_id = $this->getShutDownUtilities()->primeMoverGetProcessedID();
        $this->deleteMaintenanceOption($blog_id);        
    }
    
    /**
     * Delete maintenance option
     * @param number $blog_id
     */
    public function deleteMaintenanceOption($blog_id = 0)
    {
        $this->getSystemFunctions()->switchToBlog($blog_id);
        delete_option(self::PRIMEMOVER_MAINTENANCE);
        $this->getSystemFunctions()->restoreCurrentBlog();
    }
    
    /**
     * When doing migrations or site restoration.
     * Make sure the public page of the site is showing its under maintenance
     * @param number $blog_id
     */
    public function putPublicSiteToMaintenance($blog_id = 0)
    {
        if ( ! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        $this->getSystemFunctions()->switchToBlog($blog_id);
        update_option(self::PRIMEMOVER_MAINTENANCE, 'yes');
        $this->getSystemFunctions()->restoreCurrentBlog();
    }
    
    /**
     * Reactivate all plugins when user rejects diff
     */
    public function primeMoverReactivateAllPlugins($delete_tmpfile_post = [])
    {
        if (is_multisite() || empty($delete_tmpfile_post['diff_reject'])) {
            return;
        }
        $diff_reject = $delete_tmpfile_post['diff_reject'];
        if ('false' === $diff_reject) {
            return;
        }
        if ( ! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        $singlesite_plugins = get_option(self::SINGLESITE_PLUGINS);
        activate_plugins($singlesite_plugins, '', false, true);
        delete_option(self::SINGLESITE_PLUGINS);
    }
    
    /**
     * Deactivate all plugins before import
     * @param number $blogid_to_import
     * @param boolean $import_initiated
     */
    public function primeMoverDeactivateAllPlugins($blogid_to_import = 0, $import_initiated = false)
    {
        if (is_multisite()) {
            return;
        }
        if ( ! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        /**
         * This must be a single site
         * We want to deactivate plugins only if import is not yet initiated
         */
        if ($import_initiated) {
            return;
        }

        $activated_plugins = $this->getSystemFunctions()->getActivatedPlugins();
        $corecomponents = $this->getSystemInitialization()->getCoreComponents();
        
        $thirdparty_plugins = array_diff($activated_plugins, $corecomponents);        
        update_option(self::SINGLESITE_PLUGINS, $thirdparty_plugins);
        
        if ( empty($thirdparty_plugins) ) {
            return;
        }
        deactivate_plugins($thirdparty_plugins, true, false);        
    }
    
    /**
     * Delete runtime error log on fresh boot
     * @param string $process_id
     * @param number $blog_id
     */
    public function primeMoverDeleteRunTimeErrorLog($process_id = '', $blog_id = 0)
    {
        $this->getShutDownUtilities()->primeMoverDeleteErrorLog($blog_id, false);
    }
    
    /**
     * Get shutdown utilities
     * @return \Codexonics\PrimeMoverFramework\utilities\PrimeMoverShutdownUtilities
     */
    public function getShutDownUtilities()
    {
        return $this->shutdown_utilities;
    }
    
    /**
     * Catch fatal runtime errors on progress handlers
     * @param number $blog_id
     * @return array|boolean[]|string[]
     */
    public function getFatalRunTimeErrors($blog_id = 0)
    {        
        $response = [];
        if ( ! $blog_id ) {
            return $response;
        }
        $error_log = $this->getSystemInitialization()->getErrorLogFile($blog_id);
        if ( $this->getShutDownUtilities()->primeMoverErrorLogExist( false, $blog_id, $error_log) ) {
            $errorlog_url = $this->getShutDownUtilities()->getDownloadErrorLogURL( $blog_id );
            
            $response['logexist'] = true;
            $response['error_msg'] = esc_html__('Runtime error : ', 'prime-mover' ) . '  ' . '<a href="' . esc_url($errorlog_url) . '">' .
                esc_html__( 'View error', 'prime-mover') . '</a>';
        }
        return $response;
    }
    
    /**
     * Mark that the Boot process has started
     * @param string $process_id
     * @param number $blog_id
     * @param boolean $diffmode
     * @param string $mode
     */
    public function primeMoverBooted($process_id = '', $blog_id = 0, $diffmode = false, $mode = 'import')
    {
        if ( ! $process_id || ! $mode ) {
            return;
        }
        $this->updateTrackerProgress('boot', $mode, $process_id);
    }
    
    /**
     * Setup progress error
     * @param array $args
     * @return array
     */
    public function setProgressError( array $args )
    {
        $args['prime_mover_progress_error_message'] = esc_js(
            sprintf(__('Progress reporting fails for blog ID {{BLOGID}}. Retry is attempted but still fails. %s',
                'prime-mover'),
                '<strong>' . __('Server Error : {{PROGRESSSERVERERROR}}', 'prime-mover') . '</strong>'));
                return $args;
    }
    
    /**
     * Get System Initialization
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemInitialization
     * @compatibility 5.6
     */
    public function getSystemInitialization()
    {
        return $this->system_initialization;
    }
    
    /**
     * Get System authorization
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemAuthorization
     * @compatibility 5.6
     */
    public function getSystemAuthorization()
    {
        return $this->system_authorization;
    }

    /**
     * Set Shutdown nonces
     * @compatible 5.6
     */
    public function setProgressShutDownNonces( array $args )
    {
        $args['prime_mover_import_shutdown_nonce'] = wp_create_nonce('prime_mover_import_shutdown_nonce');
        $args['prime_mover_export_shutdown_nonce'] = wp_create_nonce('prime_mover_export_shutdown_nonce');
        
        return $args;
    }
    
    /**
     * Common shutdown processor
     * @param string $mode
     * @compatibility 5.6
     */
    public function commonShutDownProcessor($mode = 'import')
    {
        //Initialize response
        $response = [];        
        $shutdown_nonce = 'prime_mover_' . $mode . '_shutdown_nonce';
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            wp_send_json( $response );
        }
        
        //Initialize args
        $args = [
            $shutdown_nonce => FILTER_SANITIZE_STRING,
            'process_id' => FILTER_SANITIZE_STRING,
            'blog_id' => FILTER_SANITIZE_NUMBER_INT
        ];
        
        //Validate shutdown ajax
        $shutdown_post = $this->getSystemInitialization()->getUserInput('post', $args, 'common_shutdown_processor', $mode, 0, true);
        if ( ! isset($shutdown_post[$shutdown_nonce]) || ! isset($shutdown_post[ 'process_id' ]) ||  ! isset($shutdown_post[ 'blog_id' ])) {
            wp_send_json( $response );
        }
        if ( ! wp_verify_nonce($shutdown_post[$shutdown_nonce], $shutdown_nonce)) {
            wp_send_json( $response );
        }
        
        //Get process id
        $process_id = $shutdown_post[ 'process_id' ];
        if ( ! $process_id) {
            wp_send_json( $response );
        }

        //Get blog_id
        $blog_id = $shutdown_post[ 'blog_id' ];
        if ( ! $blog_id) {
            wp_send_json( $response );
        }
  
        $this->getSystemFunctions()->switchToBlog($blog_id);
        delete_option(self::PRIMEMOVER_MAINTENANCE);
        $this->getSystemFunctions()->restoreCurrentBlog();

        do_action('prime_mover_log_processed_events', "Shutdown request fully validated, ready to shutdown $mode process", $blog_id, $mode, 'commonShutDownProcessor', $this);        
        do_action('prime_mover_shutdown', $process_id, $blog_id);
        $response['status'] = 'shutdown_processed';
        
        do_action('prime_mover_log_processed_events', "Shutdown request completed, response:", $blog_id, $mode, 'commonShutDownProcessor', $this);
        do_action('prime_mover_log_processed_events', $response, $blog_id, $mode, 'commonShutDownProcessor', $this);
        
        wp_send_json( $response );
    }
    
    /**
     * On public facing pages, return 503 maintenance mode at the official start of restoration.
     */
    public function publicMaintenanceDuringRestore()
    {
        if (wp_doing_ajax()) {
            return;
        }
        $blog_id = 1;
        if (is_multisite()) {
            $blog_id = get_current_blog_id();
        }
        $nonce = wp_create_nonce( 'prime-mover-disable-maintenance' );
        $arr_params = ['prime_mover_maintenance' => 'off', 'prime_mover_blogid' => $blog_id, 'prime_mover_disable_wpnonce' => $nonce];
        $url = esc_url(add_query_arg($arr_params));
        
        $message = '';
        
        if ($this->isNowPublicMaintenance($blog_id)) {
            $message .= '<p>' . esc_html__('Site is currently undergoing maintenance work. Please come back later.', 'prime-mover') . '</p>';
            if ($this->getSystemAuthorization()->isUserAuthorized()) {
                $message .= '<p><a href="' . $url . '">' .
                    esc_html__('Administrators : Click this link if you want to DISABLE MAINTENANCE MODE NOW', 'prime-mover') .
                    '</a>.</p>';
            }
        } else {            
            $message .= '<p>' . esc_html__('Prime Mover Maintenance Mode is now disabled. PLEASE RELOAD THIS PAGE.', 'prime-mover') . '</p>';
        }
        
        wp_die(
            $message, 
            esc_html__('Maintenance Mode', 'prime-mover'),             
            ['response' => 503]
        );        
    }
    
    /**
     * Common progress processor
     * @param string $mode
     * @compatibility 5.6
     */
    public function commonProgressProcessor($mode = 'import')
    {
        //Initialize response
       
        
        $response = [];
        $status_string = $mode . '_status';
        $output_string = $mode . '_result';
        $download_string = 'download_result';
        
        $progress_nonce = 'prime_mover_' . $mode . '_progress_nonce';
        
        $response[$status_string] = '';
        $response[$output_string] = '';
        $response[$download_string] = '';
        
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return wp_send_json( $response );
        }
        
        //Initialize args
        $args = [
            $progress_nonce => FILTER_SANITIZE_STRING,
            'blog_id' => FILTER_SANITIZE_NUMBER_INT,
            'action' => FILTER_SANITIZE_STRING,
            'trackercount' => FILTER_SANITIZE_NUMBER_INT,
            'diffmode' => FILTER_VALIDATE_BOOLEAN
        ];
        
        //Validate progress ajax
        $progress_post = $this->getSystemInitialization()->getUserInput('post', $args, 'common_progress_processor', $mode, 0, true);
        
        if ( ! isset($progress_post[$progress_nonce]) || ! isset($progress_post[ 'blog_id' ]) ) {
            return wp_send_json( $response );
        }
        if ( ! wp_verify_nonce($progress_post[$progress_nonce], $progress_nonce)) {
            return wp_send_json( $response );
        }
        
        //Get blog ID processed
        $blog_id = $progress_post[ 'blog_id' ];
        if ( ! $blog_id) {
            return wp_send_json( $response );
        }
        
        //Get user process ID
        $process_id = $this->generateTrackerId($blog_id, $mode);
        
        if ( ! empty($progress_post['trackercount']) && isset($progress_post['diffmode']) ) {
            $trackercount = (int) $progress_post['trackercount']; 
            $diffmode = (bool)$progress_post['diffmode'];
            if (1 === $trackercount) {                
                do_action('prime_mover_bootup', $process_id, $blog_id, $diffmode, $mode);
                do_action('prime_mover_log_processed_events', "Tracker count: " . $trackercount . ' BOOT-UP EVENT -DIFF MODE: ' . $diffmode,  $blog_id, $mode, 'commonProgressProcessor', $this);
                
                $response['bootup'] = true;
                return wp_send_json( $response );                
            }
            do_action('prime_mover_log_processed_events', "Tracker count: " . $trackercount,  $blog_id, $mode, 'commonProgressProcessor', $this);
        }        
        
        //Catch fatal runtime errors
        $runtime_errors = $this->getFatalRunTimeErrors($blog_id);
        if ( ! empty($runtime_errors) ) {
            return wp_send_json( $runtime_errors );
        }
        
        $latest = $this->getTrackerProgressNonCached($blog_id, $mode);
        do_action('prime_mover_log_processed_events', "Getting latest progress: $latest with process id: $process_id", $blog_id, $mode, 'commonProgressProcessor', $this);
        if ($latest) {
            $response[$status_string] = $latest;            
        }
        $standard_processes = ['diffdetected', 'stoptracking'];
        if (in_array($latest, $standard_processes)) {
            $response[$output_string] = $this->getSystemFunctions()->getSiteOption($mode . '_' . $process_id);
            do_action('prime_mover_log_processed_events', "Standard process detected: $latest", $blog_id, $mode, 'commonProgressProcessor', $this);             
            
            if ('stoptracking' === $latest) {                
                do_action('prime_mover_log_processed_events', $response, $blog_id, $mode, 'commonProgressProcessor', $this);                
            }            
        }
        if ('package_downloaded' === $latest) {            
            
            $download_process_id = 'download_' . $process_id;
            $response[$download_string] = $this->getSystemFunctions()->getSiteOption($download_process_id);
            
            do_action('prime_mover_log_processed_events', "Package download event detected: $download_process_id", $blog_id, $mode, 'commonProgressProcessor', $this);           
            do_action('prime_mover_log_processed_events', $response, $blog_id, $mode, 'commonProgressProcessor', $this);
        }       
        
        $response = apply_filters('prime_mover_filter_progress_response', $response, $latest, $process_id, $blog_id, $mode);
        
        //Update client of the status
        wp_send_json( $response );
    }
    
    /**
     * Get tracker progress non cached
     * @param number $blog_id
     * @param string $mode
     * @return string|mixed|boolean|NULL|array
     */
    public function getTrackerProgressNonCached($blog_id = 0, $mode = 'import')
    {
        if ( ! $blog_id || ! $mode ) {
            return '';
        }
        
        $process_id = $this->generateTrackerId($blog_id, $mode);  
        $network_id = get_current_network_id();
        
        $notoptions_key = "$network_id:notoptions";
        wp_cache_delete( $notoptions_key, 'site-options' );
        
        $progress = $this->getSystemFunctions()->getSiteOption($process_id, '', false);
        
        do_action('prime_mover_log_processed_events', "Tracker progress non-cached request", $blog_id, $mode, 'commonProgressProcessor', $this);
        do_action('prime_mover_log_processed_events', "getSiteOption($process_id) = $progress", $blog_id, $mode, 'commonProgressProcessor', $this);
        
        return $progress;
    }
    
    /**
     * Generate tracker ID for monitoring the progress of a specific import/export process.
     * @param int $blog_id
     * @param string $mode
     * @return number|string
     * @compatibility 5.6
     */
    public function generateTrackerId( $blog_id = 0, $mode = 'import')
    {
        $process_id = '';
        if ( ! $blog_id ) {
            return $process_id; 
        }        
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return $process_id;
        }
        $user_ip = $this->getSystemInitialization()->getUserIp();
        if (! $user_ip) {
            return $process_id;
        }
        $user_id = get_current_user_id();
        $browser = $_SERVER['HTTP_USER_AGENT'];
        
        $string = $browser . $user_ip . $user_id . $blog_id . $mode;        
        
        return hash('sha256', $string);
    }
    
    /**
     * Progress trackers initialize
     * @param int $blog_id
     * @param string $mode
     * @compatibility 5.6
     */
    public function initializeProgressTracker( $blog_id = 0, $mode = 'import')
    {
        if (! $this->getSystemAuthorization()->isUserAuthorized() || ! $blog_id) {
            return;
        }
        
        do_action('prime_mover_log_processed_events', "Initializing progress tracker", $blog_id, $mode, 'initializeProgressTracker', $this); 
        
        //Generate import id for this import
        $process_id = $this->generateTrackerId($blog_id, $mode);
        
        $this->setActiveProcesses($process_id, $blog_id);
        
        //Initialize the import ID property
        if ('import' === $mode) {
            $this->getSystemInitialization()->setImportId($process_id);
        } 
        
        if ('export' === $mode) {
            $this->getSystemInitialization()->setExporttId($process_id);
        } 
    }
    
    /**
     * Save tracker progress
     * @param string $progress
     * @param string $mode
     * @param string $process_id
     */
    public function updateTrackerProgress($progress = '', $mode = 'import', $process_id = '')
    {        
        if ( ! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        if ('import' === $mode ) {
            $import_id = $this->getSystemInitialization()->getImportId();
            if ($process_id) {
                $import_id = $process_id;
            }            
            if ($import_id) {
                add_action( "update_site_option_{$import_id}", [$this, 'dontCache'], 10, 1);
                $this->getSystemFunctions()->updateSiteOption($import_id , $progress);
                $blog_id = $this->getActiveProcessBlogId($import_id);
                do_action('prime_mover_log_processed_events', "Update tracker progress: updateSiteOption($import_id, $progress);", $blog_id, 'import', 'updateTrackerProgress', $this);
            }
        }
        if ('export' === $mode ) {
            $export_id = $this->getSystemInitialization()->getExportId();
            if ($process_id) {
                $export_id = $process_id;
            }           
            if ($export_id) {
                add_action( "update_site_option_{$export_id }", [$this, 'dontCache'], 10, 1);
                $this->getSystemFunctions()->updateSiteOption($export_id, $progress);
                $blog_id = $this->getActiveProcessBlogId($export_id);
                do_action('prime_mover_log_processed_events', "Update tracker progress: updateSiteOption($export_id, $progress);", $blog_id, 'export', 'updateTrackerProgress', $this);
            }
        }  
    }
 
    /**
     * Dont cache tracker progress for reporting accuracy
     * @param string $option
     */
    public function dontCache($option = '')
    {
        $network_id = get_current_network_id();
        $cache_key = "$network_id:$option";
        
        wp_cache_delete( $cache_key, 'site-options');
    }

    /**
     * Delete download result ID on PHP shutdown
     * @param number $import_id
     * @param number $blog_id
     */
    public function primeMoverDeleteDownloadResultId($import_id = 0, $blog_id = 0)
    {
        $option = 'download_' . $import_id;
        delete_site_option($option);    
        
        do_action('prime_mover_log_processed_events', "Deleted download result ID: $option", $blog_id, 'import', 'primeMoverDeleteDownloadResultId', $this);        
    }
    
    /**
     * Delete export result ID on PHP shutdown
     * @param number $export_id
     * @param number $blog_id
     */
    public function primeMoverDeleteExportResultId($export_id = 0, $blog_id = 0)
    {
        $export_result_id = 'export_' . $export_id;
        
        delete_site_option($export_result_id); 
        do_action('prime_mover_log_processed_events', "Deleted export result ID: $export_id", $blog_id, 'export', 'primeMoverDeleteExportResultId', $this);
    }
    
    /**
     * Delete import result ID on PHP shutdown
     * @param number $import_id
     * @param number $blog_id
     */
    public function primeMoverDeleteImportResultId($import_id = 0, $blog_id = 0)
    {
        $import_result_id = 'import_' . $import_id;
        delete_site_option($import_result_id);
        do_action('prime_mover_log_processed_events', "Deleted import result ID:  $import_result_id", $blog_id, 'import', 'primeMoverDeleteImportResultId', $this);
    }
    
    /**
     * Delete import ID just before PHP shuts down execution.
     * @compatibility 5.6
     * @param number $import_id
     * @param number $blog_id
     */
    public function primeMoverDeleteImportId($import_id = 0, $blog_id = 0)
    {
        delete_site_option($import_id);
        do_action('prime_mover_log_processed_events', "Deleted import ID:  $import_id", $blog_id, 'import', 'primeMoverDeleteImportId', $this);
    }
    
    /**
     * 
     * Delete export ID just before PHP shuts down execution.
     * @compatibility 5.6
     * @param number $export_id
     * @param number $blog_id
     */
    public function primeMoverDeleteExportId($export_id = 0, $blog_id = 0)
    {        
        delete_site_option($export_id);
        do_action('prime_mover_log_processed_events', "Deleted export ID:  $export_id", $blog_id, 'export', 'primeMoverDeleteExportId', $this);
    }
    
    /**
     * Delete download size ID
     * @param number $import_id
     * @param number $blog_id
     */
    public function primeMoverDeleteDownloadSizeId($import_id = 0, $blog_id = 0)
    {
        $option = 'download_size_' . $import_id;
        delete_site_option($option);
        
        do_action('prime_mover_log_processed_events', "Deleted download size ID: $option", $blog_id, 'import', 'primeMoverDeleteDownloadSizeId', $this);         
    }
    
    /**
     * Delete dropbox upload size ID
     * @param number $export_id
     * @param number $blog_id
     */
    public function primeMoverDeleteUploadDropBoxSize($export_id = 0, $blog_id = 0)
    {
        $option = 'dropboxupload_size_' . $export_id;
        delete_site_option($option);
        
        do_action('prime_mover_log_processed_events', "Deleted dropbox upload size ID: $option", $blog_id, 'export', 'primeMoverDeleteUploadDropBoxSize', $this);
    }
 
    /**
     * Delete download tmp file ID
     * @param number $import_id
     * @param number $blog_id
     */
    public function primeMoverDeleteDownloadTmpFileId($import_id = 0, $blog_id = 0)
    {
        $option = 'download_tmp_path_' . $import_id;
        delete_site_option($option);
        
        do_action('prime_mover_log_processed_events', "Deleted download tmp file ID: $option", $blog_id, 'import', 'primeMoverDeleteDownloadTmpFileId', $this);
    }
    
    /**
     * Set downloaded package
     * @param string $process_id
     * @param number $content_length
     */
    public function setDownloadedPackageSize($process_id = '', $content_length = 0)
    {
        if ( ! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        if ( ! $process_id || ! $content_length ) {
            return;
        }
        
        $key = 'download_size_' . $process_id;
        add_action( "update_site_option_{$key}", [$this, 'dontCache'], 10, 1);
        
        $this->getSystemFunctions()->updateSiteOption($key, $content_length);
    }
    
    /**
     * Set upload size for Dropbox package
     * @param string $process_id
     * @param number $content_length
     * @param number $fileSize
     */
    public function setUploadSizeDropBox($process_id = '', $content_length = 0, $fileSize = 0) 
    {
        if ( ! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        if ( ! $process_id || ! $content_length || ! $fileSize) {
            return;
        }
        
        $key = 'dropboxupload_size_' . $process_id;
        add_action( "update_site_option_{$key}", [$this, 'dontCache'], 10, 1);
        $value = ['current_upload_size' => $content_length, 'total_upload_size' => $fileSize];
        
        $this->getSystemFunctions()->updateSiteOption($key, $value);
    }
    
    /**
     * Set temp file path
     * @param array $r
     */
    public function setTempFilePath($r = [])
    {
        if ( ! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        if (empty($r['filename'])) {
            return;
        }
        
        $import_blog_id = $this->getSystemInitialization()->getImportBlogID();
        if ( ! $import_blog_id ) {
            return;
        }
        
        $process_id = $this->generateTrackerId($import_blog_id, 'import');        
        $key = 'download_tmp_path_' . $process_id;
        
        add_action( "update_site_option_{$key}", [$this, 'dontCache'], 10, 1);        
        $this->getSystemFunctions()->updateSiteOption($key, $r['filename']);         
    }
    
    /**
     * Delete user progress meta
     * @param number $process_id
     * @param number $blog_id
     */
    public function primeMoverDeleteUserProgressMeta($process_id = 0, $blog_id = 0)
    {
        $user_id = get_current_user_id();
        if ( ! $user_id ) {
            return;
        }
        delete_user_meta($user_id, $process_id);
    }
}