<?php
namespace Codexonics\PrimeMoverFramework\classes;

/*
 * This file is part of the Codexonics.PrimeMoverFramework package.
 *
 * (c) Codexonics Ltd
 *
 * This package is Open Source Software. For the full copyright and license
 * information, please view the LICENSE file which was distributed with this
 * source code.
 */

use Codexonics\PrimeMoverFramework\interfaces\PrimeMoverSystemCheck;
use Codexonics\PrimeMoverFramework\utilities\PrimeMoverSystemCheckUtilities;
use RecursiveDirectoryIterator;
use RecursiveIteratorIterator;

if (! defined('ABSPATH')) {
    exit;
}

/**
 * The Prime Mover System Checks Class
 *
 * The Prime Mover System Check Class aims to provide the system checks for the facility of this plugin.
 *
 */
class PrimeMoverSystemChecks implements PrimeMoverSystemCheck
{
    private $system_authorization;
    private $system_initialization;
    private $system_functions;
    private $system_check_utilities;
    
    /**
     * Constructor
     * @param PrimeMoverSystemAuthorization $system_authorization
     * @param PrimeMoverSystemInitialization $system_initialization
     * @param PrimeMoverSystemFunctions $system_functions
     * @param PrimeMoverSystemCheckUtilities $system_check_utilities
     */
    public function __construct(
        PrimeMoverSystemAuthorization $system_authorization,
        PrimeMoverSystemInitialization $system_initialization,
        PrimeMoverSystemFunctions $system_functions,
        PrimeMoverSystemCheckUtilities $system_check_utilities
    ) 
    {
        $this->system_authorization = $system_authorization;
        $this->system_initialization = $system_initialization;
        $this->system_functions = $system_functions;
        $this->system_check_utilities = $system_check_utilities;
    }
    
    /**
     * @compatible 5.6
     * @return \Codexonics\PrimeMoverFramework\utilities\PrimeMoverSystemCheckUtilities
     */
    public function getSystemCheckUtilities() 
    {
        return $this->system_check_utilities;
    }
    
    /**
     * Get system functions
     * @compatible 5.6
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemFunctions
     */
    public function getSystemFunctions()
    {
        return $this->system_functions;
    }
    
    /**
     * Get System Initialization
     * @compatible 5.6
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemInitialization
     */
    public function getSystemInitialization()
    {
        return $this->system_initialization;
    }
    
    /**
     * Get System authorization
     * @compatible 5.6
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemAuthorization
     */
    public function getSystemAuthorization()
    {
        return $this->system_authorization;
    }
    
    /**
     * System hooks
     * @compatible 5.6
     * @tested PrimeMoverFramework\Tests\TestMigrationSystemChecks::itAddsInitHooks() 
     */
    public function systemCheckHooks()
    {
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        add_filter('prime_mover_filter_site_footprint', [$this, 'addSchemeToSystemFootprint'], 10, 2);
        add_filter('prime_mover_filter_site_footprint', [$this, 'addUploadInfoToSystemFootprint'], 10, 2);
        
        add_filter('prime_mover_can_decrypt_media', [$this, 'canDecryptMedia'], 10, 3);
    }

    /**
     * Add upload info to system footprint
     * @param array $footprint
     * @param int $blogid_to_export
     * @compatible 5.6
     */
    public function addUploadInfoToSystemFootprint(array $footprint, $blogid_to_export = 0)
    {
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            $footprint;
        }
        if (! $blogid_to_export || empty($footprint)) {
            return $footprint;
        }
        $upload_information_path = $this->getSystemFunctions()->primeMoverGetBlogsDirPath($blogid_to_export);
        $upload_information_url = $this->getSystemFunctions()->primeMoverGetBlogsDirUrl($blogid_to_export);
        
        if (! $upload_information_path || ! $upload_information_url) {
            return $footprint;
        }
        
        $upload_information_path = rtrim($upload_information_path, DIRECTORY_SEPARATOR);
        $footprint['upload_information_path'] = $upload_information_path;
        $footprint['upload_information_url'] = $upload_information_url;
        
        $legacy_base_url = $this->getSystemCheckUtilities()->getLegacyBaseURL($blogid_to_export);
        if ( $this->getSystemCheckUtilities()->isLegacyMultisiteBaseURL($blogid_to_export) && $legacy_base_url ) {
            $footprint['legacy_upload_information_url'] = $legacy_base_url;
        }
        
        return $footprint;
    }
    
    /**
     * Add site scheme to system footprint
     * @param array $footprint
     * @param int $blogid_to_export
     * @compatible 5.6
     */
    public function addSchemeToSystemFootprint(array $footprint, $blogid_to_export = 0)
    {
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            $footprint;
        }
        if (! $blogid_to_export || empty($footprint)) {
            return $footprint;
        }
        if (is_ssl()) {
            $footprint['scheme'] = PRIME_MOVER_SECURE_PROTOCOL;
        } else {
            $footprint['scheme'] = PRIME_MOVER_NON_SECURE_PROTOCOL;
        }
        return $footprint;
    }
    /**
     * Check if essential plugin requirements are meet.
     * @return boolean
     * {@inheritDoc}
     * @see PrimeMoverSystemCheck::primeMoverEssentialRequisites()
     * @compatible 5.6
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverHookedMethods::itReturnsTrueWhenAllRequirementsMeet()
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverHookedMethods::itReturnsFalseWhenFolderNotCreated() 
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverHookedMethods::itReturnsFalseWhenFileSystemNotInitialized()
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverHookedMethods::itReturnsFalseIfZipDisabled()
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverHookedMethods::itReturnsFalseWheNotComplyingPHPMinimumRequirement() 
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverHookedMethods::itReturnsFalseWhenItDoesNotHaveSystem()
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverHookedMethods::itReturnsFalseWhenItDoesNotHaveFileInfoExtension() 
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverHookedMethods::itReturnsFalseWhenItDoesNotHaveOpenSSLSupport()
     */
    public function primeMoverEssentialRequisites()
    {
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        $all_requisites_meet	= false;
        /**
         * Export folder check
         */
        if ((true === $this->getSystemInitialization()->getMultisiteExportFolderCreated()) &&
            /**
             * WP filesystem check
             */
            (true === $this->getSystemInitialization()->getMultisiteWpFilesystemInitialized()) &&
            /**
             * Check if zip enabled
             */
            (true === $this->getSystemFunctions()->primeMoverCheckIfZipEnabled()) &&
            /**
             * Check if meeting minimum PHP version requirements
             */
            (true === $this->getSystemFunctions()->compliedMinimumRequirement()) &&
            /**
             * Check if mbstring enabled
             */
            (true === $this->primeMoverCheckIfMbstringEnabled()) &&
            /**
             * Check if fileinfo extension loaded
             */
            (true === $this->primeMoverCheckIfFileinfoExtensionEnabled())
            ) {
            $all_requisites_meet	= true;
        }
        return $all_requisites_meet;
    }
    
    /**
     * Check if fileinfo extension is enabled
     * @return boolean
     * @compatible 5.6
     */
    public function primeMoverCheckIfFileinfoExtensionEnabled()
    {
        $extension_loaded = false;
        if (extension_loaded('fileinfo')) {
            $extension_loaded = true;
        }
        
        return $extension_loaded;
    }
    
    /**
     * Check if mbstring extension is running
     * Publicly overridable no auth check
     * @return boolean
     * @compatible 5.6
     */
    public function primeMoverCheckIfMbstringEnabled()
    {
        $enabled = false;
        if (extension_loaded('mbstring')) {
            $enabled = true;
        }
        return $enabled;
    }
    
    /**
     * @compatible 5.6
     * Generate system footprint for checking system compatibility when doing export/import
     * {@inheritDoc}
     * @see PrimeMoverSystemCheck::primeMoverGenerateSystemFootprint()
     */
    public function primeMoverGenerateSystemFootprint($blogid_to_export = 0, $tmp_folderpath = '', $ret = [])
    {
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        //Initialize
        $activated_plugins_list = [];
        $footprint = [];
        
        /**
         * PLUGINS
         */
        $activated_plugins_list = $this->getSystemFunctions()->getActivatedPlugins($blogid_to_export, $activated_plugins_list);        
        global $wp_filesystem;
        
        //Append plugin versions to footprint array
        foreach ($activated_plugins_list as $plugin_to_check) {
            $plugin_full_path = PRIME_MOVER_PLUGIN_CORE_PATH . $plugin_to_check;
            
            if (! $wp_filesystem->exists($plugin_full_path)) {
                continue;
            }
            
            $plugin_meta_data = get_plugin_data($plugin_full_path);
            if ((is_array($plugin_meta_data)) && (isset($plugin_meta_data['Version']))) {
                if (!empty($plugin_meta_data['Version'])) {
                    $footprint['plugins'][	$plugin_to_check ]	= $plugin_meta_data['Version'];
                }
            }
        }
        
        /**
         * THEMES
         */
        //Get theme information on this sub-site
        $stylesheet = $this->getSystemFunctions()->getBlogOption($blogid_to_export, 'stylesheet');
        $theme_template = $this->getSystemFunctions()->getBlogOption($blogid_to_export, 'template');
        $stylesheet_info = wp_get_theme($stylesheet, PRIME_MOVER_THEME_CORE_PATH);
        $theme_template_info = wp_get_theme($theme_template, PRIME_MOVER_THEME_CORE_PATH);
        
        //Checks if a child theme is in use
        $using_child_theme = $this->getSystemFunctions()->isUsingChildTheme($blogid_to_export);
        
        
        //Append theme versions to footprint array
        $footprint['stylesheet'][ $stylesheet ] = $stylesheet_info->get('Version');
        $footprint['template'][ $theme_template ] =	$theme_template_info->get('Version');
        $footprint['using_child_theme'] = $using_child_theme;
        
        /**
         * SITEURL
         */
        $site_url =	$this->getSystemFunctions()->getBlogOption($blogid_to_export, 'siteurl');
        $url_parsed = parse_url($site_url);
        $site_url_parsed = '';
        if ((isset($url_parsed['host'])) && (!empty($url_parsed['host']))) {
            $site_url_parsed .= $url_parsed['host'];
        }
        if ((isset($url_parsed['path'])) && (!empty($url_parsed['path']))) {
            $site_url_parsed .= $url_parsed['path'];
        }
        if (!empty($site_url)) {
            $footprint['site_url'] = $site_url_parsed;
        }
        
        /**
         * WORDPRESS ROOT DIRECTORY
         */
        if (defined('ABSPATH')) {
            $footprint['wp_root'] = get_home_path();
        }
        
        /**
         * DB PREFIX
         */
        $db_prefix = $this->getSystemFunctions()->getDbPrefixOfSite($blogid_to_export);
        if ($db_prefix) {
            $footprint['db_prefix'] = $this->getSystemFunctions()->getDbPrefixOfSite($blogid_to_export);
        }
        
        /** Allow apps to filter this when needed */
        $footprint = apply_filters('prime_mover_filter_site_footprint', $footprint, $blogid_to_export, $ret);
        return $footprint;
    } 
    
    /**
     * Check for files having Windows MAX_PATH limit
     * @param RecursiveIteratorIterator $files
     * @return array|number[]
     */
    protected function validWindowsPaths(RecursiveIteratorIterator $files)
    {
        $non_conforming = [];
        if ( ! $this->isWindows() ) {
            return $non_conforming;
        }
        foreach ($files as $file => $fileinfo) {
            $file = realpath($file);
            $file = wp_normalize_path($file);  
            if (in_array(basename($file), ['.svn', '.', '..'])) {
                continue;
            }
            if (false !== strpos($file, '/.svn')) {
                continue;
            }
            if (false !== strpos($file, '/.git')) {
                continue;
            }
            if ( is_dir($file) ) {
                continue;
            }
            $length = mb_strlen($file);
            if ($length > 255 ) {
                $non_conforming[] = $length;
            }
        }
        
        return $non_conforming;
    }
    
    /**
     * Zip directory
     * @compatible 5.6
     * {@inheritDoc}
     * @see PrimeMoverSystemCheck::primeMoverZipDirectory()
     */
    public function primeMoverZipDirectory($source, $destination, $encrypt = false)
    {
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        if ((false === $this->primeMoverEssentialRequisites()) || !file_exists($source)
            || !is_writeable(dirname($destination))) {
                return ['close' => false, 'non_conforming' => []];
            }
            $source = realpath($source);           
            $source = wp_normalize_path($source);
            $destination =  wp_normalize_path($destination);
            
            $zip = new \ZipArchive();
            
            if (!(file_exists($destination))) {
                $zip->open($destination, \ZipArchive::CREATE);
            } else {
                if ($zip->open($destination, \ZIPARCHIVE::CREATE) !== true) {
                    return ['close' => false, 'non_conforming' => []];
                }
            }
            if ($encrypt) {
                $zip = $this->getSystemInitialization()->setEncryptionPassword($zip);
            }            
            if (true === is_dir($source)) {
                $zip->addEmptyDir(basename($source)); 
                if ($encrypt) {
                    $zip->setEncryptionName(basename($source), \ZipArchive::EM_AES_256);
                }
                $files = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($source), RecursiveIteratorIterator::SELF_FIRST);
                $non_conforming = $this->validWindowsPaths($files);
                foreach ($files as $file => $object) {                   
                    if (in_array(basename($file), ['.svn', '.git', '.', '..'])) {
                        continue;
                    }
                    $file = realpath($file);
                    $file = wp_normalize_path($file); 
                    if ( false !== strpos($file, '/.svn')) {
                        continue;
                    }
                    if (false !== strpos($file, '/.git')) {
                        continue;
                    }                    
                    if (true === is_dir($file)) {
                        $dirx = basename($source) . '/' . str_replace(
                            $source . '/',
                            '',
                            $file . '/'
                            );
                        $zip->addEmptyDir($dirx); 
                        if ($encrypt) {
                            $zip->setEncryptionName($dirx, \ZipArchive::EM_AES_256);
                        }
                    } elseif (true === is_file($file)) {
                        $filez = basename($source) . '/' . str_replace($source . '/', '', $file);
                        $zip->addFile($file, $filez);
                        if ($encrypt) {
                            $zip->setEncryptionName($filez , \ZipArchive::EM_AES_256);
                        }
                    }
                }
            } elseif (true === is_file($source)) {
                $zip->addFile($file);
                if ($encrypt) {
                    $zip->setEncryptionName($file, \ZipArchive::EM_AES_256);
                }                
            }
            
            $close = $zip->close();            
            return ['close' => $close, 'non_conforming' => $non_conforming];
    }
    
    /**
     * Add zip file type support by default
     * @param string $value
     * @param string $option
     * @param number $network_id
     * @return string
     * @compatible 5.6
     * @tested PrimeMoverFramework\Tests\TestMigrationSystemChecks::itAddsZipFileTypeSupportWhenSet()
     * @tested PrimeMoverFramework\Tests\TestMigrationSystemChecks::itAddsZipFileTypeSupportWhenNotSet()
     * @tested PrimeMoverFramework\Tests\TestMigrationSystemChecks::itDoesNotAnymoreAddsZipIfAlreadySupported()
     */
    public function addZipFileTypeSupport($value = '', $option = '', $network_id = 0)
    {
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        if (! $network_id || ! $option) {
            return $value;
        }
        if ('upload_filetypes' === $option) {
            $supported_filetypes = explode(" ", $value);
            $supported_filetypes = array_filter($supported_filetypes);
            if (! in_array('zip', $supported_filetypes)) {
                $supported_filetypes[] = 'zip';
                $value = implode(" ", $supported_filetypes);
            }
        }
        return $value;
    }
    
    /**
     * Add plugin action links
     * @param array $actions
     * @param string $plugin_file
     * @param array $plugin_data
     * @param string $context
     * @return array
     * @compatible 5.6
     * @tested PrimeMoverFramework\Tests\TestMigrationSystemChecks::itAddsPluginActionLinksWhenMatched() 
     * @tested PrimeMoverFramework\Tests\TestMigrationSystemChecks::itDoesNotAddpluginactionlinksWhenNotMatched() 
     */
    public function addPluginActionLinks($actions = [], $plugin_file = '', $plugin_data = [], $context ='')
    {
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        if (! $plugin_file || ! isset($plugin_data['PluginURI'])) {
            return $actions;
        }
        $pluginfile = basename($plugin_file);
        $pluginuri	= $plugin_data['PluginURI'];
        if ($this->getSystemInitialization()->getPluginUri() === $pluginuri && $this->getSystemInitialization()->getPluginFile() === $pluginfile) {
            $actions[] = '<a class="edit" href="' . esc_url_raw(network_admin_url('sites.php')) . '">' . esc_html__('Migrate/Backup Sites', 'prime-mover') . '</a>';
        }
        return $actions;
    }
    
    /**
     * Analyze the most correct MySQL dump path based on the system this plugin is running.
     * Inspired by https://wordpress.org/plugins/duplicator/
     * @return boolean|string|
     * @compatible 5.6
     * @tested PrimeMoverFramework\Tests\TestMigrationSystemChecks::itGetsMySQLDumpPath() 
     * @tested PrimeMoverFramework\Tests\TestMigrationSystemChecks::itReturnsFalseIfNoSystemForMySQLDump() 
     * @tested PrimeMoverFramework\Tests\TestMigrationSystemChecks::itReturnsNullIfNotAuthorizedToDump()
     * @tested PrimeMoverFramework\Tests\TestMigrationSystemChecks::itReturnsFalseWhenNoMySQLDumpHandler() 
     * @tested PrimeMoverFramework\Tests\TestMigrationSystemChecks::itGetsMySQLDumpPathInWindows()
     * @tested PrimeMoverFramework\Tests\TestMigrationSystemChecks::itReturnsFalseWhenNoMySQLDumpHandlerInWindows()
     */
    public function getMySqlDumpPath()
    {
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        //Is system() possible
        if (! $this->hasSystem()) {
            return false;
        }
        $is_windows = false;
        
        if ($this->isWindows()) {
            $is_windows = true;            
            
        } 
        $paths = [];
        
        /**
         * Using MySQL environment variable(this wont go wrong)
         */
        $binary_executable = $this->getSystemCheckUtilities()->getMySQLBaseDirExecutablePath('mysqldump', $is_windows);
        if ($binary_executable && ! in_array($binary_executable, $paths)) {
            $paths[] = $binary_executable;
        }
        
        // Find the one which works
        foreach ($paths as $path) {
            if ($this->isExecutable($path)) {
                return $path;
            }
        }
        
        return false;
    }    
        
    /**
     * Checks if executable
     * @param string $path
     */
    public function isExecutable($path = '')
    {
        return @is_executable($path);
    }    
        
    /**
     * Can system() be called on this server
     * Modified from Snap Creek | https://wordpress.org/plugins/duplicator/
     * @return bool Returns true if system() can be called on server
     * @compatible 5.6
     *
     */
    public function hasSystem()
    {        
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        $cmds = $this->getSystemInitialization()->getCoreSystemFunctions();
        if (array_intersect($cmds, array_map('trim', explode(',', @ini_get('disable_functions'))))) {
            return false;
        }
        
        if (extension_loaded('suhosin')) {
            $suhosin_ini = @ini_get("suhosin.executor.func.blacklist");
            if (array_intersect($cmds, array_map('trim', explode(',', $suhosin_ini)))) {
                return false;
            }
        }
        
        if (! @shell_exec('echo multisite migration')) {
            return false;
        }
        
        return true;
    }
  
    /**
     * Checks if WAMP server
     * @return boolean
     * @tested PrimeMoverFramework\Tests\TestMigrationSystemChecks::itReturnsTrueIfWampServer
     * @tested PrimeMoverFramework\Tests\TestMigrationSystemChecks::itReturnsFalseIfNotWampServer
     * @tested PrimeMoverFramework\Tests\TestMigrationSystemChecks::itReturnsFalseIfNotOnWindows()
     */
    public function isWampServer()
    {
        return ($this->isWindows() && ! empty($_SERVER['DOCUMENT_ROOT']) && strpos(strtolower($_SERVER['DOCUMENT_ROOT']), 'wamp') !== false);
    }
    
    /**
     * Checks if using Microsoft IIS server
     * @return boolean
     */
    public function isIISServer()
    {
        global $is_IIS, $is_iis7;
        $ret = false;
        if ($is_IIS || $is_iis7) {
            $ret = true;
        }
        return $ret;
    }
    
    /**
     * Is the server running Windows operating system
     * CREDITS: Snap Creek | https://wordpress.org/plugins/duplicator/
     * @return bool Returns true if operating system is Windows
     * @compatible 5.6
     *
     */
    public function isWindows()
    {
        if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
            return true;
        }
        return false;
    }
    
    /**
     * Checks if can support encrypted Zip Archives
     * @return boolean
     */
    public function canSupportEncryptedArchive()
    {        
        $compatible_php_version = false;
        $has_encrypted_zip_support = false;
        if (version_compare(PHP_VERSION, '7.2.0') >= 0) {
            $compatible_php_version = true;
        }
        $ziparchive = new \ZipArchive();
        $methods = get_class_methods($ziparchive);
        if (in_array('setEncryptionName', $methods)) {
            $has_encrypted_zip_support = true;
        }
        
        return ($compatible_php_version && $has_encrypted_zip_support);
    }
    
    /**
     * Can decrypt media
     * @param boolean $ret
     * @param array $verify_result
     * @param array $phpinfoarray
     * @return string|boolean
     */
    public function canDecryptMedia($ret = true, $verify_result = [], $phpinfoarray = [])
    {        
        if (empty($verify_result) || empty($phpinfoarray)) {
            return $ret;
        }
        if (empty($verify_result['encrypted_media'])) {
            return $ret;
        }
        $encrypted_media = $verify_result['encrypted_media'];
        if ('false' === $encrypted_media) {
            return $ret;
        } 
        return $this->isLibZipVersionCompatible($phpinfoarray);
    }
    
    /**
     * Checks if libzip version is compatible
     * @param array $phpinfoarray
     * @return boolean
     */
    public function isLibZipVersionCompatible($phpinfoarray = [])
    {
        if (empty($phpinfoarray)) {
            return false;
        }
        $libzip_installed_version = $this->getSystemFunctions()->getLibZipversion($phpinfoarray);
        if (version_compare($libzip_installed_version, '1.2.0') >= 0) {
            return true;
        } else {
            return false;
        }
    }
}
