<?php
namespace Codexonics\PrimeMoverFramework\classes;

/*
 * This file is part of the Codexonics.PrimeMoverFramework package.
 *
 * (c) Codexonics Ltd
 *
 * This package is Open Source Software. For the full copyright and license
 * information, please view the LICENSE file which was distributed with this
 * source code.
 */

use Codexonics\PrimeMoverFramework\interfaces\PrimeMoverSystemProcessor;
use Codexonics\PrimeMoverFramework\utilities\PrimeMoverExportUtilities;
use Codexonics\PrimeMoverFramework\utilities\PrimeMoverUploadUtilities;

if (! defined('ABSPATH')) {
    exit;
}

/**
 * Prime Mover System Processors Class
 *
 * The Prime Mover System Processor Class aims to provide the AJAX processor facility for exporting,importing and deleting temporary files.
 */
class PrimeMoverSystemProcessors implements PrimeMoverSystemProcessor
{
    private $system_authorization;
    private $system_checks;
    private $system_initialization;
    private $system_functions;
    private $importer;
    private $upload_utilities;
    private $export_utilities;
    private $progress_handlers;
    
    /**
     * Constructor
     * @param PrimeMoverSystemAuthorization $system_authorization
     * @param PrimeMoverSystemChecks $system_checks
     * @param PrimeMoverSystemInitialization $system_initialization
     * @param PrimeMoverSystemFunctions $system_functions
     * @param PrimeMoverImporter $importer
     * @param PrimeMoverUploadUtilities $upload_utilities
     * @param PrimeMoverExportUtilities $export_utilities
     * @param PrimeMoverProgressHandlers $progress_handlers
     */
    public function __construct(
        PrimeMoverSystemAuthorization $system_authorization,
        PrimeMoverSystemChecks $system_checks,
        PrimeMoverSystemInitialization $system_initialization,
        PrimeMoverSystemFunctions $system_functions,
        PrimeMoverImporter $importer,
        PrimeMoverUploadUtilities $upload_utilities,
        PrimeMoverExportUtilities $export_utilities,
        PrimeMoverProgressHandlers $progress_handlers
    ) 
    {
        $this->system_authorization = $system_authorization;
        $this->system_checks = $system_checks;
        $this->system_initialization = $system_initialization;
        $this->system_functions = $system_functions;
        $this->importer = $importer;
        $this->upload_utilities = $upload_utilities;
        $this->export_utilities = $export_utilities;
        $this->progress_handlers = $progress_handlers;
    }
 
    /**
     * Get progress handlers
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverProgressHandlers
     * @compatible 5.6
     */
    public function getProgressHandlers() 
    {
        return $this->progress_handlers;
    }
    
    /**
     * Get export utilities
     * @return \Codexonics\PrimeMoverFramework\utilities\PrimeMoverExportUtilities
     * @compatible 5.6
     */
    public function getExportUtilities()
    {
        return $this->export_utilities;
    }
    
    /**
     *
     * Get upload utilities
     * @return \Codexonics\PrimeMoverFramework\utilities\PrimeMoverUploadUtilities
     * @compatible 5.6
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverSystemProcessors::itProcessUploadsWhenAllSet() 
     */
    public function getUploadUtilities()
    {
        return $this->upload_utilities;
    }
    /**
     *
     * Get Importer
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverImporter
     * @compatible 5.6
     */
    public function getImporter()
    {
        return $this->importer;
    }
    
    /**
     *
     * Get system functions
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemFunctions
     * @compatible 5.6
     */
    public function getSystemFunctions()
    {
        return $this->system_functions;
    }
    
    /**
     *
     * Get System Initialization
     * @compatible 5.6
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemInitialization
     */
    public function getSystemInitialization()
    {
        return $this->system_initialization;
    }
    
    /**
     *
     * Get System checks
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemChecks
     * @compatible 5.6
     */
    public function getSystemChecks()
    {
        return $this->system_checks;
    }
    
    /**
     *
     * Get System authorization
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemAuthorization
     * @compatible 5.6
     */
    public function getSystemAuthorization()
    {
        return $this->system_authorization;
    }

    /**
     *
     * Verify encrypted package as requested from client side
     * @compatible 5.6
     */
    public function primeMoverVerifyEncryptedPackage()
    {
        $response = [];
        if ( ! $this->getSystemAuthorization()->isUserAuthorized()) {
            $response['result'] = false;
            $response['error'] = __('Unauthorized request', 'prime-mover');
            return wp_send_json($response);
        }

        $args = [
            'prime_mover_decryption_check_nonce' => FILTER_SANITIZE_STRING,
            'stringtocheck' => FILTER_SANITIZE_STRING,
            'encrypted_media' => FILTER_SANITIZE_STRING,
            'blog_id' => FILTER_SANITIZE_NUMBER_INT
        ];

        $verify_result = $this->getSystemInitialization()->getUserInput('post', $args, 'verify_encrypted_package', '', 0, true, true);
        if ( empty($verify_result['prime_mover_decryption_check_nonce'] ) ) {
            $response['result'] = false;
            $response['error'] = __('Incorrect nonce', 'prime-mover');
            return wp_send_json($response);
        }

        if ( ! wp_verify_nonce($verify_result['prime_mover_decryption_check_nonce'], 'prime_mover_decryption_check_nonce') ) {
            $response['result'] = false;
            $response['error'] = __('Incorrect nonce', 'prime-mover');
            return wp_send_json($response);
        }
        
        if ( empty($verify_result['stringtocheck'] ) && ! empty($verify_result['validation_errors']) ) {
            $response['result'] = false;
            $response['error'] = $verify_result['validation_errors']['stringtocheck'];
            return wp_send_json($response);
        }
        
        $phpinfoarray = $this->getProgressHandlers()->getShutDownUtilities()->phpinfo2array();        
        $can_decrypt_media = apply_filters('prime_mover_can_decrypt_media', true, $verify_result, $phpinfoarray);
        if (false === $can_decrypt_media) {
            $response['result'] = false;
            $response['error'] = $this->getSystemInitialization()->returnCommonMediaDecryptionError();
            return wp_send_json($response);
        }
        $response['result'] = true;
        return wp_send_json($response);
    }
    
    /**
     *
     * Delete temporary file when user cancels the import
     * @compatible 5.6
     */
    public function primeMoverTempfileDeleteProcessor()
    {
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        
        //Initialize response
        $response = [];
        $response['tempfile_deletion_status'] = false;
        
        //Initialize args
        $args = [
            'prime_mover_deletetmpfile_nonce' => FILTER_SANITIZE_STRING,
            'temp_file_to_delete' => FILTER_SANITIZE_STRING,
            'action' => FILTER_SANITIZE_STRING,
            'diff_reject' => FILTER_SANITIZE_STRING,
            'blog_id' => FILTER_SANITIZE_NUMBER_INT,
            'mode' => FILTER_SANITIZE_STRING
        ];
        
        //Validate
        $posted_blog_id = $this->getSystemInitialization()->getUserInput('post', ['blog_id' => FILTER_SANITIZE_NUMBER_INT], 'delete_tmp_dir', '', 0, true);
        $posted_mode = $this->getSystemInitialization()->getUserInput('post', ['mode' => FILTER_SANITIZE_STRING], 'delete_tmp_dir', '', 0, true);
        $blog_id = 0;
        if ( ! empty($posted_blog_id['blog_id']) ) {
            $blog_id = $posted_blog_id['blog_id'];
        }
        $mode = 'import';
        if ( ! empty($posted_mode['mode']) ) {
            $mode = $posted_mode['mode'];
        }
        $delete_tmpfile_post = $this->getSystemInitialization()->getUserInput('post', $args, 'delete_tmp_dir', $mode, $blog_id, true);
        if (isset($delete_tmpfile_post['prime_mover_deletetmpfile_nonce']) &&
            isset($delete_tmpfile_post[ 'temp_file_to_delete' ]) && wp_verify_nonce($delete_tmpfile_post['prime_mover_deletetmpfile_nonce'], 'prime_mover_deletetmpfile_nonce')) {
            //Get temp directory to delete
            global $wp_filesystem;
            $temp_dir = $delete_tmpfile_post[ 'temp_file_to_delete' ];
            if ($temp_dir && $wp_filesystem->exists($temp_dir)) {
                $this->getSystemFunctions()->primeMoverDoDelete($temp_dir);
                $response['tempfile_deletion_status'] = true;
            }
        }
        //Other actions
        do_action('prime_mover_do_things_tmp_deleted', $delete_tmpfile_post);
        
        //Update client of the status        
        wp_send_json($response);
    }    
    
    /**
     *
     * Export progress shutdown
     * @compatible 5.6
     */
    public function primeMoverShutdownExportProcessor()
    {
        $this->getProgressHandlers()->commonShutDownProcessor('export');
    }
    
    /**
     *
     * Import progress shutdown
     * @compatible 5.6
     */
    public function primeMoverShutdownImportProcessor()
    {
        $this->getProgressHandlers()->commonShutDownProcessor('import');
    }
    
    /**
     *
     * Import progress process
     * @compatible 5.6
     */
    public function primeMoverImportProgressProcessor()
    {
        $this->getProgressHandlers()->commonProgressProcessor('import');
    }
 
    /**
     *
     * Export progress process
     * @compatible 5.6
     */
    public function primeMoverExportProgressProcessor()
    {
        $this->getProgressHandlers()->commonProgressProcessor('export');
    }
    
    /**
     * AJAX Processor for "chunk" uploading
     * @compatible 5.6
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverSystemProcessors::itProcessUploadsWhenAllSet() 
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverSystemProcessors::itDoesNotProcessUploadsIfNotAuthorized() 
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverSystemProcessors::itDoesNotProcessUploadsWhenNonceFailed()
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverSystemProcessors::itDoesNotProcessUploadsIfNotReallyUploaded()
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverSystemProcessors::itDoesNotProcessUploadsIfNotCorrectMimeType()
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverSystemProcessors::itProcessMissingChunks() 
     */
    public function primeMoverUploadsProcessor()
    {
        $all_clear	= false;
        
        //Initialize response
        $response = [];

        if ( ! $this->getSystemAuthorization()->isUserAuthorized()) {
            $response['status']= false ;
            $response['error'] = esc_js(__('Unauthorized request.', 'prime-mover'));
            
            return wp_send_json($response); 
        }
        
        $uploads_ajax_input = $this->getUploadUtilities()->getUploadsAjaxInput();
        if ( ! empty($uploads_ajax_input['multisite_blogid_to_import']) ) {
            $this->getSystemInitialization()->setImportBlogID($uploads_ajax_input['multisite_blogid_to_import']);
        }
        $all_clear = $this->getUploadUtilities()->validateChunkUploadCall($uploads_ajax_input, $uploads_ajax_input['multisite_blogid_to_import']);
        $assembled = $this->getUploadUtilities()->maybePackageIsNowAssembled($uploads_ajax_input['multisite_blogid_to_import']);        
        
        if ($assembled) {
            $response['assembled']= true;
            do_action('prime_mover_log_processed_events', "Package already assembled, no more work needed", $uploads_ajax_input['multisite_blogid_to_import'], 'import', 'primeMoverUploadsProcessor', $this);
            return wp_send_json($response);
        }
        
        $isFixingMissingChunk = $this->getUploadUtilities()->isFixingMissingChunk($uploads_ajax_input);
        
        if (false === $all_clear) {
            do_action('prime_mover_log_processed_events', "Upload chunk validation failed.", $uploads_ajax_input['multisite_blogid_to_import'], 'import', 'primeMoverUploadsProcessor', $this);
            return wp_send_json($response);
        }
        
        /**
         * All clear here
         */
        $files_array = $all_clear;
        
        if (! isset($files_array['file'])) {
            $response['status']= false ;
            $response['error']	= esc_html__('No file uploaded', 'prime-mover');
            
            return wp_send_json($response);
        }
        
        $uploadedfile 		= $files_array['file'];
        $upload_overrides 	= [ 'test_form' => false, 'test_type' => false ];
        $fileName = sanitize_file_name($files_array["file"]["name"]);
        
        // Get which chunk is being received
        $chunk = (int) $uploads_ajax_input['chunk'];
        $chunks = (int) $uploads_ajax_input['chunks'];
        
        do_action('prime_mover_log_processed_events', "Server side processing chunk: $chunk of $chunks", $uploads_ajax_input['multisite_blogid_to_import'], 'import', 'primeMoverUploadsProcessor', $this);        
        $this->getUploadUtilities()->setChunk($chunk);
        
        $ret = $this->getUploadUtilities()->moveUploadedChunkToUploads($uploadedfile, $upload_overrides, $fileName);
        
        //Check for uploading errors
        global $wp_filesystem;
        if (! isset($ret['file']) || ! $wp_filesystem->exists($ret['file'])) {
            return wp_send_json($response);
        }
        
        if (isset($ret['error'])) {
            //Error exist, terminate upload and inform client.
            $response['status']= false ;
            $response['error']	= esc_html__('Chunk upload error.', 'prime-mover');
            $this->getSystemFunctions()->primeMoverDoDelete($ret['file']);
            return wp_send_json($response);
        }
        
        $filePath = dirname($ret['file']) . DIRECTORY_SEPARATOR . $fileName;
        $done = $this->getUploadUtilities()->maybeReassembleImportPackageChunks($chunks, $chunk, $filePath, $isFixingMissingChunk);        
        
        if (! is_bool($done) && $done > 0) {
            do_action('prime_mover_log_processed_events', "Reporting missing chunk back to client browser: $done", $uploads_ajax_input['multisite_blogid_to_import'], 'import', 'primeMoverUploadsProcessor', $this);
            $response['missing_chunk'] = $done;
            return wp_send_json($response);
        }
        
        while($this->getUploadUtilities()->maybePackageIsUnderAssemblyNow($uploads_ajax_input['multisite_blogid_to_import'])) {
            sleep(1);
        }
        
        if (true === $done && ! $this->getUploadUtilities()->isReallyZip($filePath)) {
            $response['status']= false ;
            $response['error'] = esc_html__('File type error.', 'prime-mover');
            
            do_action('prime_mover_log_processed_events', "File type error DETECTED: $filePath", $uploads_ajax_input['multisite_blogid_to_import'], 'import', 'primeMoverUploadsProcessor', $this);
            $this->getSystemFunctions()->primeMoverDoDelete($filePath);
            return wp_send_json($response);
        }
        $this->getSystemInitialization()->setSlowProcess();
        $response['status']= true ;
        $response['done'] = $done;
        
        $response['chunk']= $chunk;
        $response['chunks']= $chunks;
        $response['filepath'] = $filePath;
        
        $actual_progress = $this->getUploadUtilities()->getActualUploadProgress($uploads_ajax_input['multisite_blogid_to_import'], $chunk);
        $response['actualprogress'] = $actual_progress;
        
        do_action('prime_mover_log_processed_events', "Returning response $chunk of $chunks of $filePath", $uploads_ajax_input['multisite_blogid_to_import'], 'import', 'primeMoverUploadsProcessor', $this); 
        do_action('prime_mover_log_processed_events', "Returning actual upload progress of $actual_progress out of $chunks", $uploads_ajax_input['multisite_blogid_to_import'], 'import', 'primeMoverUploadsProcessor', $this); 
        wp_send_json($response);
    }
    
    /**
     * Process import request via AJAX
     * {@inheritDoc}
     * @see PrimeMoverSystemProcessor::primeMoverImportProcessor()
     * @compatible 5.6
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverSystemProcessors::itRunsPrimeMoverImportProcessorIfAllClear()
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverSystemProcessors::itDoesNotRunImportProcessorIfNotAuthorized()
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverSystemProcessors::itReturnsErrorIfImportProcessorFails()
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverSystemProcessors::itReturnsGenericErrorIfImportReturnsUndefined()
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverSystemProcessors::itDoesNotRunImportProcessorIfNonceFailed()
     */
    public function primeMoverImportProcessor()
    {
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        $response		= [];
        $all_clear		= false;
        $ret			= [];
        $files_array	= [];       
        $args		= [
            'nonce_to_continue'	=> FILTER_SANITIZE_STRING,
            'data_to_continue'	=> FILTER_SANITIZE_STRING,
            'diff_blog_id' => FILTER_SANITIZE_NUMBER_INT,
            'prime_mover_next_import_method' => FILTER_SANITIZE_STRING,
            'prime_mover_current_import_method' => FILTER_SANITIZE_STRING
        ];
        $continue_import_post = $this->getSystemInitialization()->getUserInput('post', $args, 'diff_import_processor_ajax', 'import');                
        if ( ! empty($continue_import_post['nonce_to_continue']) && wp_verify_nonce($continue_import_post['nonce_to_continue'], 'prime_mover_continue_import') && ! empty($continue_import_post[ 'data_to_continue' ]) ) {            
            
            $initiating_variables = $this->isProcessAlreadyInitiated('import', $continue_import_post);
            $import_method = $initiating_variables['next_process_method'];
            $import_initiated = $initiating_variables['process_initiated'];
            
            $data_to_continue = $continue_import_post[ 'data_to_continue' ];
            $data_to_continue = stripslashes(html_entity_decode($data_to_continue));
            $data_to_continue = str_replace('\\', '/', $data_to_continue);
            $data_to_continue = json_decode($data_to_continue, true);
            if ((is_array($data_to_continue)) && (!empty($data_to_continue))) {
                global $wp_filesystem;
                
                if ((isset($data_to_continue['unzipped_directory'])) &&
                   (isset($data_to_continue['diff'])) &&
                   (isset($data_to_continue['blog_id']))
                        ) {
                    $unzipped_directory_to_check	= $data_to_continue['unzipped_directory'];                    
                    if ($wp_filesystem->exists($unzipped_directory_to_check)) {                        
                        unset($data_to_continue['diff']);                        
                        $data_to_continue['diff_confirmation'] = true;                        
                        $blogid_to_import	= (int) $data_to_continue['blog_id'];
                        if ($blogid_to_import > 0) {                            
                            $ret = $data_to_continue;
                            do_action('prime_mover_log_processed_events', 'Diff all cleared and validated, continuing the import', $blogid_to_import, 'import', 'primeMoverImportProcessor', $this);
                            $all_clear	= true;
                        }
                    }
                }
            }           
        } else {            
            $methods_input = $this->getSystemInitialization()->getUserInput('post', ['prime_mover_next_import_method' => FILTER_SANITIZE_STRING, 'prime_mover_current_import_method' => FILTER_SANITIZE_STRING], 'import_processor_ajax', 'import');
            $initiating_variables = $this->isProcessAlreadyInitiated('import', $methods_input);
            $import_method = $initiating_variables['next_process_method'];
            $import_initiated = $initiating_variables['process_initiated'];
            $args = [
                'prime_mover_import_nonce' => FILTER_SANITIZE_STRING,
                'multisite_blogid_to_import' => FILTER_SANITIZE_NUMBER_INT,
                'multisite_import_package_uploaded_file' => FILTER_SANITIZE_STRING,
                'prime_mover_next_import_method' => FILTER_SANITIZE_STRING,
                'prime_mover_current_import_method' => FILTER_SANITIZE_STRING
            ];
            if ($import_initiated) {
                unset($args['multisite_import_package_uploaded_file']); 
            }
            $import_ajax_input = $this->getSystemInitialization()->getUserInput('post', $args, 'import_processor_ajax', 'import');             
            if ( ! empty($import_ajax_input['prime_mover_import_nonce']) && wp_verify_nonce($import_ajax_input['prime_mover_import_nonce'], 'prime_mover_import_nonce') && 
                ! empty($import_ajax_input[ 'multisite_blogid_to_import' ]) ) {                       
                $blogid_to_import	= (int) $import_ajax_input[ 'multisite_blogid_to_import' ];
                $requisites_meet = false;
                if ($blogid_to_import > 0 && true === $this->getSystemChecks()->primeMoverEssentialRequisites()) {
                    $requisites_meet = true;
                }
                $import_package_path = '';
                if ( ! $import_initiated && ! empty($import_ajax_input[ 'multisite_import_package_uploaded_file']) ) {
                    $import_package_path = $import_ajax_input[ 'multisite_import_package_uploaded_file'];
                }
                if ( ! $import_initiated && $blogid_to_import > 0 && $requisites_meet && $this->getSystemFunctions()->fileExists($import_package_path)) {                    
                    do_action('prime_mover_log_processed_events', 'Original handler cleared and validated, starting the import.', $blogid_to_import, 'import', 'primeMoverImportProcessor', $this);
                    $all_clear	= true;                            
                } 
                if ($import_initiated && $requisites_meet) {
                    do_action('prime_mover_log_processed_events', 'Original handler cleared and validated, continuing the import.', $blogid_to_import, 'import', 'primeMoverImportProcessor', $this);
                    $all_clear	= true;   
                }                
            }
        }
        if (true === $all_clear) {            
            $this->getProgressHandlers()->initializeProgressTracker($blogid_to_import, 'import');
            if (! isset($ret['diff'])) {
                $import_progress = $this->getProgressHandlers()->getTrackerProgressNonCached($blogid_to_import, 'import');
                if ( ! $import_initiated && $import_progress && 'boot' !== $import_progress) {
                    do_action('prime_mover_log_processed_events', 'Import already started, aborting with progress: ' . $import_progress, $blogid_to_import, 'import', 'primeMoverImportProcessor', $this);
                    wp_die();
                }              
                if ( ! $import_initiated ) {
                    $this->getProgressHandlers()->updateTrackerProgress(esc_html__('Starting import', 'prime-mover'));
                    do_action('prime_mover_log_processed_events', 'Import request received..', 0, 'import', 'importProcessorTriggeredStatus', $this);
                }
                if (isset($import_ajax_input)) {
                    do_action('prime_mover_log_processed_events', $import_ajax_input, 0, 'import', 'importProcessorTriggeredStatus', $this);
                }                
            }
            
            if (defined('PRIME_MOVER_TEST_UPLOAD') && true === PRIME_MOVER_TEST_UPLOAD) {
                $response['status'] = true ;
                $response['import_successful'] = '<img src="' . esc_url_raw(plugins_url('res/img/done.png', dirname(__FILE__))) . '" />';
                
                $this->saveImportResultForOutput($response, 'stoptracking');
                return wp_die();
            }
            
            $files_array = [];
            if ( ! empty($import_package_path) ) {
                $files_array['file'] = $import_package_path;
            }
            set_time_limit(0);
            do_action('prime_mover_before_doing_import', $blogid_to_import, $import_initiated);
            $results = apply_filters("prime_mover_do_import_{$import_method}", $ret, $blogid_to_import, $files_array);
            
            do_action('prime_mover_log_processed_events', 'Exited import results filter.', $blogid_to_import, 'import', 'primeMoverImportProcessor', $this);
            $status = '';
            $stop_tracking = false;
            
            if (isset($results['error'])) {
                
                do_action('prime_mover_log_processed_events', 'Import error found', $blogid_to_import, 'import', 'primeMoverImportProcessor', $this);
                $stop_tracking = true;
                $status = 'stoptracking';
                $response['status']	= false ;
                $error_message		= $results['error'];
                $response['import_not_successful'] = esc_js($error_message);
                
            } elseif (isset($results['diff'])) {
                
                $stop_tracking = true;
                $status = 'diffdetected';
                $nonce_to_user = wp_create_nonce('prime_mover_continue_import');
                $diff = $results['diff'];
                $diff_friendly = $this->getSystemFunctions()->printFriendlyDiffMessages($diff);
                
                $response['diff_detected'] = true ;
                $response['diff'] = $diff_friendly;
                $response['continue_nonce']	= $nonce_to_user;
                $response['results'] = $results;
                $response['next_method'] = $results['next_method'];
                $response['current_method'] = $results['current_method'];
                if ( ! empty($results['unzipped_directory']) ) {
                    $response['unzipped_directory'] = $results['unzipped_directory'];
                }
                do_action('prime_mover_log_processed_events', 'Diff detected', $blogid_to_import, 'import', 'primeMoverImportProcessor', $this);
                
            } elseif (isset($results['ongoing_import']) && isset($results['next_method']) && isset($results['current_method'])) {
                
                $response['ongoing_import'] = $results['ongoing_import'];
                $response['next_method'] = $results['next_method'];
                $response['current_method'] = $results['current_method'];
                if ( ! empty($results['unzipped_directory']) ) {
                    $response['unzipped_directory'] = $results['unzipped_directory'];
                }
                
                return wp_send_json($response);
                
            } elseif ((isset($results['import_success'])) && ((true === $results['import_success']))) {         
                
                do_action('prime_mover_log_processed_events', 'Import success, returning stoptracking', $blogid_to_import, 'import', 'primeMoverImportProcessor', $this);
                $stop_tracking = true;
                $status = 'stoptracking';
                $response['status']				= true ;
                $response['import_successful']	= '<img src="' . esc_url_raw(plugins_url('res/img/done.png', dirname(__FILE__))) . '" />'; 
                
            } else {   
                
                $stop_tracking = true;
                $response['status']= false ;
                $status = 'stoptracking';
                $response['import_not_successful'] = esc_js(__('Import failed for unknown reason.', 'prime-mover'));
            }  
            
            if ($stop_tracking) {
                do_action('prime_mover_log_processed_events', 'Saving import result output:', $blogid_to_import, 'import', 'primeMoverImportProcessor', $this);
                do_action('prime_mover_log_processed_events', $response, $blogid_to_import, 'import', 'primeMoverImportProcessor', $this);
                
                $this->saveImportResultForOutput($response, $status);  
            }           
        } 
        wp_die();
    }
    
    /**
     * Is Process already initiated
     * @param string $mode
     * @param array $process_input
     * @return array
     */
    protected function isProcessAlreadyInitiated($mode = 'export', $process_input = [])
    {
        $initiated = false;
        $methods = $this->getSystemInitialization()->getPrimeMoverExportMethods();
        if ('import' === $mode) {
            $methods = $this->getSystemInitialization()->getPrimeMoverImportMethods();
        }        
        $method = $methods[0];
        $key = "prime_mover_next_{$mode}_method";
        if ( ! empty($process_input[$key])) {
            $initiated = true;
            $method = $process_input[$key];
        }
        
        return [ 'process_initiated' => $initiated , 'next_process_method' => $method];
    }
    
    /**
     * Save import result for output
     * @param array $response
     */
    protected function saveImportResultForOutput($response = [], $status = '') 
    {
        if ( ! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        $import_id = $this->getSystemInitialization()->getImportId();
        $response['process_id'] = $import_id;
        
        $option = 'import_' . $import_id;
        $this->getSystemFunctions()->updateSiteOption($option, $response);
        if ($status) {
            $this->getProgressHandlers()->updateTrackerProgress($status);
        }
        sleep(3); 
    }
    
    /**
     * Process export request via AJAX
     * @compatible 5.6
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverSystemProcessors::itRunsPrimeMoverExportProcessorIfAllClear() 
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverSystemProcessors::itReturnsErrorIfDetectedOnExportProcessor()
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverSystemProcessors::itDoesNotAllowExportProcessorCallIfNotAuthorized()
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverSystemProcessors::itDoesNotAllowExportProcessorCallIfNonceFailed()
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverSystemProcessors::itDoesNotAllowExportProcessorCallIfOptionInvalid()
     */
    public function primeMoverExportProcessor()
    {
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        $all_clear	= false;        
        $args = apply_filters( 'prime_mover_export_processor_args' , 
            [
                'prime_mover_export_nonce' => FILTER_SANITIZE_STRING, 
                'multisite_blogid_to_export' => FILTER_SANITIZE_NUMBER_INT, 
                'multisite_export_options' => FILTER_SANITIZE_STRING,
                'prime_mover_export_targetid' => FILTER_SANITIZE_NUMBER_INT,
                'prime_mover_next_export_method' => FILTER_SANITIZE_STRING,
                'prime_mover_current_export_method' => FILTER_SANITIZE_STRING
            ]);

        $multisite_export_options = '';        
        $response	= [];        
        $export_ajax_input = $this->getSystemInitialization()->getUserInput('post', $args, 'export_processor_ajax', 'export');
        
        $initiating_variables = $this->isProcessAlreadyInitiated('export', $export_ajax_input);
        $export_method = $initiating_variables['next_process_method'];
        $export_initiated = $initiating_variables['process_initiated'];
        
        if ( ! empty($export_ajax_input['prime_mover_export_nonce']) && wp_verify_nonce($export_ajax_input['prime_mover_export_nonce'], 'prime_mover_export_nonce') && 
            ! empty($export_ajax_input[ 'multisite_blogid_to_export' ]) ) {                    
            $blogid_to_export	= $export_ajax_input[ 'multisite_blogid_to_export' ];
            $blogid_to_export	= intval($blogid_to_export);
            
            $multisite_export_options = apply_filters('prime_mover_get_export_option', $export_ajax_input['multisite_export_options'], $blogid_to_export);
            if (($blogid_to_export > 0) && (true === $this->getSystemChecks()->primeMoverEssentialRequisites()) && $multisite_export_options) {
                $this->getSystemInitialization()->setExportBlogID($blogid_to_export);
                $all_clear	= true;
                do_action('prime_mover_log_processed_events', 'Export parameters cleared and validated, starting the export.', $blogid_to_export, 'export', 'primeMoverExportProcessor', $this);
            }                            
        }
        
        if (true === $all_clear) {   
            $export_options_array = [];

            $this->getProgressHandlers()->initializeProgressTracker($blogid_to_export, 'export');
            do_action('prime_mover_before_doing_export', $blogid_to_export, $export_initiated);
            if ( ! $export_initiated ) {
             $this->getProgressHandlers()->updateTrackerProgress(esc_html__('Starting export', 'prime-mover'), 'export');
            }
            $export_options_array = ['multisite_export_options' => $multisite_export_options ];
            if ( ! empty($export_ajax_input['prime_mover_export_targetid'] ) ) {
                $export_options_array['prime_mover_export_targetid'] = $export_ajax_input['prime_mover_export_targetid'];
            }           

            set_time_limit(0);
            $processor_array = apply_filters( 'prime_mover_export_processor_array', $export_options_array, $blogid_to_export, $export_ajax_input);
            
            $results = apply_filters("prime_mover_export_{$export_method}", $processor_array, $blogid_to_export);
            $stop_tracking = false;
            
            if (isset($results['error'])) {
                $stop_tracking = true;
                $response['status']	= false ;
                $error_message		= $results['error'];
                $response['export_not_successful'] = esc_js($error_message);
                do_action('prime_mover_log_processed_events', 'Export result error found:' . $error_message, $blogid_to_export, 'export', 'primeMoverExportProcessor', $this);
            } elseif (isset($results['ongoing_export']) && isset($results['next_method']) && isset($results['current_method'])) {                
                $response['ongoing_export'] = $results['ongoing_export'];
                $response['next_method'] = $results['next_method'];
                $response['current_method'] = $results['current_method'];
                if ( ! empty($results['temp_folder_path']) ) {
                    $response['temp_folder_path'] = $results['temp_folder_path'];
                }
                return wp_send_json($response);                
            } elseif (isset($results['multisite_export_location']) && 'export_directory' === $results['multisite_export_location']) { 
                $stop_tracking = true;
                do_action('prime_mover_log_processed_events', 'Export location defined', $blogid_to_export, 'export', 'primeMoverExportProcessor', $this);
                $response['status'] = true ;
                $response['export_location'] = 'export_directory';
                $response['restore_url'] = esc_url_raw($results['download_url']);
                $response['message'] = apply_filters('prime_mover_filter_custom_export_success_message', esc_js(__('Export saved !', 'prime-mover')), $blogid_to_export);                   
            } elseif (isset($results['download_url'])) {  
                $stop_tracking = true;
                do_action('prime_mover_log_processed_events', 'Download URL defined', $blogid_to_export, 'export', 'primeMoverExportProcessor', $this);
                $download_url				= $results['download_url'];
                $response['status']			= true ;
                $response['download_link']	= esc_url_raw($download_url);                
            } else {
                $stop_tracking = true;
                do_action('prime_mover_log_processed_events', 'Generic export not successful logged', $blogid_to_export, 'export', 'primeMoverExportProcessor', $this);
                $response['status']= false ;
                $response['export_not_successful'] = esc_js(__('Export failed', 'prime-mover'));               
            }            
            
            if ($stop_tracking) {
                $this->logStopTracking($blogid_to_export, $results);
                $this->saveCompletedExportResponse($blogid_to_export, $response);
            }            
        }
        wp_die();
    }
    
    /**
     * Log stop tracking
     * @param number $blogid_to_export
     * @param array $results
     */
    protected function logStopTracking($blogid_to_export = 0, $results = [])
    {
        do_action('prime_mover_log_processed_events', 'Stop tracking requested', $blogid_to_export, 'export', 'logStopTracking', $this);
        do_action('prime_mover_log_processed_events', 'Export result:', $blogid_to_export, 'export', 'logStopTracking', $this);
        do_action('prime_mover_log_processed_events', $results, $blogid_to_export, 'export', 'logStopTracking', $this);
    }

    /**
     * Save completed export response
     * @param number $blogid_to_export
     * @param array $response
     */
    protected function saveCompletedExportResponse($blogid_to_export = 0, $response = [])
    {
        do_action('prime_mover_log_processed_events', 'Saving export result output:', $blogid_to_export, 'export', 'primeMoverExportProcessor', $this);
        do_action('prime_mover_log_processed_events', $response, $blogid_to_export, 'export', 'primeMoverExportProcessor', $this);
        
        $this->saveExportResultForOutput($response, 'stoptracking');         
    }
    
    /**
     * Save export result for outputting
     * @param array $response
     * @param string $status
     */
    protected function saveExportResultForOutput($response = [], $status = '')
    {
        if ( ! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        $export_id = $this->getSystemInitialization()->getExportId();
        $response['process_id'] = $export_id;
        
        $option = 'export_' . $export_id;
        $this->getSystemFunctions()->updateSiteOption($option, $response);
        if ($status) {
            $this->getProgressHandlers()->updateTrackerProgress($status, 'export');
        }
        sleep(3); 
    }
}
