<?php
namespace Codexonics\PrimeMoverFramework\compatibility;

/*
 * This file is part of the Codexonics.PrimeMoverFramework package.
 *
 * (c) Codexonics Ltd
 *
 * This package is Open Source Software. For the full copyright and license
 * information, please view the LICENSE file which was distributed with this
 * source code.
 */

use Codexonics\PrimeMoverFramework\classes\PrimeMover;
use WP_Screen;

if (! defined('ABSPATH')) {
    exit;
}

/**
 * Prime Mover Compatibility Class
 * Helper class for interacting with third party plugins
 *
 */
class PrimeMoverCompatibility
{     
    private $prime_mover;
    private $import_utilities;
    
    /**
     * Construct
     * @param PrimeMover $prime_mover
     * @param array $utilities
     */
    public function __construct(PrimeMover $prime_mover, $utilities = [])
    {
        $this->prime_mover = $prime_mover;
        $this->import_utilities = $utilities['import_utilities'];
    }
    
    /**
     * Get Prime Mover instance
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMover
     */
    public function getPrimeMover()
    {
        return $this->prime_mover;
    }
    
    /**
     * Get system functions
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemFunctions
     */
    public function getSystemFunctions()
    {
        return $this->getPrimeMover()->getSystemFunctions();
    }
    
    /**
     * Get system authorization
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemAuthorization
     */
    public function getSystemAuthorization()
    {
        return $this->getPrimeMover()->getSystemAuthorization();
    }
    
    /**
     * Get system initialization
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemInitialization
     */
    public function getSystemInitialization()
    {
        return $this->getPrimeMover()->getSystemInitialization();
    }
    
    /**
     * Initialize hooks
     */
    public function initHooks()
    {
        add_action('prime_mover_after_db_processing', [$this, 'deactivateReallySimpleSSLNonSecureSite'], 0, 2);
        add_action('prime_mover_load_module_apps', [$this, 'ensureCorrectSchemeIsUsed'], 0);
        add_action('init', [$this, 'destroyInCompatibleSessionOnAjax'], 999999);
        
        add_filter('wp_auth_check_load', [$this, 'removeInterimLoginOnMigrationToolsScreen'], 10, 2);
        add_filter('prime_mover_footprint_keys', [$this, 'primeMoverGenerateFootPrintKeys'], 99999, 2);
        add_filter('prime_mover_validate_site_footprint_data', [$this, 'primeMoverGenerateFootprintConfiguration'], 99999, 2);
    }
 
    /**
     * Log Prime Mover footprint keys
     * @param array $keys
     * @param array $footprint
     * @return array
     */
    public function primeMoverGenerateFootPrintKeys($keys = [], $footprint = [])
    {
        do_action('prime_mover_log_processed_events', "Footprint keys for validation :", 1, 'export', 'primeMoverGenerateFootPrintKeys', $this);
        do_action('prime_mover_log_processed_events', $keys, 1, 'export', 'primeMoverGenerateFootPrintKeys', $this);
        do_action('prime_mover_log_processed_events', "Custom log for site footprint data before check:", 1, 'export', 'primeMoverGenerateFootprintConfiguration', $this);
        do_action('prime_mover_log_processed_events', $footprint, 1, 'export', 'primeMoverGenerateFootprintConfiguration', $this);
        
        return $keys;        
    }
    
    /**
     * Log Prime Mover footprint configuration
     * @param boolean $overall_valid
     * @param array $footprint_temp
     * @return string
     */
    public function primeMoverGenerateFootprintConfiguration($overall_valid = true, $footprint_temp = [])
    {
        do_action('prime_mover_log_processed_events', "Footprint data validity is: $overall_valid", 1, 'export', 'primeMoverGenerateFootprintConfiguration', $this);
        do_action('prime_mover_log_processed_events', "Custom log for site footprint data after check:", 1, 'export', 'primeMoverGenerateFootprintConfiguration', $this);
        do_action('prime_mover_log_processed_events', $footprint_temp, 1, 'export', 'primeMoverGenerateFootprintConfiguration', $this);
        
        return $overall_valid;
    }
    /**
     * Disable interim login on Migration Tools screen
     * @param boolean $show
     * @param WP_Screen $screen
     * @return string|string|boolean
     */
    public function removeInterimLoginOnMigrationToolsScreen($show = true, WP_Screen $screen)
    {
        if (is_multisite()) {
            return $show;
        }
        if ( ! isset($screen->id) ) {
            return $show;
        }
        if ('tools_page_migration-tools' === $screen->id) {
            $show = false;
        }
        return $show;
    }
    
    /**
     * Destroy incompatible session on Prime Mover Ajax actions
     */
    public function destroyInCompatibleSessionOnAjax()
    {
        if ( ! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }  
        if ( ! wp_doing_ajax() ) {
            return;
        }
        if ( ! is_admin() ) {
            return;
        }
        $args = ['action' => FILTER_SANITIZE_STRING];
        $action = $this->getSystemInitialization()->getUserInput('post', $args, 'disable_session', 'export', 0, true); 
        if (empty($action)) {
            return;
        }
        if ( ! isset($action['action']) ) {
            return;            
        }
        $action_posted = $action['action'];
        $session_started = true;
        $valid_actions = array_keys($this->getPrimeMover()->getSystemInitialization()->getPrimeMoverAjaxActions());
        if (session_status() === PHP_SESSION_NONE) {
            $session_started = false;
        }
        if (in_array($action_posted, $valid_actions) && $session_started) {
            session_destroy();
        }        
    }
    
    /**
     * Ensure correct scheme is consistent because it can cause AJAX login session issues.
     */
    public function ensureCorrectSchemeIsUsed()
    {
        if ( ! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }        
        if ( ! is_admin() || ! is_ssl() || is_multisite() ) {
            return;
        }
        if (wp_doing_ajax()) {
            return;
        }
        if (doing_action('prime_mover_do_import')) {
            return;
        }
        if (doing_action('prime_mover_export')) {
            return;
        }        
        $site_url = get_option('siteurl');
        $parsed = wp_parse_url($site_url);
        if (empty($parsed['scheme'])) {
            return;
        }
        $scheme = $parsed['scheme'];
        $correct = '';
        if ('http' === $scheme) {       
            $correct = str_replace('http://', 'https://', $site_url);
            update_option( 'siteurl', $correct);
        }        
    }
    
    /**
     * Deactivate Really Simple SSL Plugin if restored on site without SSL working
     * @param array $ret
     * @param number $blog_id
     */
    public function deactivateReallySimpleSSLNonSecureSite($ret = [], $blog_id = 0)
    {
        if ( ! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        $this->getSystemFunctions()->switchToBlog($blog_id);
        $active_plugins = $this->getSystemFunctions()->getActivatedPlugins();
        if ( ! is_array( $active_plugins ) ) {
            return;
        }
        foreach ($active_plugins as $plugin) {
            if (false !== strpos($plugin, 'rlrsssl-really-simple-ssl.php') && ! is_ssl() ) {
                deactivate_plugins($plugin, true);
            }
        }
        $this->getSystemFunctions()->restoreCurrentBlog();
    }
}