<?php
namespace Codexonics\PrimeMoverFramework\utilities;

use Codexonics\PrimeMoverFramework\classes\PrimeMoverExporter;
use Codexonics\PrimeMoverFramework\classes\PrimeMoverImporter;

/*
 * This file is part of the Codexonics.PrimeMoverFramework package.
 *
 * (c) Codexonics Ltd
 *
 * This package is Open Source Software. For the full copyright and license
 * information, please view the LICENSE file which was distributed with this
 * source code.
 */

if (! defined('ABSPATH')) {
    exit;
}

/**
 * Prime Mover Component Auxiliary
 * Auxiliary component class for interacting with Prime Mover components
 *
 */
class PrimeMoverComponentAuxiliary
{        
    private $exporter;
    private $importer;
    
    /**
     * Constructor
     * @param PrimeMoverExporter $exporter
     * @param PrimeMoverImporter $importer
     */
    public function __construct(PrimeMoverExporter $exporter, PrimeMoverImporter $importer)
    {
        $this->exporter = $exporter;
        $this->importer = $importer;
    }
    
    /**
     * Get exporter
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverExporter
     */
    public function getExporter()
    {
        return $this->exporter;
    }
    
    /**
     * Get importer
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverImporter
     */
    public function getImporter()
    {
        return $this->importer;
    }
    
    /**
     * Get system authorization
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemAuthorization
     */
    public function getSystemAuthorization()
    {
        return $this->getExporter()->getSystemAuthorization();
    }
    
    /**
     * Get system initialization
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemInitialization
     */
    public function getSystemInitialization()
    {
        return $this->getExporter()->getSystemInitialization();
    }
    
    /**
     * Get system functions
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemFunctions
     */
    public function getSystemFunctions()
    {
        return $this->getExporter()->getSystemFunctions();
    }
    
    /**
     * Init hook class
     */
    public function initHooks()
    {
        add_action('prime_mover_before_db_dump_export', [$this, 'backupControlPanelSettings'], 15);
        add_action('prime_mover_before_db_dump_export', [$this, 'backupGearBoxPackages'], 20);
        add_action('prime_mover_before_db_dump_export', [$this, 'sanitizePrimeMoverSetting'], 25);

        add_action('prime_mover_after_db_dump_export', [$this, 'restoreControlPanelSettings'], 15);  
        add_action('prime_mover_after_db_dump_export', [$this, 'restoreGearBoxPackages'], 20);         
        add_action('prime_mover_shutdown_actions', [$this, 'restoreControlPanelSettings']);
        add_action('prime_mover_shutdown_actions', [$this, 'restoreGearBoxPackages']);
    }
    
    /**
     * Restore all multisite migration settings
     * @param array $settings
     * @moved
     */
    public function restoreAllPrimeMoverSettings($settings = [])
    {
        if ( ! is_array($settings) || empty($settings) ) {
            return;
        }
        if ( ! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        $setting_name = $this->getSystemInitialization()->getControlPanelSettingsName();
        update_option($setting_name, $settings);
    }
    
    /**
     * Delete all settings
     * @return boolean
     * @moved
     */
    public function deleteAllPrimeMoverSettings()
    {
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return false;
        }
        $setting_name = $this->getSystemInitialization()->getControlPanelSettingsName();
        return delete_option($setting_name);
    }
    
    /**
     * Restore control panel settings after dB processing
     * @moved
     */
    public function restoreControlPanelSettings()
    {
        if ( ! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        if (is_multisite()) {
            return;
        }
        $current_user_id = get_current_user_id();
        $current_settings = get_user_meta($current_user_id, $this->getSystemInitialization()->getMigrationCurrentSettings(), true);
        if (empty($current_settings)) {
            return;
        }
        $this->deleteAllPrimeMoverSettings();
        $this->restoreAllPrimeMoverSettings($current_settings);
        delete_user_meta($current_user_id, $this->getSystemInitialization()->getMigrationCurrentSettings());
    }
    
    /**
     * Sanitize prime mover setting before dumping database
     */
    public function sanitizePrimeMoverSetting()
    {
        if ( ! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        if (is_multisite()) {
            return;
        }
        $control_panel_settings = $this->getSystemInitialization()->getControlPanelSettingsName();
        delete_option($control_panel_settings);
    }
    
    /**
     * Restore gearbox packages after dB processing
     * @moved
     */
    public function restoreGearBoxPackages()
    {
        if ( ! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        if (is_multisite()) {
            return;
        }
        $current_user_id = get_current_user_id();
        $gearbox_packages = get_user_meta($current_user_id, $this->getSystemInitialization()->getCurrentGearBoxPackagesMetaKey(), true);
        if ( ! is_array($gearbox_packages) || empty($gearbox_packages) ) {
            return;
        }
        
        foreach ($gearbox_packages as $option_name => $option_value) {
            update_option($option_name, $option_value);
        }
        delete_user_meta($current_user_id, $this->getSystemInitialization()->getCurrentGearBoxPackagesMetaKey());
    }
    
     /**
     * Get all multisite migration settings
     * @return mixed|boolean|NULL|array
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverSettings::itGetsAllPrimeMoverSettings()
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverSettings::itReturnsOriginalValueIfEncryptedKeyNotSet()
     * @moved
     */
    public function getAllPrimeMoverSettings() 
    {
        $setting_name = $this->getSystemInitialization()->getControlPanelSettingsName();
        return get_option($setting_name);
    }
    
    /**
     * Backup control panel settings before dB processing
     * @moved
     */
    public function backupControlPanelSettings()
    {
        if ( ! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        if (is_multisite()) {
            return;
        }
        $current_user_id = get_current_user_id();
        $current_settings = $this->getAllPrimeMoverSettings();
        
        update_user_meta($current_user_id, $this->getSystemInitialization()->getMigrationCurrentSettings(), $current_settings);
    }
    
    /**
     * Generate download URL args
     * @param string $hash
     * @param number $blog_id
     * @return string[]|number[]
     * @moved
     */
    public function generateDownloadURLArgs($hash = '', $blog_id = 0)
    {
        return [
            'prime_mover_export_hash' => $hash,
            'prime_mover_blogid' => $blog_id
        ];
    }
    
    /**
     * Get download URL of zip for clipboard copying purposes
     * @param string $sanitized_name
     * @param number $blog_id
     * @return string
     * @moved
     */
    public function generateDownloadURLForClipBoard($sanitized_name = '', $blog_id = 0, $ret_array = false)
    {
        $ret = '';
        if ( ! $sanitized_name || ! $blog_id) {
            return $ret;
        }
        $option_name = $this->getSystemInitialization()->generateZipDownloadOptionName($sanitized_name, $blog_id);        
        $downloadurl_params = get_option($option_name);
        if ( ! $downloadurl_params ) {
            return $ret;
        }
        if (empty($downloadurl_params['blogid']) || empty($downloadurl_params['hash']) || empty($downloadurl_params['site_title'])) {
            return $ret;
        }
        $blog_id = (int)$blog_id;
        $blogid_db = (int)$downloadurl_params['blogid'];
        if ($blogid_db !== $blog_id) {
            return $ret;
        }
        $hash = $downloadurl_params['hash'];
        $download_args = $this->generateDownloadURLArgs($hash, $blog_id);
        
        if ($ret_array) {
            
            return [
                'download_url' => $this->getSystemInitialization()->getDownloadURLGivenParameters($download_args),
                'download_params' => $downloadurl_params
            ];
            
        }
        
        return $this->getSystemInitialization()->getDownloadURLGivenParameters($download_args);
    }
    
    /**
     * Validate backups to be listed
     * @param array $backups
     * @param number $blog_id
     * @return array
     * @moved
     */
    public function validateBackups($backups = [], $blog_id = 0)
    {
        if ( ! is_array($backups) ) {
            return $backups;
        }
        $files = array_keys($backups);
        foreach ($files as $file) {
            $sanitized_name = sanitize_html_class($file);
            $validated_download = $this->generateDownloadURLForClipBoard($sanitized_name, $blog_id, true);
            if ( ! $validated_download && isset($backups[$file]) ) {
                unset($backups[$file]);
                continue;
            }
            if ( ! empty($validated_download['download_params']['site_title'] ) ) {
                $backups[$file]['site_title'] = $validated_download['download_params']['site_title'];
            }
            $backups[$file]['date'] = '';
            if ( ! empty($validated_download['download_params']['creation_timestamp'] ) ) {
                $backups[$file]['date'] = $validated_download['download_params']['creation_timestamp'];
            }
        }
        return $backups;
    }
    
    /**
     * Backup gearbox packages before dB processing
     * @moved
     */
    public function backupGearBoxPackages()
    {
        if ( ! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        if (is_multisite()) {
            return;
        }
        $current_user_id = get_current_user_id();
        $blog_id = 1;
        delete_user_meta($current_user_id, $this->getSystemInitialization()->getCurrentGearBoxPackagesMetaKey());
        $gearbox_packages = [];
        
        $delete = false;
        $current_filter = current_filter();
        if ('prime_mover_before_db_dump_export' === $current_filter) {
            $delete = true;
        }
        $backups = $this->generateBackupFilesArray($blog_id, $delete);        
        $packages = [];
        
        if ( ! is_array($backups)) {
            return;
        }
        $files = array_keys($backups);
        $result = $this->generateGearBoxPackages($files, $packages, $blog_id, $gearbox_packages);
        
        if ($delete) {
            $packages = $result['packages'];
            foreach ($packages as $package) {
                delete_option($package);
            }            
        }
        update_user_meta($current_user_id, $this->getSystemInitialization()->getCurrentGearBoxPackagesMetaKey(), $result['gearbox_packages']);        
    } 
    
    /**
     * Generate backup files array
     * @param number $blog_id
     * @param boolean $delete
     * @return array|string
     */
    private function generateBackupFilesArray($blog_id = 0, $delete = false)
    {
        $override_dir = false;
        if ($delete && ! defined('PRIME_MOVER_PANEL_VERSION') ) {
            $override_dir = true;
        }
        $dir = '';
        if ($override_dir) {
            $setting = get_option($this->getSystemInitialization()->getControlPanelSettingsName());
            if ( ! empty($setting['basedir_backup_path']) ) {
                $dir = $setting['basedir_backup_path'];
            }
        }
        $backups = $this->getSystemFunctions()->getFilesToRestore($blog_id, $dir);
        $backups = $this->validateBackups($backups, $blog_id); 
        
        return $backups;
    }
    
    /**
     * Generate gearbox packages
     * @param array $files
     * @param array $packages
     * @param number $blog_id
     * @param array $gearbox_packages
     * @return mixed|boolean|NULL|array
     */
    private function generateGearBoxPackages($files = [], $packages = [], $blog_id = 0, $gearbox_packages = [])
    {
        $result = [];        
        foreach ($files as $file) {
            $sanitized_name = sanitize_html_class($file);
            $option_name = $this->getSystemInitialization()->generateZipDownloadOptionName($sanitized_name, $blog_id);
            $package_setting = get_option($option_name);
            if ($package_setting ) {
                $packages[] = $option_name;
            }            
            if ( ! is_array($package_setting) || empty($package_setting['hash']) ) {
                continue;
            }
            $hash = $package_setting['hash'];
            $hash_setting = get_option($hash);            
            if ( ! $hash_setting ) {
                continue;
            }
            $packages[] = $hash;
            $gearbox_packages[$option_name] = $package_setting;
            $gearbox_packages[$hash] = wp_normalize_path($hash_setting);
        }
        
        $result['packages'] = $packages;
        $result['gearbox_packages'] = $gearbox_packages;
        
        return $result;
    }
}