<?php
namespace Codexonics\PrimeMoverFramework\utilities;

/*
 * This file is part of the Codexonics.PrimeMoverFramework package.
 *
 * (c) Codexonics Ltd
 *
 * This package is Open Source Software. For the full copyright and license
 * information, please view the LICENSE file which was distributed with this
 * source code.
 */

use Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemAuthorization;
use Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemInitialization;
use Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemFunctions;

if (! defined('ABSPATH')) {
    exit;
}

/**
 * Prime Mover Download Utilities
 * Helper functionality for Download
 * Export success triggers a javascript redirect with the URL to download the package
 * This class it's processor.
 * It will stream the exported subsite package and then deletes it.
 *
 */
class PrimeMoverDownloadUtilities
{
    private $system_authorization;
    private $system_initialization;
    private $system_functions;

    /**
     * Constructor
     * @param PrimeMoverSystemAuthorization $system_authorization
     * @param PrimeMoverSystemInitialization $system_initialization
     */
    public function __construct(PrimeMoverSystemAuthorization $system_authorization, PrimeMoverSystemInitialization $system_initialization, PrimeMoverSystemFunctions $system_functions)
    {
        $this->system_authorization = $system_authorization;
        $this->system_initialization = $system_initialization;
        $this->system_functions = $system_functions;
    }
    
    /**
     * 
     * Get system functions
     */
    public function getSystemFunctions()
    {
        return $this->system_functions;
    }
    
    /**
     *
     * Get system authorization
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemAuthorization
     * @compatible 5.6
     */
    public function getSystemAuthorization()
    {
        return $this->system_authorization;
    }
 
    /**
     *
     * Get System Initialization
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemInitialization
     * @compatible 5.6
     */
    public function getSystemInitialization()
    {
        return $this->system_initialization;
    }
    
    /**
     * Init hooks
     * @compatible 5.6
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverDownloadUtilities::itAddsInitHooks()
     */
    public function initHooks()
    {
        /**
         * Should be available after composeObjects.
         */
        add_action('init', [ $this, 'streamDownload' ], 2);
    }
    
    /**
     * Multisite migration stream download helper
     * @compatible 5.6
     */
    public function streamDownload()
    {
        $params = $this->getParameters();
        if (empty($params['prime_mover_export_hash']) || empty($params['prime_mover_blogid'])) {
            return;
        }
        
        $hash = $params['prime_mover_export_hash'];
        $blog_id = (int)$params['prime_mover_blogid'];
        
        if ( ! $hash || ! $blog_id ) {
            return;
        }
        do_action('prime_mover_log_processed_events', 'Stream download request received', $blog_id, 'export', 'streamDownload', $this);
        $proceed = false;
        $remote_url_request = false;
        
        if ($this->isShaString($hash, 256) && empty($params['prime_mover_download_nonce'])) {
            $remote_url_request = true;
        }
        if ( ! $remote_url_request && empty($params['prime_mover_download_nonce']) ) {
            return;
        }
        
        $nonce = $params['prime_mover_download_nonce'];
        $action = 'prime_mover_download_package_' . $blog_id;        
        $proceed = $this->validateIfUserIsAuthenticatedByCapability($proceed, $remote_url_request, $nonce, $action, $blog_id);        
        
        $input_server = $this->getSystemInitialization()->getUserInput('server', ['HTTP_X_PRIME_MOVER_DOMAIN' => 'FILTER_VALIDATE_DOMAIN', 'HTTP_X_PRIME_MOVER_AUTHORIZATION' => 'FILTER_SANITIZE_STRING'], 
            'download_authorization_parameters', '', 0, true, true);
        if ( ! $proceed && ! empty($input_server['HTTP_X_PRIME_MOVER_DOMAIN']) && ! empty($input_server['HTTP_X_PRIME_MOVER_AUTHORIZATION']) ) {
            $proceed = apply_filters('prime_mover_stream_download', $proceed, $blog_id, $remote_url_request, $hash, $input_server);
        }        
        if ( ! $proceed ) {
            status_header(401);
            exit;
        }
        if ($remote_url_request && $proceed) {
            do_action('prime_mover_log_processed_events', 'Stream download request authenticated using authorization headers', $blog_id, 'export', 'streamDownload', $this);
        }
        $download_path = get_option($hash);
        if ( ! $download_path ) {
            return;
        }
        $download_path = $this->getSystemInitialization()->getMultisiteExportFolder() . $download_path;
        if (file_exists($download_path)) {
            $this->streamDownloadHelper($blog_id, $download_path, $remote_url_request, $params);
        } else {
            do_action('prime_mover_log_processed_events', 'Stream download not found', $blog_id, 'export', 'streamDownload', $this);
            status_header(404);
        }        
        exit;
    }
    
    /**
     * Validate if user is authenticated by capability
     * @param boolean $proceed
     * @param boolean $remote_url_request
     * @param string $nonce
     * @param string $action
     * @param number $blog_id
     * @return string|boolean
     */
    private function validateIfUserIsAuthenticatedByCapability($proceed = false, $remote_url_request = false, $nonce = '', $action = '', $blog_id = 0)
    {
        if ( ! $remote_url_request && $this->getSystemAuthorization()->isUserAuthorized() && wp_verify_nonce($nonce, $action)) {
            do_action('prime_mover_log_processed_events', 'Stream download request authenticated using capability check and nonce', $blog_id, 'export', 'validateIfUserIsAuthenticatedByCapability', $this);
            $proceed = true;
        }
        if ( ! $proceed && $remote_url_request && is_user_logged_in() && $this->getSystemAuthorization()->isUserAuthorized()) {
            do_action('prime_mover_log_processed_events', 'Stream download request authenticated using capability check.', $blog_id, 'export', 'validateIfUserIsAuthenticatedByCapability', $this);
            $proceed = true;
        }
        return $proceed;
    }
    
    /**
     * Stream download helper
     * @param number $blog_id
     * @param string $download_path
     * @param boolean $remote_url_request
     * @param array $params
     */
    private function streamDownloadHelper($blog_id = 0, $download_path = '', $remote_url_request = false, $params)
    {        
        $generatedFilename = $this->createFriendlyName($blog_id, $download_path);
        header('Content-Type: application/zip');
        header('Content-Disposition: attachment; filename="'. $generatedFilename .'"');
        header('Content-Length: ' . filesize($download_path));
        
        flush();
        if (ob_get_level()) {
            ob_end_clean();
        }
        do_action('prime_mover_log_processed_events', 'Streaming download now', $blog_id, 'export', 'streamDownloadHelper', $this);        
        $this->getSystemFunctions()->readfileChunked($download_path, false);
        if ( ! $remote_url_request ) {
            if (file_exists($download_path)) {
                unlink($download_path);
            }
            delete_option($params['prime_mover_export_hash']);
            do_action('prime_mover_log_processed_events', 'Deleted download path and option', $blog_id, 'export', 'streamDownloadHelper', $this);
        }
    }
    
    /**
     * Check if sha256 string
     * @param string $string
     * @return boolean
     */
    public function isShaString($string = '', $mode = 256) 
    {
        if ( ! $string ) {
            return false;
        }
        $lengths = [256 => 64, 512 => 128];
        $length = $lengths[$mode];
        return (bool) preg_match('/^[0-9a-f]{' . $length . '}$/i', $string);
    }
    
    /**
     * Get parameters
     * @compatible 5.6
     */
    private function getParameters()
    {
        $args = [
            'prime_mover_download_nonce' => FILTER_SANITIZE_STRING,
            'prime_mover_export_hash' => FILTER_SANITIZE_STRING,
            'prime_mover_blogid' => FILTER_SANITIZE_NUMBER_INT,
        ];
        
        $args = apply_filters('prime_mover_filter_stream_download_args', $args );        
        return $this->getSystemInitialization()->getUserInput('get', $args, 'download_stream_parameters', '', 0, true);  
    }
    
    /**
     * @compatible 5.6
     * Generate friendly name of the downloaded zip
     * @param number $blogid_to_export
     * @return void|string
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverDownloadUtilities::itCreatesFriendlyName()
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverDownloadUtilities::itUsesFilenaNameWhenBlogIdNotSet()
     */
    public function createFriendlyName($blogid_to_export = 0, $download_path = '')
    {
        if (! $blogid_to_export) {
            return basename($download_path);
        }
        $blog_name = $this->getSystemFunctions()->getBlogOption($blogid_to_export, 'blogname');
        $sanitized_site_name = mb_strimwidth(sanitize_key($blog_name), 0, 20, '');
        $file_name = $sanitized_site_name . '_' . date('m-d-Y_hia') . '_export_package.zip';
        
        return sanitize_file_name($file_name);
    }
}
