<?php
namespace Codexonics\PrimeMoverFramework\utilities;

use Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemAuthorization;
use Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemFunctions;
use Freemius;

/*
 * This file is part of the Codexonics.PrimeMoverFramework package.
 *
 * (c) Codexonics Ltd
 *
 * This package is Open Source Software. For the full copyright and license
 * information, please view the LICENSE file which was distributed with this
 * source code.
 */

if (! defined('ABSPATH')) {
    exit;
}

/**
 * Prime Mover Freemius Integration
 * Utiliy class for integrating Freemius SDK with Prime Mover
 *
 */
class PrimeMoverFreemiusIntegration
{        
    private $system_authorization;
    private $freemius_options;
    private $system_functions;
    private $shutdown_utilities;
    private $freemius;
    
    const FREEMIUS_USERKEY = '_freemius_usermeta';
    
    /**
     * Constructor
     * @param PrimeMoverSystemAuthorization $system_authorization
     */
    public function __construct(PrimeMoverSystemAuthorization $system_authorization, PrimeMoverSystemFunctions $system_functions, PrimeMoverShutdownUtilities $shutdown_utilities, Freemius $freemius)
    {
        $this->system_authorization = $system_authorization;
        $this->freemius_options = [];
        $this->system_functions = $system_functions;
        $this->shutdown_utilities = $shutdown_utilities;
        $this->freemius = $freemius;
    }
    
    /**
     * Get Freemius
     * @return Freemius
     */
    public function getFreemius()
    {
        return $this->freemius;    
    }
    
    /**
     * Get shutdown utilities
     * @return \Codexonics\PrimeMoverFramework\utilities\PrimeMoverShutdownUtilities
     */
    public function getShutdownUtilities()
    {
        return $this->shutdown_utilities;        
    }
    
    /**
     * Get system functions
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemFunctions
     */
    public function getSystemFunctions()
    {
        return $this->system_functions;
    }
    
    /**
     * Set freemius options
     * @param array $options
     */
    public function setFreemiusOptions($options = [])
    {
        $this->freemius_options = $options;
    }
    
    /**
     * Get freemius options
     * @return array
     */
    public function getFreemiusOptions()
    {
        return $this->freemius_options;
    }
        
    /**
     * Get system authorization
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemAuthorization
     */
    public function getSystemAuthorization()
    {
        return $this->system_authorization;
    }
        
    /**
     * Init hook class
     */
    public function initHooks()
    {      
        add_action('prime_mover_shutdown_actions', [$this, 'restoreFreemiusSettingsOnError']);
        
        add_action('prime_mover_before_db_processing', [$this, 'backupFreemiusOptionsImport'], 10, 2);
        add_action('prime_mover_after_db_processing', [$this, 'restoreFremiusOptionsImportMultisite'], 10, 2);
        add_action('prime_mover_before_only_activated', [$this, 'restoreFreemiusOptions'], 10, 1);
        
        add_filter('prime_mover_multisite_blog_is_licensed', [$this, 'maybeBlogIDLicensed'], 10, 2);
        add_action('prime_mover_run_menus', [$this, 'networkLevelOnlyNoDelegate']); 
        
        add_action('prime_mover_dashboard_content', [$this, 'showGettingStartedOnFreeUsers'], 1); 
        add_action('admin_menu', [$this, 'outputSupportMenu'], 9999999999 );
        add_action('network_admin_menu', [$this, 'outputSupportMenu'], 9999999999 );
        add_action('admin_page_access_denied', [$this, 'redirectToExternalContactPage']);
        
        add_filter('prime_mover_filter_export_footprint', [$this, 'addBothPrimeMoverVersionsToPlugins'], 35, 1);
        add_action('prime_mover_after_db_processing', [$this, 'activateOnlyOneVersion'], 100);
        add_filter('prime_mover_is_loggedin_customer', [$this, 'primeMoverCheckIfLoggedInCustomer'], 10, 1);
        
        add_filter('prime_mover_filter_config_after_diff_check', [$this, 'primeMoverAlwaysExcludeItselfInDiff'], 10, 1);
        add_filter('prime_mover_input_footprint_package_array', [$this, 'addBothPrimeMoverVersionsToPlugins'], 35, 1);
    }
 
    /**
     * Always Exclude Prime Mover Plugin in Diffs
     * @param array $ret
     * @return array
     */
    public function primeMoverAlwaysExcludeItselfInDiff($ret = [])
    {
        if ( ! $this->getSystemAuthorization()->isUserAuthorized()) {
            return $ret;
        }
        if (empty($ret['diff'])) {
            return $ret;
        }
        if (empty($ret['diff']['plugins'])) {
            return $ret;
        }
        if (isset($ret['diff']['plugins'][PRIME_MOVER_DEFAULT_FREE_BASENAME])) {
            unset($ret['diff']['plugins'][PRIME_MOVER_DEFAULT_FREE_BASENAME]);
        }
        if (isset($ret['diff']['plugins'][PRIME_MOVER_DEFAULT_PRO_BASENAME])) {
            unset($ret['diff']['plugins'][PRIME_MOVER_DEFAULT_PRO_BASENAME]);
        }
        return $ret;
    }
    
    /**
     * API - checked if logged-in customer
     * Returns TRUE if logged-in customer, otherwise FALSE
     * @param boolean $ret
     * @return boolean
     */
    public function primeMoverCheckIfLoggedInCustomer($ret = false)
    {
        if ( ! $this->getSystemAuthorization()->isUserAuthorized()) {
            return false;
        }
        if ($this->isCustomer()) {
            return true;
        } else {
            return false;
        }        
    }
    
    /**
     * Activate only one Prime Mover version at 
     * import based on customer information
     * This should work in both single-site and multisite
     */
    public function activateOnlyOneVersion()
    {
        if ( ! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        
        $do_deactivation = false;
        $free_active = false;        
        $pro_active = false;
        
        $free_exist = true;
        $pro_exist = true;
        wp_cache_delete('alloptions', 'options');
        
        if ( ! $this->getSystemFunctions()->getPluginFullPath(PRIME_MOVER_DEFAULT_FREE_BASENAME, true) ) {
            $free_exist = false;
            deactivate_plugins(PRIME_MOVER_DEFAULT_FREE_BASENAME, true);
        }
        if ( ! $this->getSystemFunctions()->getPluginFullPath(PRIME_MOVER_DEFAULT_PRO_BASENAME, true) ) {
            $pro_exist = false;
            deactivate_plugins(PRIME_MOVER_DEFAULT_PRO_BASENAME, true);
        }
        
        if ($free_exist && is_plugin_active(PRIME_MOVER_DEFAULT_FREE_BASENAME)) {
            $free_active = true;
        }
        
        if ($pro_exist && is_plugin_active(PRIME_MOVER_DEFAULT_PRO_BASENAME)) {
            $pro_active = true;
        }
        
        if ($free_active && $pro_active) {
            $do_deactivation = true;
        }

        if ( ! $free_active && ! $pro_active ) {            
            do_action('prime_mover_log_processed_events', 'ERROR: Prime Mover encounters fatal error and deactivated.', 0, 'import', 'activateOnlyOneVersion', $this);
            do_action( 'prime_mover_shutdown_actions', ['type' => 1, 'message' => esc_html__('Prime Mover encounters fatal error and deactivated.')] );
            return wp_die();
        }        
        if ( ! $do_deactivation ) {
            do_action('prime_mover_log_processed_events', 'Only one Prime Mover version is active, this is correct.. skipping deactivation.', 0, 'import', 'activateOnlyOneVersion', $this);
            return;
        }
        if ($this->isCustomer()) {
            do_action('prime_mover_log_processed_events', 'Logged-in customer deactivating free version.', 0, 'import', 'activateOnlyOneVersion', $this);
            deactivate_plugins(PRIME_MOVER_DEFAULT_FREE_BASENAME, true);
        } else {
            do_action('prime_mover_log_processed_events', 'Non-customer logged-in deactivating pro version.', 0, 'import', 'activateOnlyOneVersion', $this);
            deactivate_plugins(PRIME_MOVER_DEFAULT_PRO_BASENAME, true);
        }
    }
    
    /**
     * Add both Prime Mover versions
     * This is to prevent from running into a situation where no Prime Mover
     * is activated after import due to differing versions.
     * @param array $export_system_footprint
     * @return void|string
     */
    public function addBothPrimeMoverVersionsToPlugins($export_system_footprint = [])
    {
        if ( ! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        } 
        if (empty($export_system_footprint['plugins'][PRIME_MOVER_DEFAULT_FREE_BASENAME])) {
            $export_system_footprint['plugins'][PRIME_MOVER_DEFAULT_FREE_BASENAME] = PRIME_MOVER_VERSION;
        }
        if (empty($export_system_footprint['plugins'][PRIME_MOVER_DEFAULT_PRO_BASENAME])) {
            $export_system_footprint['plugins'][PRIME_MOVER_DEFAULT_PRO_BASENAME] = PRIME_MOVER_VERSION;
        }
        return $export_system_footprint;
    }
    
    /**
     * Redirect to external contact page
     */
    public function redirectToExternalContactPage()
    {
        if ( ! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }       
        $args = [
            'page' => FILTER_SANITIZE_STRING
        ];
        
        $contact = $this->getSystemFunctions()->getSystemInitialization()->getUserInput('get', $args, 'verify_contact_page', '', 0, true, false);
        if (empty($contact['page'])) {
            return;
        }
        $contact_page = $contact['page'];
        if ( 'migration-panel-settings-contact' === $contact_page) {
            $this->redirecToSiteContact();
        }
    }
    
    /**
     * Redirect to site contact page
     */
    protected function redirecToSiteContact()
    {
        wp_redirect( CODEXONICS_CONTACT );
        exit;
    }
    
    /**
     * Output support menu
     */
    public function outputSupportMenu() {
        if ( $this->isCustomer()) {
            remove_submenu_page( 'migration-panel-settings', 'migration-panel-settings-wp-support-forum' );
        } else {
            remove_submenu_page( 'migration-panel-settings', 'migration-panel-settings-contact' );
        }        
    }
    
    /**
     * Check if customer is paying through Freemius
     * @return boolean
     */
    private function isCustomer()
    {        
        return $this->getFreemius()->is_paying_or_trial();        
    }
    
    /**
     * Check if Multisite network admin screens
     * @return boolean
     */
    private function isNetworkAdmin()
    {
        return (is_multisite() && is_network_admin());
    }
    
    /**
     * Render dashboard markup
     * @param string $migration_tools
     * @param string $target
     * @param string $plan
     * @param boolean $pro
     * @param string $upgrade_url
     * @param string $support
     * @param string $contact_us
     * @param string $settings
     */
    private function renderDashboardMarkup($migration_tools = '', $target = '', $plan = '', $pro = false, $upgrade_url = '', $support = '', $contact_us = '', $settings = '')
    {
        if ( ! $migration_tools || ! $target || ! $plan || ! $upgrade_url || ! $support || ! $contact_us || ! $settings ) {
            return;
        }
        ?>
        <div class="wrap">
          <div class="card">
          <h2><?php esc_html_e( 'Getting Started', 'prime-mover' ); ?></h2>
               <div class="notice-large highlight">
                    <?php if ($this->isCustomer() && ! defined('PRIME_MOVER_DB_ENCRYPTION_KEY')) : ?>
                          <p><?php printf( esc_html__( 'Thank you for using %s ! This version includes encryption support which you can optionally enable by adding this constant to your wp-config.php :', 
                            'prime-mover' ), "<strong>$plan</strong>" );?></p>
                          <p><code>defined('PRIME_MOVER_DB_ENCRYPTION_KEY', 'YOUR_ENCRYPTION_KEY')</code></p>   
                          <p><strong><?php esc_html_e( 'It is recommended to add the same encryption keys to all of your sites wp-config.php', 'prime-mover' ); ?></strong>.</p>  
                          <p><?php printf( esc_html__( 'To start creating backups / packages, go to %s', 'prime-mover' ), $target );?> : </p>               
                    <?php else : ?>
                          <p><?php printf( esc_html__( 'Thank you for using %s ! Start migrating now by going to %s', 'prime-mover' ), "<strong>$plan</strong>", $target );?> : </p>                     
                    <?php endif;?>

               </div>                                      
                     <p><a href="<?php echo esc_url($migration_tools);?>" class="button button-primary"><?php printf( esc_html__('Go to %s', 'prime-mover'), $target); ?></a></p>                
          </div>   
          
          <?php if ( ! $pro ) : ?>
           <div class="card">                      
                <h2><?php esc_html_e( 'Upgrade to Pro Version', 'prime-mover' ); ?></h2>             
                 <div class="notice-large highlight">
                         <?php if (is_multisite() ) : ?>  
                              <p><?php esc_html_e( 'Migrate so much faster and secure your migration with database / media files encryption. Plus many more useful features you can get with Pro version. 
                             Click the button below to compare FREE and PRO plans.', 'prime-mover' );?></p>        
                         <?php else : ?>                                                  
                              <?php 
                              $free_trial = admin_url('admin.php?page=migration-panel-settings-pricing&trial=true');
                              ?>
                              <p><?php printf(esc_html__( 'Migrate so much faster and secure your migration with database / media files encryption. 
Plus many more useful features you can get with Pro version. %s.
                         Click the button below if you are ready to upgrade ad compare plans.', 'prime-mover' ), '<a href="' . $free_trial . '">' . esc_html__('Start your 14-days FREE trial now', 'prime-mover') . '</a>');?> 
                              </p>                                                    
                         <?php endif; ?>
                 </div>                                      
                     <p><a href="<?php echo esc_url($upgrade_url);?>" class="button button-primary"><?php esc_html_e( 'Upgrade to Pro Version', 'prime-mover' ); ?></a></p>                 
          </div>    
          <?php endif; ?>
  
            <?php if ( $pro ) : ?>
           <div class="card">                      
               <h2><?php esc_html_e( 'Settings', 'prime-mover' ); ?></h2>             
                 <div class="notice-large highlight">
                     <p><?php printf(esc_html__( '%s includes settings page. Please take a moment to review these settings and make sure they are correct.', 'prime-mover'), "<strong>$plan</strong>");?></p> 
                 </div>                                      
                 <p><a href="<?php echo esc_url($settings);?>" class="button button-primary"><?php esc_html_e( 'Go to Settings', 'prime-mover' ); ?></a></p>                 
          </div>    
          <?php endif; ?>
                  
           <div class="card">                      
               <h2><?php esc_html_e( 'Support and Documentation', 'prime-mover' ); ?></h2>             
                   <div class="notice-large highlight">
                         <?php if ( ! $pro ) : ?>
                         <p><?php printf( esc_html__( '%s. You can also read the %s .', 
                             'prime-mover' ), 
                             $support,
                             '<a href="' . CODEXONICS_DOCUMENTATION . '">' . esc_html__('documentation here', 'prime-mover') . '</a>'                             
                             );
                         ?>
                         </p>
                         <p>
                         <?php printf( esc_html__( 'You think Prime Mover is awesome? %s. Thank you!', 'prime-mover' ), 
                             '<strong>' . esc_html__('Please support us by giving a 5-star rating', 'prime-mover') . '</strong>'                             
                             );
                         ?>                         
                         </p>
                         <?php endif; ?> 
                         <?php if ($pro) : ?>
                          <p><?php printf( esc_html__( '%s. You can also read the %s . 
                         Contact us if you like to report bugs, suggest features, ask technical support by clicking the button below.', 
                             'prime-mover' ), 
                             $support,
                              '<a href="' . CODEXONICS_DOCUMENTATION . '">' . esc_html__('documentation here', 'prime-mover') . '</a>');
                         ?>
                         </p>                        
                         <?php endif; ?>
                   </div>  
                   <?php if ( ! $pro ) : ?>                                    
                       <p><a href="https://wordpress.org/plugins/prime-mover/" class="button button-primary"><?php esc_html_e( 'Rate us', 'prime-mover' ); ?></a></p> 
                   <?php endif; ?>  
                   <?php if ( $pro ) : ?>                                    
                       <p><a href="<?php echo esc_url($contact_us);?>" class="button button-primary"><?php esc_html_e( 'Contact us', 'prime-mover' ); ?></a></p> 
                   <?php endif; ?>                  
          </div>   
	   </div>
	   <?php  
    }
    
    /**
     * Show Getting started guide on Free users
     */
    public function showGettingStartedOnFreeUsers()
    {
        $pro = false;
        $plan = "Prime Mover Free version";
        
        $support = sprintf( esc_html__( '%s is available with the free version', 'prime-mover' ), 
            '<a href="https://wordpress.org/support/plugin/prime-mover/">' . esc_html__('Community support', 'prime-mover') . '</a>');
        $settings = "#";
        
        if ($this->isCustomer()) {
            $plan = "Prime Mover Pro version";
            $support = esc_html__( 'Technical support is included with the Pro version', 'prime-mover' );
            
            $pro = true;
            $settings = admin_url( 'admin.php?page=migration-panel-basic-settings');
            if ($this->isNetworkAdmin()) {
                $settings = network_admin_url( 'admin.php?page=migration-panel-basic-settings');
            }
        }
        
        $upgrade_url = admin_url( 'admin.php?page=migration-panel-settings-pricing');
        $migration_tools = admin_url( 'tools.php?page=migration-tools');
        
        $contact_us = admin_url( 'admin.php?page=migration-panel-settings-contact');
        $target = esc_html__( 'Migration Tools', 'prime-mover' );
        
        if (is_multisite() && is_network_admin()) {
            $upgrade_url = network_admin_url( 'admin.php?page=migration-panel-settings-pricing');
            $migration_tools = network_admin_url( 'sites.php');
            $contact_us = network_admin_url( 'admin.php?page=migration-panel-settings-contact');
            $target = esc_html__( 'Network Sites', 'prime-mover' );
        }
        
        $this->renderDashboardMarkup($migration_tools, $target, $plan, $pro, $upgrade_url, $support, $contact_us, $settings);
    }
    
    /**
     * Checks if customer
     * @return boolean
     */
    public function maybeLoggedInUserIsCustomer()
    {
        if ( ! is_user_logged_in() ) {
            return true;
        }        
        return ($this->isCustomer());
    }
    
    /**
     * On multisite network admin interface
     * No need to show the Freemius delegate link
     * Since the plugin is for network administrators only.
     */
    public function networkLevelOnlyNoDelegate()
    {
        if ( ! is_multisite() || ! is_network_admin()) {
            return;
        }
        if ($this->isCustomer()) {
            return;
        }
    ?>
        <script>
        window.onload = function() {
            if (window.jQuery) {                 
            	if (jQuery('#delegate_to_site_admins').length) {
            		jQuery('#delegate_to_site_admins').remove();
            	}
            } 
        }
        </script>
    <?php
    }
    
    /**
     * Restore freemius settings on error
     */
    public function restoreFreemiusSettingsOnError()
    {
        $blog_id = $this->getShutDownUtilities()->primeMoverGetProcessedID();
        $this->restoreFreemiusOptions($blog_id);        
    }
        
    /**
     * Backup freemius settings during import
     * @param array $ret
     * @param number $blogid_to_import
     */
    public function backupFreemiusOptionsImport($ret = [], $blogid_to_import = 0)
    {
        $this->backupFreemiusOptions($blogid_to_import);        
    }
    
    /**
     * Restore freemius settings during import
     * @param array $ret
     * @param number $blogid_to_import
     */
    public function restoreFremiusOptionsImportMultisite($ret = [], $blogid_to_import = 0)
    {
        if ( ! is_multisite()) {
            return;
        }
        $this->restoreFreemiusOptions($blogid_to_import);
    }
    
    /**
     * Checks if multisite blog_id is licensed using API
     * @param boolean $ret
     * @param number $blog_id
     * @return boolean
     */
    public function maybeBlogIDLicensed($ret = false, $blog_id = 0)
    {
        return $this->isBlogLicensed($blog_id);
    }

    /**
     * Is blog ID licensed in multisite
     * @param number $blog_id
     * @return boolean
     */
    protected function isBlogLicensed($blog_id = 0)
    {
        if ( ! is_multisite() && $this->isCustomer()) {
            return true; 
        }
        $freemius = $this->getFreemius();
        if ( ! method_exists($freemius, 'get_install_by_blog_id') ) {
            return false;
        }
        $install = $this->getFreemius()->get_install_by_blog_id($blog_id);       
        if ( ! is_object($install) || ! \FS_Plugin_License::is_valid_id( $install->license_id )) {
            return false;
        }            
        return true;
    }
    
    /**
     * Backup Freemius related options before dB processing
     * @param number $blog_id
     */
    private function backupFreemiusOptions($blog_id = 0)
    {
        if ( ! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        
        $settings_array = [];
        $current_user_id = get_current_user_id();
        $current_options = $this->getAllFreemiusSDKOptions($blog_id);
        if ( ! is_array($current_options) ) {
            return;
        }
        $this->setFreemiusOptions($current_options);
        foreach ($current_options as $option) {
            $this->getSystemFunctions()->switchToBlog($blog_id);
            $settings_array[$option] = get_option($option);
            $this->getSystemFunctions()->restoreCurrentBlog();
        }
        update_user_meta($current_user_id, self::FREEMIUS_USERKEY, $settings_array);
    }

    /**
     * Get all Freemius SDK options
     * @return array|mixed[]
     */
    private function getAllFreemiusSDKOptions($blog_id = 0)
    {
        $this->getSystemFunctions()->switchToBlog($blog_id);
        
        global $wpdb;
        $affected_options = [];
        
        $options_query = "SELECT option_name FROM {$wpdb->prefix}options WHERE option_name LIKE 'fs_%'";        
        $option_query_results = $wpdb->get_results($options_query, ARRAY_N);        
        
        if ( ! is_array($option_query_results) || empty($option_query_results)) {
            $this->getSystemFunctions()->restoreCurrentBlog();
            return [];
        }
        
        foreach ($option_query_results as $v) {
            if ( ! is_array($v)) {
                continue;
            }
            $affected_options[] = reset($v);
        }
        
        $this->getSystemFunctions()->restoreCurrentBlog();        
        return $affected_options;
    }
    
    /**
     * Delete all freemius options
     * @param number $blog_id
     */
    private function deleteAllFreemiusOptions($blog_id = 0)
    {
        if ( ! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        $current_options = $this->getAllFreemiusSDKOptions($blog_id);
        if ( ! is_array($current_options) ) {
            return;
        }        
        foreach ($current_options as $option) {
            $this->getSystemFunctions()->switchToBlog($blog_id);
            delete_option($option);
            $this->getSystemFunctions()->restoreCurrentBlog();
        }
    }
    
    /**
     * Restore Freemius settings after dB processing
     */
    public function restoreFreemiusOptions($blog_id = 0)
    {
        if ( ! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        $current_user_id = get_current_user_id();
        $current_settings = get_user_meta($current_user_id, self::FREEMIUS_USERKEY, true);
        if (empty($current_settings)) {
            return;
        }
        $this->deleteAllFreemiusOptions($blog_id );
        foreach ($current_settings as $option_name => $option_value) {
            $this->getSystemFunctions()->switchToBlog($blog_id);
            update_option($option_name, $option_value);
            $this->getSystemFunctions()->restoreCurrentBlog();
        }        
        delete_user_meta($current_user_id, self::FREEMIUS_USERKEY);
    }
}