<?php
namespace Codexonics\PrimeMoverFramework\utilities;

/*
 * This file is part of the Codexonics.PrimeMoverFramework package.
 *
 * (c) Codexonics Ltd
 *
 * This package is Open Source Software. For the full copyright and license
 * information, please view the LICENSE file which was distributed with this
 * source code.
 */

use Codexonics\PrimeMoverFramework\classes\PrimeMoverImporter;

if (! defined('ABSPATH')) {
    exit;
}

/**
 * Prime Mover Import Utilities
 * Helper functionality for Import.
 *
 */
class PrimeMoverImportUtilities
{
    private $importer;
    private $export_utilities;
    private $lock_utilities;
    
    /**
     * Constructor
     * @param PrimeMoverImporter $importer
     * @param PrimeMoverExportUtilities $export_utilities
     * @param PrimeMoverLockUtilities $lock_utilities
     */
    public function __construct(PrimeMoverImporter $importer, PrimeMoverExportUtilities $export_utilities, PrimeMoverLockUtilities $lock_utilities)
    {
        $this->importer = $importer;
        $this->export_utilities = $export_utilities;
        $this->lock_utilities = $lock_utilities;
    }

    /**
     * Get lock utilities
     * @return \Codexonics\PrimeMoverFramework\utilities\PrimeMoverLockUtilities
     */
    public function getLockUtilities()
    {
        return $this->lock_utilities;
    }
    
    /**
     * Get importer
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverImporter
     * @compatibility 5.6
     */
    public function getImporter()
    {
        return $this->importer;
    }
    
    /**
     * Get export utilities
     * @return \Codexonics\PrimeMoverFramework\utilities\PrimeMoverExportUtilities
     * @compatibility 5.6
     */
    public function getExportUtilities()
    {
        return $this->export_utilities;
    }
    
    /**
     * Gets System authorization
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemAuthorization
     * @compatibility 5.6
     */
    public function getSystemAuthorization()
    {
        return $this->getImporter()->getSystemAuthorization();
    }
    
    /**
     * Init hooks
     * @compatibility 5.6
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverImportUtilities::itAddsInitHooks()
     */
    public function initHooks()
    {
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        add_action('prime_mover_render_import_button', [$this, 'showCoreImportButton'], 10, 1);
        add_action('prime_mover_import_plugins', [ $this, 'primeMoverImportPluginsFunc' ], 10, 2);
        add_action('prime_mover_import_themes', [ $this, 'primeMoverImportThemesFunc' ], 10, 2);
        
        add_action('prime_mover_do_after_import_button', [ $this, 'importWarnMismatchSite'], 3, 1);
        add_action('prime_mover_do_after_import_button', [ $this, 'importWrongFileType'], 4, 1);
        add_action('prime_mover_do_after_import_button', [ $this, 'importWarning'], 5, 1);
        add_action('prime_mover_do_after_import_button', [ $this, 'importDiffDialog'], 10, 1);
        
        add_action('prime_mover_do_after_import_button', [ $this, 'importCancelSuccess'], 15, 1);
        add_action('prime_mover_do_after_import_button', [ $this, 'importCancelFail'], 20, 1);
        add_action('prime_mover_do_after_import_button', [ $this, 'genericFailNotice'], 25, 1);
        
        add_filter('prime_mover_filter_config_after_diff_check', [ $this, 'verifyPluginsThemesIncluded'], 15, 1);
        add_filter('prime_mover_filter_config_after_diff_check', [ $this, 'filterDiff'], 10, 1);    
        
        add_action('prime_mover_after_actual_import', [ $this, 'markedUpdateTrackerCompleted'], PHP_INT_MAX );
        add_filter('prime_mover_ajax_rendered_js_object', [$this, 'setRestoreModeDialogTexts'], 100, 1 );
        add_filter('prime_mover_filter_upload_phrase', [$this, 'singleSiteUploadInfoCompat'], 10, 3);
        
        add_filter('prime_mover_ajax_rendered_js_object', [$this, 'setImportProcessError'], 110, 1 );
    }    

    /**
     * Setup ongoing import process error
     * @param array $args
     * @return array
     */
    public function setImportProcessError( array $args )
    {       
        $args['prime_mover_importprocess_error_message'] = esc_js(
            "<p>" . sprintf(__('Import process fails for site ID : {{BLOGID}}. Retry is attempted but still fails after %s seconds.', 'prime-mover'), '<strong>{{RETRYSECONDS}}</strong>') . "</p>" .
            "<p>" . '<strong>' . __('Server Error : {{PROGRESSSERVERERROR}}', 'prime-mover') . '</strong>' . "</p>" .
            "<p>" . __('Error occurs while processing', 'prime-mover') . ' ' . "<strong>{{IMPORTMETHODWITHERROR}}</strong>" . ' ' . __('method.', 'prime-mover') . "</p>" .
            "<p><strong>" . sprintf(__('Can you try increasing the web server timeout beyond %s', 'prime-mover'), '<strong>{{FIXEDSECONDS}}</strong>') . ' ' . __('seconds', 'prime-mover') ."?</strong></p>" .
            "<p>" . __('This might help resolve this issue when restoring/importing large sites.', 'prime-mover') . "</p>"
            );        
        
        $args['prime_mover_unknown_import_process_error'] = esc_js(__('unknown', 'prime-mover'));
        
        return $args;
    }
    
    /**
     * Single site alternative upload URL replace
     * @param array $upload_phrase
     * @param array $ret
     * @param array $replaceables
     * @return array
     */
    public function singleSiteUploadInfoCompat($upload_phrase = [], $ret = [], $replaceables = [])
    {
        if (is_multisite()) {            
            return $upload_phrase;
        }
        if ( ! empty($ret['imported_package_footprint']['legacy_upload_information_url']) ) {            
            return $upload_phrase;
        }
        if (empty($upload_phrase['wpupload_url']['search']) || empty($upload_phrase['wpupload_url']['replace'])) {
            return $upload_phrase;
        }
        if (empty($ret['origin_site_url'])) {
            return $upload_phrase;
        }
        $origin_site_url = $ret['origin_site_url'];
        $current_upload_url = $upload_phrase['wpupload_url']['search'];
        $target_upload_url = $upload_phrase['wpupload_url']['replace'];
        
        if (false !== strpos($current_upload_url, $origin_site_url)) {            
            return $upload_phrase;
        }
        
        $origin_domain = parse_url($current_upload_url, PHP_URL_HOST);
        $alternative_upload_url = str_replace($origin_domain, $origin_site_url, $current_upload_url);

        $upload_phrase['wpupload_url_alternative']['search'] =  $alternative_upload_url;
        $upload_phrase['wpupload_url_alternative']['replace'] = $target_upload_url;        
        
        return $upload_phrase;
    }
    
    /**
     * Set restore mode dialog texts
     * @param array $args
     * @return array
     */
    public function setRestoreModeDialogTexts( array $args ) 
    {
        
        $args['prime_mover_upload_package_mode'] = esc_js(__('Upload package', 'prime-mover'));
        $args['prime_mover_restorewithinserver_mode'] = esc_js(__('Restore within server backup', 'prime-mover'));
        $args['prime_mover_encrypted_note'] = apply_filters('prime_mover_encryption_note_text', '');
        
        $args['prime_mover_restore_remote_url_mode'] = esc_js(__('Restore from a remote URL package', 'prime-mover'));
        $args['prime_mover_restore_encrypted_package_warning'] = esc_js(__('Yes', 'prime-mover'));
        $args['prime_mover_restore_noencryption_warning'] = esc_js(__('No', 'prime-mover'));
        
        return $args;
    }
    
    /**
     * Show core import button
     * @param number $blog_id
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverImportUtilities::itShowsCoreImportButton()
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverImportUtilities::itLoadsMobileClassOnCoreImportWhenMobile()
     */
    public function showCoreImportButton($blog_id = 0) 
    {
        if ( ! $blog_id ) {
            return;
        }
        $mobile_class = '';
        $button_text = sprintf( esc_html__('Restore blog ID: %d', 'prime-mover'), $blog_id );
        if (wp_is_mobile()) {
            $mobile_class = 'prime_mover_is_mobile';
            $button_text = esc_html__('Restore', 'prime-mover');
        }
        if ( ! is_multisite() ) {
            $button_text = esc_html__('Import', 'prime-mover');
        }
    ?>
       <label for="js-prime_mover_importing_blog_<?php echo esc_attr($blog_id) ; ?>" id="js-prime-mover-browseupload-label-<?php echo esc_attr($blog_id) ; ?>" class="button-primary prime-mover-fileupload-label">
           <?php echo $button_text; ?>
       </label>
	    	<input name ="prime_mover_importbrowsefile" 
	    			type ="file"
	    			class = "prime_mover_importbrowsefile js-prime_mover_importbrowsefile <?php echo esc_attr($mobile_class);?>"
	    			accept ="<?php echo esc_attr(implode(",", $this->getImporter()->getSystemInitialization()->getZipMimeTypes())); ?>"
	    			id ="js-prime_mover_importing_blog_<?php echo esc_attr($blog_id) ; ?>"
	    			data-multisiteblogid = "<?php echo esc_attr($blog_id); ?>" />    
    <?php       
    }

    /**
     * Officially mark update tracker completed
     * @compatibility 5.6
     */
    public function markedUpdateTrackerCompleted() 
    {        
        $this->getImporter()->getProgressHandlers()->updateTrackerProgress(esc_html__('Import done.', 'prime-mover'));  
        sleep(3);
    }
    
    /**
     * Filter diff if its empty
     * @param array $ret
     * @compatibility 5.6
     */
    public function filterDiff(array $ret)
    {
        if (! isset($ret['diff'])) {
            return $ret;
        }
        $is_empty_diff = array_filter($ret['diff']);
        if (empty($is_empty_diff)) {
            unset($ret['diff']);
        }
        return $ret;
    }
    
    /**
     * Verify if themes and plugins are included in import package
     * @param array $import_data
     * @return array
     * @compatibility 5.6
     */
    public function verifyPluginsThemesIncluded(array $import_data)
    {
        $include_themes = $this->checkIfPackageIncludeThemes($import_data);
        $include_plugins = $this->checkIfPackageIncludePlugins($import_data);
        
        if ($include_themes && $include_plugins && isset($import_data['diff'])) {
            //Importing plugins/themesRemove diff
            unset($import_data['diff']);
        }
        
        return $import_data;
    }
    
    /**
     *
     * @param int $blog_id
     * @compatibility 5.6
     */
    public function importCancelFail($blog_id = 0)
    {
        if ( ! $blog_id ) {
            return;
        }
        ?>
        <div style="display:none;" id="js-prime-mover-cancel-import-diff-fail-<?php echo esc_attr($blog_id); ?>" title="<?php esc_attr_e('Import Cancelled!', 'prime-mover')?>"> 
			<p><?php esc_html_e('You cancel the import. Nothing is changed on the server.', 'prime-mover'); ?></p>  	  	
        </div>
    <?php
    }

    /**
     *
     * @param int $blog_id
     * @compatibility 5.6
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverImportUtilities::itDisplaysFailedUploadNotice() 
     */
    public function genericFailNotice($blog_id = 0)
    {
        if ( ! $blog_id ) {
            return;   
        }
        ?>
        <div style="display:none;" id="js-prime-mover-import-generic-fail-<?php echo esc_attr($blog_id); ?>" title="<?php esc_attr_e('Error!', 'prime-mover')?>"> 
			<p></p>  	  	
        </div>
    <?php
    }
    
    /**
     *
     * @param int $blog_id
     * @compatibility 5.6
     */
    public function importCancelSuccess($blog_id = 0)
    {
        if ( ! $blog_id ) {
            return;   
        }
        ?>
        <div style="display:none;" id="js-prime-mover-cancel-import-diff-<?php echo esc_attr($blog_id); ?>" title="<?php esc_attr_e('Import Cancelled!', 'prime-mover')?>"> 
			<p><?php esc_html_e('You cancel the import and temp file deleted. Nothing is changed on the server.', 'prime-mover')  ?></p>  	  	
        </div>
    <?php
    }

    /**
     * Warn user of a mismatch import
     * @param int $blog_id
     * @compatibility 5.6
     */
    public function importWarnMismatchSite($blog_id = 0)
    {
        if ( ! $blog_id ) {
            return;   
        }
        ?>
        <div style="display:none;" id="js-prime-mover-wrong-importedsite-<?php echo esc_attr($blog_id); ?>" title="<?php esc_attr_e('Error!', 'prime-mover')?>"> 
			<p><?php echo $this->getImporter()->getSystemInitialization()->returnCommonWrongTargetSiteError(); ?></p>  	  	
        </div>
    <?php
    }
    
    /**
     * Shows wrong import filetype
     * @param int $blog_id
     * @compatibility 5.6
     */
    public function importWrongFileType($blog_id = 0)
    {
        if ( ! $blog_id ) {
            return;   
        }
        ?>
        <div style="display:none;" id="js-prime-mover-wrong-filetype-<?php echo esc_attr($blog_id); ?>" title="<?php esc_attr_e('Error!', 'prime-mover')?>"> 
			<p><?php esc_html_e('Invalid file type or corrupted zip file.', 'prime-mover'); ?></p>  	  	
        </div>
    <?php
    }
    
    /**
     * Shows import warning to the user
     * @param int $blog_id
     * @compatibility 5.6
     */
    public function importWarning($blog_id = 0)
    {
        if ( ! $blog_id ) {
            return;   
        }
        ?>
        <div style="display:none;" id="js-prime-mover-import-warning-confirm-<?php echo esc_attr($blog_id); ?>" title="<?php esc_attr_e('Heads Up!', 'prime-mover')?>"> 
          <p class="js-prime-mover-upload-option-selected prime-mover-upload-option-selected">
          <?php echo esc_html__('It will take around ', 'prime-mover') . ' ' . "<span id='js-prime-mover-computed-uploadtime'></span>" . ' ' . 
          esc_html__('to upload this package based on your upload speed.', 'prime-mover'); ?></p>
    	  <p><?php esc_html_e('Import process deletes old site and will be replaced with the imported package.', 'prime-mover') ?></p>
    	  <p><?php esc_html_e('Please verify that the following restoration information is correct:', 'prime-mover') ?></p>
    	  
    	  <ul>
    	      <li class="restoration-mode-prime-mover-li" id="js-restoration-mode-prime-mover"><strong><?php esc_html_e('Restoration mode', 'prime-mover');?></strong>: <span class="js-prime-mover-warning-restoration-mode"></span></li> 
    	      <li class="restoration-source-prime-mover-li" id="js-restoration-source-prime-mover"><strong><?php esc_html_e('Source', 'prime-mover');?></strong>: <span class="js-prime-mover-warning-source-of-origin"></span></li>
    	      <li class="restoration-package-size-prime-mover-li" id="js-restoration-package-size-prime-mover"><strong><?php esc_html_e('Package size', 'prime-mover');?></strong>: <span class="js-prime-mover-package-size-dialog"></span></li> 
    	      <?php if (is_multisite() ) { ?>   	      
    	      <li class="restoration-blog-id-prime-mover" id="js-restoration-blog-id-prime-mover"><strong><?php esc_html_e('Target blog ID', 'prime-mover');?></strong>: <span class="js-prime-mover-warning-target-blog-id"></span></li>
    	      <?php } ?>
    	      <li class="restoration-site-title-prime-mover-li" id="js-restoration-site-title-prime-mover"><strong><?php esc_html_e('Site title', 'prime-mover');?></strong>: <span class="js-prime-mover-warning-target-site-title"></span></li>
    	      <li class="restoration-encrypted-database-prime-mover-li" id="js-restoration-encrypted-database-prime-mover"><strong><?php esc_html_e('Encrypted database', 'prime-mover');?></strong>: <span class="js-prime-mover-warning-restoring-encrypted"></span></li>
    	      <li class="restoration-encrypted-mediafiles-prime-mover-li" id="js-restoration-encrypted-mediafiles-prime-mover"><strong><?php esc_html_e('Encrypted media files', 'prime-mover');?></strong>: <span class="js-prime-mover-encrypted-media"></span></li>
    	      <li class="restoration-description-prime-mover-li" id="js-restoration-description-prime-mover"><strong><?php esc_html_e('Description', 'prime-mover');?></strong>: <span class="js-prime-mover-warning-scope-mode"></span></li>
    	  </ul>    	  
    	  <p><span class="js-prime-mover-encrypted-package-note prime-mover-encrypted-package-note"></span> <?php esc_html_e('To be safe, make sure you have backups ready.', 'prime-mover'); ?> <strong><?php  esc_html_e('Are you sure you want to do this?', 'prime-mover'); ?></strong></p>   	  	
        </div>
    <?php
    }
    
    /**
     * Markup for import diff dialog
     * @param int $blog_id
     * @compatibility 5.6
     */
    public function importDiffDialog($blog_id = 0)
    {
        if ( ! $blog_id ) {
            return;   
        }
        ?>
        <div style="display:none;" id="js-prime-mover-import-diff-confirm-<?php echo esc_attr($blog_id); ?>" title="<?php esc_attr_e('Warning', 'prime-mover')?>!"></div>
    <?php
    }
    
    /**
     * Import themes based on the package
     * @compatibility 5.6
     * @param array $import_data
     * @param number $blog_id
     */
    public function primeMoverImportThemesFunc($import_data = [], $blog_id = 0)
    {
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        
        $include_themes = $this->checkIfPackageIncludeThemes($import_data);
        if (! $include_themes ||
             empty($import_data['imported_package_footprint']['stylesheet']) ||
             empty($import_data['imported_package_footprint']['template']) ||
             empty($import_data['imported_package_footprint']['using_child_theme'])) {
            return;
        }
        
        $using_child_theme = $import_data['imported_package_footprint']['using_child_theme'];
        $template = key($import_data['imported_package_footprint']['template']);
        
        $template_path_package = $include_themes . $template . DIRECTORY_SEPARATOR;
        global $wp_filesystem;
        if (! $wp_filesystem->exists($template_path_package)) {
            return;
        }
        $template_path_wpcontent = $this->getExportUtilities()->getThemeFullPath($template, false);
        $this->getImporter()->getProgressHandlers()->updateTrackerProgress(esc_html__('Importing themes..', 'prime-mover'));
        
        //Parent theme first
        $this->handleThemeImport($template_path_wpcontent, $template_path_package, $blog_id);
        if ('no' === $using_child_theme) {
            return;
        }
        
        //Child theme
        $child_theme_name = key($import_data['imported_package_footprint']['stylesheet']);
        $child_template_path_package = $include_themes . $child_theme_name . DIRECTORY_SEPARATOR;
        if (! $wp_filesystem->exists($child_template_path_package)) {
            return;
        }
        $child_theme_path_wpcontent = $this->getExportUtilities()->getThemeFullPath($child_theme_name, false);
        $this->handleThemeImport($child_theme_path_wpcontent, $child_template_path_package, $blog_id);
    }
    
    /**
     * Handle new theme import
     * @param string $template_path_wpcontent
     * @param string $template_path_package
     * @compatibility 5.6
     * @param number $blog_id
     */
    private function handleThemeImport($template_path_wpcontent = '', $template_path_package = '', $blog_id = 0)
    {
        global $wp_filesystem;
        
        //Theme hash check, skip this process if target and source are basically the same..
        $source_hash = $this->getImporter()->getSystemFunctions()->hashEntity($template_path_package);
        $target_hash = $this->getImporter()->getSystemFunctions()->hashEntity($template_path_wpcontent);
        if ($source_hash === $target_hash) {
            do_action('prime_mover_log_processed_events', "THEME HASHES MATCHED, NO NEED TO COPY: $template_path_package TO: $template_path_wpcontent", $blog_id, 'import', 'handleThemeImport', $this);
            return;
        }
        
        $this->getImporter()->getSystemFunctions()->enableMaintenanceDuringImport();        
        $this->getImporter()->getSystemFunctions()->primeMoverDoDelete($template_path_wpcontent);
        
        //Re-create blank theme directory
        $theme_make_directory_result = $wp_filesystem->mkdir($template_path_wpcontent);
        if (false === $theme_make_directory_result) {
            return;
        }
        
        //Copy theme files from import package to themes dir
        do_action('prime_mover_log_processed_events', "COPYING THEME: $template_path_package TO: $template_path_wpcontent", $blog_id, 'import', 'handleThemeImport', $this);
        $copy_directory_result	= $this->getSystemCheckUtilities()->copyDir($template_path_package, $template_path_wpcontent);
        if (is_wp_error($copy_directory_result)) {
            return;
        }
        $this->getImporter()->getSystemFunctions()->disableMaintenanceDuringImport();
    }
    
    /**
     * Get system check utilities
     * @return \Codexonics\PrimeMoverFramework\utilities\PrimeMoverSystemCheckUtilities
     */
    public function getSystemCheckUtilities()
    {
        return $this->getImporter()->getSystemChecks()->getSystemCheckUtilities();        
    }
    
    /**
     * Checks if we need to restore plugins
     * @param array $import_data
     * @return boolean
     */
    public function maybeImportPlugins($import_data = [])
    {
        $include_plugins = $this->checkIfPackageIncludePlugins($import_data);
        if (! $include_plugins || ! isset($import_data['imported_package_footprint']['plugins'])) {
            return false;
        }
        $plugins = $import_data['imported_package_footprint']['plugins'];
        if (empty($plugins) || ! is_array($plugins)) {
            return false;
        }
        return ['plugins' => $plugins, 'include_plugins' => $include_plugins];
    }
    
    /**
     * Import plugins based on the package
     * @compatibility 5.6
     * @param array $import_data
     * @param number $blog_id
     */
    public function primeMoverImportPluginsFunc($import_data = [], $blog_id = 0)
    {                
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        if (false === $this->maybeImportPlugins($import_data)) {
            return;
        }
        
        $plugin_details = $this->maybeImportPlugins($import_data);
        $plugins = $plugin_details['plugins'];
        $include_plugins = $plugin_details['include_plugins'];
        
        $lock_file = $this->getImporter()->getSystemInitialization()->getProcessingPluginPath();        
        global $wp_filesystem;

        $fp = $this->getLockUtilities()->openLockFile($lock_file, false);
        if ( ! $fp ) {
            do_action('prime_mover_log_processed_events', "ERROR: Failed to open plugins copy lock file.", $blog_id, 'import', 'primeMoverImportPluginsFunc', $this);
            return;
        }
        
        if ($this->getLockUtilities()->createProcessLockFile($fp)) {
            do_action('prime_mover_log_processed_events', "Obtained plugin lock file. Starting copying plugins..", $blog_id, 'import', 'primeMoverImportPluginsFunc', $this);
            $this->getImporter()->getProgressHandlers()->updateTrackerProgress(esc_html__('Importing plugins..', 'prime-mover'));
            foreach ($plugins as $plugin => $version) {
                //Formulate plugin path at plugins dir
                $plugin_path_target = $this->getExportUtilities()->getPluginFullPath($plugin, false);
                
                //Formulate plugin path at imported package dir
                $ds = DIRECTORY_SEPARATOR;
                if ($this->getExportUtilities()->isPluginFile($plugin_path_target)) {
                    $ds = '';
                }
                $plugin_dirname = basename($plugin_path_target);
                if ($ds) {
                    $plugin_dirname = dirname($plugin) . $ds;
                }
                $plugins_imported_package_path = $include_plugins . $plugin_dirname;
                
                if (! $wp_filesystem->exists($plugins_imported_package_path)) {
                    continue;
                }
                
                //Hash comparison
                $source_hash = $this->getImporter()->getSystemFunctions()->hashEntity($plugins_imported_package_path);
                $target_hash = $this->getImporter()->getSystemFunctions()->hashEntity($plugin_path_target);
                if ($source_hash === $target_hash) {
                    do_action('prime_mover_log_processed_events', "PLUGIN HASHES MATCHED, NO NEED TO COPY: $plugins_imported_package_path TO: $plugin_path_target", $blog_id, 'import', 'primeMoverImportPluginsFunc', $this);
                    continue;
                }
                
                //Plugin exist in package, delete old plugin at plugins directory
                $this->getImporter()->getSystemFunctions()->enableMaintenanceDuringImport();
                $this->getImporter()->getSystemFunctions()->primeMoverDoDelete($plugin_path_target);
                
                //Re-create blank plugin directory
                if ($ds) {
                    $plugin_make_directory_result = $wp_filesystem->mkdir($plugin_path_target);
                    if (false === $plugin_make_directory_result) {
                        continue;
                    }
                }
                
                //Copy plugin files from import package to plugins dir
                $copy_directory_result = '';
                
                if ($ds) {
                    $copy_directory_result	= $this->getSystemCheckUtilities()->copyDir($plugins_imported_package_path, $plugin_path_target);
                } else {
                    $copy_directory_result	= $wp_filesystem->copy($plugins_imported_package_path, $plugin_path_target, true);
                }
                if (is_wp_error($copy_directory_result)) {
                    continue;
                }
                
                do_action('prime_mover_log_processed_events', "COPYING: $plugins_imported_package_path TO: $plugin_path_target", $blog_id, 'import', 'primeMoverImportPluginsFunc', $this);
                do_action('prime_mover_log_processed_events', "Copy directory result:", $blog_id, 'import', 'primeMoverImportPluginsFunc', $this);
                do_action('prime_mover_log_processed_events', $copy_directory_result, $blog_id, 'import', 'primeMoverImportPluginsFunc', $this);
                
                $this->getImporter()->getSystemFunctions()->disableMaintenanceDuringImport();
            }
            
            $unlock = $this->getLockUtilities()->unLockFile($fp);
            if ($unlock) {
                do_action('prime_mover_log_processed_events', "Plugin processing completed, lock successfully released.", $blog_id, 'import', 'primeMoverImportPluginsFunc', $this);
            }            
        } else {
            do_action('prime_mover_log_processed_events', "ERROR: Failed to obtain Dropbox lock file.", $blog_id, 'import', 'primeMoverImportPluginsFunc', $this);            
            return;
        }       
        $this->getLockUtilities()->closeLock($fp);
    }
  
    /**
     * Checks if package includes themes to import
     * @param array $import_data
     * @return string
     * @compatibility 5.6
     */
    private function checkIfPackageIncludeThemes($import_data = [])
    {
        $ret = '';
        global $wp_filesystem;
        $themes_package_path = $this->getThemespackagepath($import_data);
        if ($wp_filesystem->exists($themes_package_path)) {
            $ret = $themes_package_path;
        }
        return $ret;
    }
    
    /**
     * Checks if package includes plugins to import
     * @param array $import_data
     * @return string
     * @compatibility 5.6
     */
    private function checkIfPackageIncludePlugins($import_data = [])
    {
        $ret = '';
        global $wp_filesystem;
        $plugins_package_path = $this->getPluginspackagepath($import_data);
        if ($wp_filesystem->exists($plugins_package_path)) {
            $ret = $plugins_package_path;
        }
        return $ret;
    }
 
    /**
     * Get plugins path
     * @param array $import_data
     * @return string
     * @compatibility 5.6
     */
    private function getPluginspackagepath($import_data = [])
    {
        $ret = '';
        if (empty($import_data['unzipped_directory'])) {
            return $ret;
        }
        $unzipped_directory = $import_data['unzipped_directory'];
        $plugins_foldername = $this->getExportUtilities()->getPluginFoldername();
        $plugins_package_path = $unzipped_directory . $plugins_foldername . DIRECTORY_SEPARATOR;
        
        return $plugins_package_path;
    }
    
    /**
     * Get themes path
     * @param array $import_data
     * @return string
     * @compatibility 5.6
     */
    private function getThemespackagepath($import_data = [])
    {
        $ret = '';
        if (empty($import_data['unzipped_directory'])) {
            return $ret;
        }
        $unzipped_directory = $import_data['unzipped_directory'];
        $themes_foldername = $this->getExportUtilities()->getThemeFoldername();
        $themes_package_path = $unzipped_directory . $themes_foldername . DIRECTORY_SEPARATOR;
        
        return $themes_package_path;
    }
    
    /**
     * Analyze zip package description
     * @param string $tmp_path
     * @param number $blog_id
     * @param boolean $encrypted
     * @param \ZipArchive $zip
     * @param string $package_type
     * @param boolean $blog_id_check
     * @param boolean $array_type
     * @return string[]|string[]|mixed[]
     */
    private function analyzeZipPackageDescription($tmp_path = '', $blog_id = 0, $encrypted = false, \ZipArchive $zip, $package_type = '', $blog_id_check = true, $array_type = false)
    {
        $dir = trim($zip->getNameIndex(0), '/');
        $plugin_dir = $dir . "/plugins/";
        $package_type_data = [];
        
        $blog_id = (int)$blog_id;
        $blog_id_package = $this->getRealBlogIDFromZipPackage($zip, $blog_id);
        $blog_id_package = (int)$blog_id_package;
        
        if ($blog_id_check && $blog_id_package !== $blog_id) {
            $package_type_data[$blog_id_package] = '';
            return $package_type_data;
        }
        $blog_id = $blog_id_package;
        $media_dir = $dir . "/media.zip";       
        
        $database_file = $blog_id . '.sql';
        if ($encrypted) {
            $database_file = $database_file . '.enc';
        }
        
        $plugins_exist = $zip->locateName($plugin_dir);
        $media_exist = $zip->locateName($media_dir);
        $database_exist = $zip->locateName($database_file, \ZIPARCHIVE::FL_NODIR);
        
        $footprint_exist = $zip->locateName('footprint.json', \ZIPARCHIVE::FL_NODIR);
        $blogidtext_exist = $zip->locateName('blogid.txt', \ZIPARCHIVE::FL_NODIR);
        
        if (false !== $plugins_exist) {
            if (false !== $media_exist) {
                $package_type_slug = 'complete_export_mode';
            } else {
                $package_type_slug = 'development_package';
            }            
        } elseif (false !== $media_exist) {
            $package_type_slug = 'db_and_media_export';
        } elseif (false !== $database_exist) {
            $package_type_slug = 'dbonly_export';
        }
        
        if (false === $database_exist || false === $footprint_exist || false === $blogidtext_exist) {
            $package_type_text = '';
        }
        
        $export_modes = $this->getExportUtilities()->getExportModes();        
        if ( ! empty($export_modes[$package_type_slug]) ) {
            $package_type_text = $export_modes[$package_type_slug];
        }       
        if ($array_type) {
            $package_type = [];
            $package_type[$package_type_slug] = $package_type_text;
        } else {
            $package_type = $package_type_text;
        }
        $package_type_data[$blog_id] = $package_type;
        return $package_type_data;        
    }
    
    /**
     * Get real blog ID from zip package
     * @param \ZipArchive $zip
     * @param number $passed_blog_id
     * @return number
     */
    public function getRealBlogIDFromZipPackage(\ZipArchive $zip, $passed_blog_id = 0)
    {
        $blogidtext_exist = $zip->locateName('blogid.txt', \ZIPARCHIVE::FL_NODIR);
        if ( ! $blogidtext_exist) {
            return 0;    
        }
        $read_blog_id = $zip->getFromIndex($blogidtext_exist);
        $read_blog_id = (int)$read_blog_id;
        
        if ($read_blog_id) {
            $passed_blog_id = $read_blog_id;
        }
        
        return $passed_blog_id;
    }
    
    /**
     * Get zip package description
     * @param string $tmp_path
     * @param number $blog_id
     * @param boolean $encrypted
     * @param boolean $delete_if_invalid
     * @param boolean $blog_id_check
     * @param boolean $array_type
     * @return boolean|mixed
     */
    public function getZipPackageDescription($tmp_path = '', $blog_id = 0, $encrypted = false, $delete_if_invalid = true, $blog_id_check = true, $array_type = false)
    {
        if ( ! $tmp_path ) {
            return false;
        }
        $zip = new \ZipArchive();
        $package_type = '';
        $blog_id = (int)$blog_id;
        $result = [];
        $package_type_data = [];
        $package_type_data[$blog_id] = $package_type;       
        
        if (true === $zip->open($tmp_path)) {            
            $package_type_data = $this->analyzeZipPackageDescription($tmp_path, $blog_id, $encrypted, $zip, $package_type, $blog_id_check, $array_type);
        }          
        if ($array_type && ! empty($package_type_data[$blog_id])) {
            $result = $package_type_data[$blog_id];
            $package_type = reset($result);
        } else {
            $package_type = reset($package_type_data); 
        }
        if ( ! $package_type && $delete_if_invalid) {
            $this->getImporter()->getSystemFunctions()->primeMoverDoDelete($tmp_path);
        }
        if ($array_type && is_array($result) && ! empty($result)) {
            return $result;
        } else {
            return $package_type; 
        }        
    }
    
    /**
     * Checks if zip package dB is encrypted
     * @param string $tmp_path
     * @return boolean
     */
    public function isZipPackageDbEncrypted($tmp_path = '')
    {
        return $this->getImporter()->getSystemFunctions()->isZipPackageHasEntityEncrypted($tmp_path, 'signature.enc');
    }

    /**
     * Checks if zip package media files is encrypted
     * @param string $tmp_path
     * @return boolean
     */
    public function isZipPackageMediaEncrypted($tmp_path = '')
    {
        return $this->getImporter()->getSystemFunctions()->isZipPackageHasEntityEncrypted($tmp_path, 'media.enc');
    }
    
    /**
     * Get real blog ID from zip package
     * @param \ZipArchive $zip
     * @return number
     */
    public function getSiteTitleFromZipPackage($tmp_path = '')
    {
        if ( ! $tmp_path ) {
            return '';
        }
        $za = new \ZipArchive();
        $zip = $za->open($tmp_path);
        if (true !== $zip) {
            return '';
        }        
        $footprint = $za->locateName('footprint.json', \ZIPARCHIVE::FL_NODIR);
        if ( ! $footprint) {
            return '';
        }
        $footprint_json = $za->getFromIndex($footprint);
        $system_footprint_package_array	= json_decode($footprint_json, true);
        if ( ! empty($system_footprint_package_array['site_title'] ) ) {
            return $system_footprint_package_array['site_title'];
        }
        return '';
    }
}
