<?php
namespace Codexonics\PrimeMoverFramework\utilities;

/*
 * This file is part of the Codexonics.PrimeMoverFramework package.
 *
 * (c) Codexonics Ltd
 *
 * This package is Open Source Software. For the full copyright and license
 * information, please view the LICENSE file which was distributed with this
 * source code.
 */

use Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemFunctions;
use Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemInitialization;
use Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemAuthorization;
use WP_Error;

if (! defined('ABSPATH')) {
    exit;
}

/**
 * Prime Mover System Check Utilities
 * Helper functionality for System check class
 *
 */
class PrimeMoverSystemCheckUtilities
{
    private $system_authorization;
    private $system_initialization;
    private $system_functions;
    
    /**
     * Constructor
     * @param PrimeMoverSystemAuthorization $system_authorization
     * @param PrimeMoverSystemInitialization $system_initialization
     * @param PrimeMoverSystemFunctions $system_functions
     */
    public function __construct(
        PrimeMoverSystemAuthorization $system_authorization,
        PrimeMoverSystemInitialization $system_initialization,
        PrimeMoverSystemFunctions $system_functions
        ) 
    {
            $this->system_authorization = $system_authorization;
            $this->system_initialization = $system_initialization;
            $this->system_functions = $system_functions;
    }
    
    /**
     * Get system functions
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemFunctions
     * @compatible 5.6
     */
    public function getSystemFunctions()
    {
        return $this->system_functions;
    }
    
    /**
     * Get System Initialization
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemInitialization
     * @compatible 5.6
     */
    public function getSystemInitialization()
    {
        return $this->system_initialization;
    }
    
    /**
     * Get System authorization
     * @return \Codexonics\PrimeMoverFramework\classes\PrimeMoverSystemAuthorization
     * @compatible 5.6
     */
    public function getSystemAuthorization()
    {
        return $this->system_authorization;
    }
    
    /**
     * Initialize hooks
     * @compatible 5.6
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverSystemCheckUtilities::itAddsInitHoooks()
     */
    public function initHooks() 
    {
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        add_filter('prime_mover_footprint_keys', [ $this, 'addLegacyBaseURLtoFootPrint' ], 25, 2);        
        add_filter('prime_mover_validate_site_footprint_data', [ $this, 'validateLegacySiteIfBaseURLSet' ], 25, 2);
        add_filter('prime_mover_filter_replaceables', [ $this, 'replaceLegacyBaseUrls' ], 75, 2);   
        
        add_filter('prime_mover_filter_site_footprint', [ $this, 'addVersionToFootPrint' ], 25, 2);
        add_filter('prime_mover_footprint_keys', [ $this, 'addVersionKeysToFootPrint' ], 30, 2); 
        add_filter('prime_mover_validate_site_footprint_data', [ $this, 'validateVersionIfSet' ], 35, 2);
        
        add_action('prime_mover_after_actual_import', [$this, 'maybeDeactivateSomePlugins'], 10, 2);
    }
    
    /**
     * Maybe deactivate some plugins based on exported footprint config
     * @param array $ret
     * @param number $blogid_to_import
     * @return void
     */
    public function maybeDeactivateSomePlugins($ret = [], $blogid_to_import = 0)
    {
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        
        if (isset($ret['error']) || isset($ret['diff'])) {            
            return;
        }        
        
        if (! isset($ret['imported_package_footprint']['plugins'])) {
            return;
        }
        $source_site_active_plugins = $ret['imported_package_footprint']['plugins'];
        if (empty($source_site_active_plugins) || ! is_array($source_site_active_plugins)) {
            return;
        }  
        $source_activated = array_keys($source_site_active_plugins);
        $this->getSystemFunctions()->switchToBlog($blogid_to_import);
        
        wp_cache_delete('alloptions', 'options');
        $active_plugins = $this->getSystemFunctions()->getBlogOption($blogid_to_import, 'active_plugins');
       
        if ( ! is_array($active_plugins)) {
            return $ret;
        }

        foreach ($active_plugins as $k => $active_plugin) {           
            if ( ! in_array($active_plugin, $source_activated )) {
                deactivate_plugins($active_plugin, true, false);
            }
        }        
        $this->getSystemFunctions()->restoreCurrentBlog();
    }
  
    /**
     *Validate if version set in footprint is valid
     * @param bool $overall_valid
     * @param array $footprint_temp
     * @return boolean
     * @compatible 5.6
     */
    public function validateVersionIfSet($overall_valid = true, array $footprint_temp) 
    {
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return $overall_valid;
        }
        if (! isset($footprint_temp['prime_mover_plugin_version'])) {
            $overall_valid = false;
        }        
        return $overall_valid;
    }
    
    /**
     * Add version to footprint
     * @param array $footprint
     * @param int $blogid_to_export
     * @return array
     * @compatible 5.6
     */
    public function addVersionToFootPrint(array $footprint, $blogid_to_export = 0) 
    {
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            $footprint;
        }
        $footprint['prime_mover_plugin_version'] = PRIME_MOVER_VERSION;        
        
        return $footprint;        
    }

    /**
     * Added version keys to footprint
     * @param array $footprint_keys
     * @param array $footprint
     * @return array
     * @compatible 5.6
     */
    public function addVersionKeysToFootPrint(array $footprint_keys, array $footprint)
    {
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return $footprint_keys;
        }
        if ( ! isset( $footprint['prime_mover_plugin_version'] ) ) {
            return $footprint_keys;
        }
        if (in_array( 'prime_mover_plugin_version', $footprint_keys )) {
            return $footprint_keys;
        }
        
        $footprint_keys[] = 'prime_mover_plugin_version';
        return $footprint_keys;
    }
    
    /**
     * Add in replaceables the legacy base URLS whenever applicable
     * @param array $replaceables
     * @param array $ret
     * @compatible 5.6
     */
    public function replaceLegacyBaseUrls( array $replaceables, array $ret) 
    {
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return $replaceables;
        }
        if ( empty( $ret['imported_package_footprint']['legacy_upload_information_url'] ) ) {
            return $replaceables;
        }
        if ( empty( $replaceables['wpupload_url']['replace'] ) ) {
            return $replaceables;
        }
        if ( ! empty( $replaceables['legacybase_url'] ) ) {
            return $replaceables;
        }
        
        $reference_index = array_search("wpupload_url",array_keys($replaceables));
        
        $offset = $reference_index + 1;
        $replaceables = array_slice($replaceables, 0, $offset, true) +
        array('legacybase_url' => array( 'search' => $ret['imported_package_footprint']['legacy_upload_information_url'], 'replace' => $replaceables['wpupload_url']['replace'] ) ) +
        array_slice($replaceables, $offset, NULL, true);
        
        return $replaceables;        
    }
    
     /**
     * Analyze if the site being exported uses legacy upload base URLs
     * @param int $blog_id
     * @return boolean
     * @compatible 5.6
     */
    public function isLegacyMultisiteBaseURL( $blog_id = 0) 
    {
        $legacy = false;
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return $legacy;
        }
        if ( ! $blog_id ) {
            return $legacy;
        }
        
        $this->getSystemFunctions()->switchToBlog($blog_id);
        if ( $this->getSystemFunctions()->getSiteOption( 'ms_files_rewriting' ) && defined( 'UPLOADS' ) ) {
            $legacy = true;
            $this->getSystemInitialization()->setLegacyMultisite($legacy);
        }  
        
        $this->getSystemFunctions()->restoreCurrentBlog();
        return $legacy;        
    }
    
    /**
     * Return legacy base URLs for old multisites
     * @param int $blog_id
     * @return string
     * @compatible 5.6
     */
    public function getLegacyBaseURL($blog_id = 0) 
    {
        $ret = '';
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return $ret;
        }
        if ( ! $blog_id ) {
            return $ret;
        }
        
        $this->getSystemFunctions()->switchToBlog($blog_id);
        $siteurl = get_option( 'siteurl' );
        if ( ! $siteurl ) {
            return $ret;
        }
        
        $this->getSystemFunctions()->restoreCurrentBlog();
        return trailingslashit( $siteurl ) . 'files';        
    }
    
    /**
     * Adds Legacy base URl to footprint config if site is legacy
     * @param array $footprint_keys
     * @param array $footprint
     * @return array
     * @compatible 5.6
     */
    public function addLegacyBaseURLtoFootPrint(array $footprint_keys, array $footprint) 
    {
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return $footprint_keys;
        }
        if ( ! $this->getSystemInitialization()->getLegacyMultisite()) {
            return $footprint_keys;
        }
        if ( ! isset( $footprint['legacy_upload_information_url'] ) ) {
            return $footprint_keys;
        }
        if (in_array( 'legacy_upload_information_url', $footprint_keys )) {
            return $footprint_keys;
        }
        
        $footprint_keys[] = 'legacy_upload_information_url';
        return $footprint_keys;
    }
    
    /**
     * 
     * @param bool $overall_valid
     * @param array $footprint_temp
     * @return boolean
     * @compatible 5.6
     */
    public function validateLegacySiteIfBaseURLSet($overall_valid = true, array $footprint_temp) 
    {
        if (! $this->getSystemAuthorization()->isUserAuthorized()) {
            return $overall_valid;
        }
        if ( ! $this->getSystemInitialization()->getLegacyMultisite()) {
            return $overall_valid;
        }
        if (! isset($footprint_temp['legacy_upload_information_url'])) {
            $overall_valid = false;
        }
        
        return $overall_valid;        
    }
    
    /**
     * Return MySQL executable based on base directory location
     * @param string $command
     * @return void|string
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverSystemCheckUtilities::itGetsMySQLBaseDirExecutablePath() 
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverSystemCheckUtilities::itReturnsNullWhenThereIsNoResult()
     * @tested PrimeMoverFramework\Tests\TestPrimeMoverSystemCheckUtilities::itReturnsEmptyWhenExecutableDoesNotExist()
     */
    public function getMySQLBaseDirExecutablePath($command = 'mysql', $is_windows = false)
    {
        global $wp_filesystem, $wpdb;
        $executable = '';
        $result = $wpdb->get_results("SHOW VARIABLES WHERE Variable_name = 'basedir'", ARRAY_N);
        if ( ! is_array($result) ) {
            return;
        }
        if (empty($result[0])) {
            return;
        }
        $data = $result[0];
        if ( ! is_array($data)) {
            return;
        }
        if ( empty($data[1]) ) {
            return;
        }
        $basedir = $data[1];
        $basedir = rtrim($basedir, DIRECTORY_SEPARATOR);
        if ($is_windows) {
            $command = $command . '.exe';
        }
        $path = $basedir . DIRECTORY_SEPARATOR . 'bin' . DIRECTORY_SEPARATOR . $command;
        if ($wp_filesystem->exists($path)) {
            $executable = $path;
        }
        return $executable;
    }
    
    /**
     * Cross platform compatible copy dir function
     * Derived as improvement from WordPress core version:
     * Source: https://developer.wordpress.org/reference/functions/copy_dir/
     * Bug: https://core.trac.wordpress.org/ticket/46581
     * @param string $from
     * @param string $to
     * @param array $skip_list
     * @param array $excluded_files
     * @return \WP_Error|boolean|\WP_Error|boolean
     */
    public function copyDir( $from = '', $to = '', $skip_list = [], $excluded_filetypes = []) 
    {
        global $wp_filesystem;
        
        $dirlist = $wp_filesystem->dirlist( $from );
        
        $from = rtrim($from, DIRECTORY_SEPARATOR) . DIRECTORY_SEPARATOR;
        $to   = rtrim($to, DIRECTORY_SEPARATOR) . DIRECTORY_SEPARATOR;
        
        $escape = DIRECTORY_SEPARATOR;
        if ('\\' === DIRECTORY_SEPARATOR) {
            $escape = '\\\\';
        }
        foreach ( (array) $dirlist as $filename => $fileinfo ) {
            $filename = (string)$filename;
            if ( in_array( $filename, $skip_list, true ) ) {
                continue;
            }            
            if ( 'f' == $fileinfo['type'] ) {
                $ext = strtolower(pathinfo($filename, PATHINFO_EXTENSION));
                if (in_array($ext, $excluded_filetypes, true)) {
                    continue;
                }                              
                if ( ! $wp_filesystem->copy( $from . $filename, $to . $filename, true, FS_CHMOD_FILE ) ) {                    
                    $wp_filesystem->chmod( $to . $filename, FS_CHMOD_FILE );
                    if ( ! $wp_filesystem->copy( $from . $filename, $to . $filename, true, FS_CHMOD_FILE ) ) {
                        return new WP_Error( 'copy_failed_copy_dir', __( 'Could not copy file.' ), $to . $filename );
                    }
                }
            } elseif ( 'd' == $fileinfo['type'] ) {
                if ( ! $wp_filesystem->is_dir( $to . $filename ) ) {
                    if ( ! $wp_filesystem->mkdir( $to . $filename, FS_CHMOD_DIR ) ) {
                        return new WP_Error( 'mkdir_failed_copy_dir', __( 'Could not create directory.' ), $to . $filename );
                    }
                }
                $sub_skip_list = array();
                foreach ( $skip_list as $skip_item ) {
                    if ( 0 === strpos( $skip_item, $filename . DIRECTORY_SEPARATOR ) ) {
                        $sub_skip_list[] = preg_replace( '!^' . preg_quote( $filename, '!' ) . $escape . '!i', '', $skip_item );
                    }
                }                
                $result = $this->copyDir( $from . $filename, $to . $filename, $sub_skip_list, $excluded_filetypes);
                if ( is_wp_error( $result ) ) {
                    return $result;
                }
            }
        }
        return true;
    }
}
